// Made by A.O. on 4.11.1997
// Converts root file with MCTracks and MCHits data containers in the events
// to .AOU (but not .APX) compatible ascii file.
// Ignore the "Error in <TObjArray::At>: index 16 out of bounds" errors if present.
//main()
{

//-----------------------------------------------+
// CopyRootToAscii.C                             |
// Read the ROOT file with Phobos events         |
// and convert it to PMC Ascii compatible file.  |
// ----------------------------------------------+

// Reset the ROOT environment
// do not reset unless you gave up defining input_name from root command line
    gROOT.Reset();

    Char_t *input_name = "pmc100ev89.hijet.aou.root";


    Int_t   myindex; 
    Int_t   maxPlanes = 146; 

    Int_t num=0; 
    Int_t numtrk=0;
    Int_t i; 
    Int_t fVerbose=0;

// Declare pointers to a TPhEvent and TPhTrack object
    TPhEvent *event;
    TPhMCTrack *track;
    TPhHit     *hit;

    Char_t FileName[256];
    const Char_t *phathome = gSystem->Getenv("PHATHOME");
    printf("Found PHAT home at %s\n",phathome);

    sprintf(FileName,"%s/data/%s",phathome,input_name);
//  Get detector object, assuming it is present in the data file.
//  If not, get it from some other ROOT file.
    TFile *f = new TFile("../init/test/Detector.root");
    TPhDetector *detector = f.Get("TPhDetector");
    f->Close();

//  Create a TPhDST object for accessing the ROOT data file
    TPhDST *dst = new TPhDST();
    dst->Open(FileName);

// Open the Ascii output file.
    sprintf(FileName,"%s/data/%s.ascii",phathome,input_name);

    FILE *out;
    out = fopen(FileName,"w");

// Write a very simplified header
   fprintf(out," * ASCII file converted from %s file\n",input_name);
   for (i = 0; i < maxPlanes; i++) {
      fprintf(out," G DEVO %s %d\n",detector->GetSensor(i)->GetChLabel(),i+1);
   }
   fprintf(out," EOH \n");

   TObjArray        *CList;
   TPhObjectContainer *DContainer;

   while(event = dst->GetNextEvent())
   {
       event->ls();

       DContainer = (TPhObjectContainer*) event->GetObject("MCTracks");
       CList = DContainer->GetObjectList();
       if( CList ) {
          printf("\nMCTracks data container: %d tracks in event\n", CList->GetEntries());

          TIter TrackIter(CList);
          while(track = (TPhMCTrack*) TrackIter()) {
            numtrk++;
            fprintf(out," P  WWWW  %5d  0  %5d  %5d  %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f\n",
                    numtrk,track->GetNHits(),track->GetGPid(),
                    track->GetPx(),track->GetPy(),track->GetPz(),
                    track->GetXProd(),track->GetYProd(),track->GetZProd());
            if (fVerbose) track->Print();
// Loop over the hits on the track
// right know we can get only Hits and not MCHits
            track->ResetHitCounter();
            while(hit = track->GetNextHit())
	    {
                fprintf(out,"  %d %8.3f %8.3f %8.3f %8.3f \n",
                        hit->GetSensorLabel(),
                        hit->GetX(),hit->GetY(),hit->GetZ(),hit->GetLength());
                if (fVerbose) printf("  %d %8.3f %8.3f %8.3f %8.3f \n",
                              hit->GetSensorLabel(),
                              hit->GetX(),hit->GetY(),hit->GetZ(),hit->GetLength());
            }
          }
       }
// Don't forget to free the memory before going to next event
       delete event;
       fprintf(out," EOE\n");
   }
}

