// GufldSwimRoot.C
// Made by A.O. on 4.12.1997
// macro used to propagate particles through the PHOBOS spectrometer up to the TOF.
// takes detector object, magnet object and input file with events with track info.
// produces a new ROOT TFile with hits in detector planes.
main()
{
// GufldSwimEvents.C
// Clean the ROOT environment 

    gROOT->Reset();
    gBenchmark->Start("swim");

// Define a couple of pointers to be used later

    TPhEvent *event;
    TPhHit   *hit;  
    TPhTrack *track;

//  Define input file name, output will have .gufldswum automatically added 
    Char_t input_name[50];
    strcpy(input_name,"pmc100ev89.hijet.aou.root");

    Char_t pathname[256];
    Char_t FileName[256];

// For some of the later stuff we will need to know the 
// location of the $PHATHOME directory. Use the TSystem 
// class (thanks, Fons) for getting this variable:

   const Char_t *phathome = gSystem->Getenv("PHATHOME");
   printf("Found PHAT home at %s\n",phathome);

//  Get detector object
    TPhDetector *detector=NULL;
    detector = TPhDetector::GetInstance();
    if(detector) printf("TPhDetector ver.%f found in ROOT\n",detector->GetVersion());
    // There is no ASCII file available for the present version of the PHOBOS geometry
    /*
    if( !detector ) {
        sprintf(pathname,"%s/init/test", phathome);
        detector = new TPhDetector();
        detector->ReadASCIIGeometry(pathname);
        if(detector) printf("TPhDetector ver.%f read from %s\n", detector->GetVersion(), pathname);
    }
    */
//  A detector saved in a ROOT file can be used
//  Use phathome to find the data file

    sprintf(FileName,"%s/init/test/%s",phathome,"Detector.root");
    if( !detector ) {
       TFile *f = new TFile(FileName);
       detector = (TPhDetector *) f.Get("TPhDetector");
       f->Close();
       if(detector) printf("TPhDetector ver.%f found in %s\n", detector->GetVersion(), FileName);
    }
//  if detector not found, try to look in the input data file
//  Open the input file with the events. 
//  Again, use phathome to find the data file

    sprintf(FileName,"%s/data/%s",phathome,input_name);
    if( !detector ) {
       TFile *f = new TFile(FileName);
       detector = (TPhDetector *) f.Get("TPhDetector");
       f->Close();
       if(detector) printf("TPhDetector ver.%f found in %s\n", detector->GetVersion(), FileName);
    }
    if( !detector ) {
       printf("Can't find/create a TPhDetector object\n");;
       exit(0);
    }
    
// Create a 1D histogram where we'll store the distribution
// of the number of hits per track

    nhits = new TH1F("nhits","No of hits per track",20,0,20); 
    hch = new TH1F("charge","Charge of the track",5,-2.5,2.5); 

    pty_in = new TH2F("phiy_in","phi vs rapidity, input",30,-6,6,36,0,360);
    pty_acc = new TH2F("phiy_acc","phi vs rapidity, accepted",30,-6,6,36,0,360);

// Also create a canvas into which the histogram will be 
// drawn

    c = new TCanvas("c","Phi vs rapidity",800,10,800,800);
    pyin = new TPad("phiyin","Phi-y input",0.02,0.02,0.49,0.98);
    pyacc = new TPad("phiyacc","Phi-y accepted",0.51,0.02,0.98,0.98);
    pyin.Draw();
    pyacc.Draw();

    c1 = new TCanvas("c1","Charge and Number of hits",800,210,800,800);
    ch = new TPad("charge","Charge",0.02,0.02,0.49,0.98);
    nhit = new TPad("nhit","Number of hits",0.51,0.02,0.98,0.98);
    ch.Draw();
    nhit.Draw();

// Now create a TPhDST object to access the data file at the 
// path given by FileName

    TPhDST *dst = new TPhDST(FileName);

// Open the output file for the events
// This has to point to a directory where you
// have write permission

    sprintf(FileName,"%s/data/%s.gufldswum",phathome,input_name);

    out = new TFile(FileName,"RECREATE","swum");

// First, TPhSwim for swimming tracks through the 
// setup

    TPhSwim     *swim = new TPhSwim();

// TPhGufldMagnet and TPhDetector are needed as interfaces to 
// the magnetic field and the geometry

    Char_t filename[100];
    Long_t type; 
    Double_t bfldmax; 
    Double_t zmagx; 
    Double_t zmagz; 
    Double_t zmoff[3]; 
    Double_t zrfld; 
    Double_t version;

    
    sprintf(filename,"%s/init/magnet/phomagv108.txt",phathome);
    type = 2; 
    bfldmax = 0; 
    zmagx = 0; 
    zmagz = 0; 
    zmoff[0] = 0; 
    zmoff[1] = 0; 
    zmoff[2] = 0; 
    zrfld = 0; 

    TPhGufldMagnet  *magnet = new TPhGufldMagnet(filename, 
       &type, &bfldmax, &zmagx, &zmagz, &zmoff, &zrfld, &version);;
    

    /*
    // Instead of TPhGufldMagnet one can use TPhToscaMagnet
    sprintf(filename,"%s/init/magnet/PhobosMagnet.root",phathome);
    fmag = new TFile(filename);
    TPhToscaMagnet *magnet = (TPhToscaMagnet *) fmag->Get("magnet");
    */

    magnet->ls();
    swim->SetMagnet(magnet);
    swim->SetDetector(detector);

// Reset the event counter
    Int_t num = 0;

// Now read events from the file until we either hit the 
// end-of-file or have read 100 events, whichever comes 
// first

    while(event = dst->GetNextEvent())
    {

        printf("Now at event %d %d\n",event->GetNRun(),event->GetNEvent());

// Here we use the TPhSwim object to propagate each track
// in the event through the PHOBOS setup (actually the 
// spectrometer and the TOF) for the present implementation
// of TPhSwim. 

	// swim->SwimEvent() returns TRUE is swimming finished succesfully
        if( !swim->SwimEvent(event) ) {
           printf("Event not propagated properly!\n");
           delete event;
           continue;
        }

// Write the event into the file opened before
        out->cd();
        event->Write();

// Find SwimTracks container
// Use the GetNextTrack() iterator from TPhEvent to 
// loop over all tracks in the event. For those with at 
// least one hit in the spectrometer, fill the number of 
// hits into a histogram that was defined above.

    TPhObjectContainer *TrackBuffer;
    TrackBuffer = (TPhObjectContainer *) event->GetObject("SwimTracks");
    if(!TrackBuffer)
    {
        fprintf(stderr,"Error in TPhSwim::SwimEvent(TPhEvent *event): \n");
        fprintf(stderr,"Can't find TPhObjectContainer 'SwimTracks'.\n");
        exit(0);
    }

    TIter NextTrack(TrackBuffer->GetObjectList());
    Float_t phi;
    while(track = (TPhTrack *) NextTrack())
        {
            phi = fmod(track->GetPsi()*180.0/3.14 + 270.0,360.0);
            if(track->GetSlopeXZ() < 0) phi = fmod(360.0 - phi,360.0);
            pty_in.Fill(track->GetRap(0.139),phi);
            if(track->GetNHits() > 0)
            {
                pty_acc.Fill(track->GetRap(0.139),phi);
                nhits.Fill(track->GetNHits());
                hch.Fill(track->GetCharge());
            }
        }
// Draw the histogram

        Int_t return;
        ch.cd();
        hch.Draw("e1l");
        nhit.cd();
        nhits.Draw("e1l");
        c1.Update();
        c.cd();
        pyin.cd();
        pty_in.Draw("box");
        pyacc.cd();
        pty_acc.Draw("box");
        c.Update();

// Release the memory for the event before reading the next one

        delete event;
        num++;
    }
    gBenchmark->Show("swim");
//  Close files
    out->Close();
}
