Char_t* checkmul()
{
// checkmul.C: basic script to benchmark the multiplicity package
// Use: with $PHATHOME and phat correctly defined, execute
//    phat $PHATHOME/phatmul/macros/checkmul.C
// Returns 0 if the result is acceptable, or a string describing the 
// error if not; prints one line, and creates a postscript file.
// Notes:
// (1) The caller is responsible for deleting the returned string if it
// is non-0; otherwise the resulting small memory leak would become
// serious only if the macro failed repeatedly during a single Phat
// session.  To avoid this problem compeletely, execute
//    phat 
//    .L $PHATHOME/phatmul/macros/checkmul.C
//    char *err
//    err = checkmul()  // Any error message will be printed
//    if (err) delete []err
// (2) Minuit, called by the vertex finder, writes the misleading message
// **********
// **    1 **SET ERR           1 
// **********
// This seems to have been fixed in recent versions of ROOT.
// 03-Feb-99: Change to new test file with MDC2 names (RV)
// $Id: checkmul.C,v 1.2 1999/02/10 17:04:35 decowski Exp $
///////////////////////////////////////////////////////////////////////

// 30-Jan-99: Expand filenames for existence check, use new TPhMulDisp (RV)
// 21-Jan-99: Move TPhVertex delete into loop where TPhEvent still exists (RV)
// 18-Jan-99: Use TPhMulDisp for display (RV), change to return string (mpd)
// 17-Jan-99: Creation from runmul.C (Robin Verdier)

//  Char_t *dataFile = "$PHATHOME/data/hijing_2cen.aou.root"; // until 1-Mar-99
  Char_t *dataFile = "$PHATHOME/data/hijing_2cen.adat.root";
  TPhDetector *det;
  TPhDST *dst;
  Char_t *errBuf = new Char_t[200];
  Char_t *errMessage = 0;
  TPhEvent *ev;
  TPhMul mp;

// Try to avoid later segment violations by checking that datafile exists.
// Unlike ROOT, fopen can't expand environmentals in names.
  Char_t *expName = gSystem->ExpandPathName(dataFile);
  FILE *dataExists = fopen (expName, "r");
  if (!dataExists) 
  { sprintf (errBuf, "Can't open file %s [%s]; use ^C to abort.",
      dataFile, expName);
    errMessage = errBuf;
  }
  else
    dataExists->fclose();
  expName->Delete();
    
  if (!errMessage)

  { Int_t dataFormat = 0; // -1 = none, 0 = .root, 1 = .apx, 2 = user's .root

// Process MC truth information if present

    Bool_t useMCHits = kTRUE;

// Set up vertex finding

    TPhVertex *v1;
    Bool_t findVertex = kTRUE;

// Select array object names.  Using the wrong set is the reason for
// the most common Mul crash.

//    Int_t nameSet = 1; // pre 30-Dec-98 data
    Int_t nameSet = 0;

// Set TPhMul switch for processing MC (truth) information if present
// ( 0 = data, 1 = MC data, 2 = MC hits, 3 = diagnostic)

    if (useMCHits) mp.fDataSource = 2;

// Specify TPhEvent names of TPhHitArray and TPhVertex objects.  As
// they are often changed capriciously, we must specify them at a highly
// visible level.

    Char_t *HitArraysName = "PMCHitArraysFromHits";
    if (nameSet == 0)
    { mp.fMCHitsName   = "PMCHits";
      mp.fMCTracksName = "PMCTracks";
    }
    else
    { HitArraysName    = "HitArraysFromHits";
      mp.fMCHitsName   = "MCHits";
      mp.fMCTracksName = "MCTracks";
    }
    mp.fHitArrayName[0] = HitArraysName;
    Char_t *vertexName = (findVertex)? "MainVertex" : "";

    dst = new TPhDST(dataFile);

// Create the detector object.

    det = dst->GetDetector();
    if (!det) 
      errMessage = "No TPhDetector object is available";
  } // if (!errMessage)

  if (!errMessage)

  { Int_t nEvents = 100; // The benchmark file actually has 2

// Loop over events

    for (Int_t iev = 0; iev < nEvents; iev++)
    { // Delete previous event if it exists
      if (ev) delete ev;

// Get an event from the DST

      ev = dst->GetNextEvent();
      if (ev)
      { det->SetEvent(ev);

// Fill the HitArrays with hits

        if (dataFormat != 1) det->FillWithEventHitArrays(HitArraysName);
// Issue a warning if the hitarrays have an unknown name
        TPhHashContainer *hitarrays 
          = (TPhHashContainer *) ev->GetObject(HitArraysName);
        Int_t nHA = (hitarrays)? hitarrays->GetEntries() : 0;
        if (!nHA)
        { sprintf (errBuf, "Event contains %d %s",
            nHA, HitArraysName);
          errMessage = errBuf;
        }
      } // if (ev

      else

      { if (!iev) errMessage = "No events found in DST.";
        break;
      }

      if (errMessage) break;

// Find the vertex

      if (strcmp(vertexName, ""))
      { 
// Following should no longer be necessary with Jan/99 ROOT
//        if (!iev)
//           printf ("You can ignore the following Minuit error message:\n");
        ev->FindVertex();
        v1 = (TPhVertex *) ev->GetObject(vertexName);
        if (v1)
          mp.SetVertex(v1);
        else
        { sprintf (errBuf, "Found no vertex \"%s\".", vertexName);
          errMessage = errBuf;
        }
      } // if (strcmp

      if (errMessage) break;

// Reconstruct the multiplicity

      mp.Process(ev);

// Delete the vertex object

      if (v1) delete v1;

    } // for (Int_t iev

  } // if (!errMessage

  if (!errMessage)

// Get the reconstructed and generated #s of charged primaries

  { Float_t nCh = mp.GetDisplay("*")->GetIntegral(13,14);
    Float_t nChGen = mp.GetDisplay("*")->GetIntegral(23,24);
    Float_t ratio = (nChGen)? nCh / nChGen : -999.;

    Bool_t accept = ratio >= 0.9 && ratio <= 1.1;
    if (accept)  
      printf ("Reconstructed / Generated nCh =%7.1f /%7.1f =%7.4f, OK.\n",
	  nCh, nChGen, ratio); 
    else
    { printf ("Reconstructed / Generated nCh =%7.1f /%7.1f =%7.4f"
  	  ", unacceptable.\n", nCh, nChGen, ratio);
      errMessage = "Unacceptable reconstruction.";
    }
// Create ps file.  A gif file can't be created in batch mode, but you
// can use the commented-out form below to create it interactively.

    if (accept)
    { Char_t *canvasName = "c1";
//    TCanvas *cc = new TCanvas (canvasName, "Multiplicity", 5, 5, 505, 505);
// The following works but has unlabelled axes
//    TGraphErrors *d1 = mp.GetGraph("d1","*+in",13,14,360.);
//    d1->Draw("ape");
      TPhMulDisp *d2 = new TPhMulDisp (mp, "*+in", -1, 13, 14, 360, 0, 0,
        "Eta", "d2Nch/dEta/dPhi", "Reconstructed");
//      d2->Draw(0, canvasName);
      d2->Draw();
//      TPhMulDisp *d3 = new TPhMulDisp (mp, "*+in", -1, 23, 24, 360, 0, 0,
//        "Eta", "d2Nch/dEta/dPhi", "Generated");
//      d3->fMColor = d2->fMColor + 2;
//      d3->fMStyle = d2->fMStyle + 1;
//      d3->Draw(1, canvasName);
      d2->fMColor = 4;
      d2->Draw(23, 24, 360, "Generated");
// Create postscript file from the current graphics pad
      gPad->Print("checkmul.ps", "ps");
// Use the following to create a gif file if in interactive mode
//    gPad->Print("checkmul.gif", "gif");
    }
  }

  if (ev) delete ev;
  if (dst) delete dst;
  if (det) TPhDetector::DeleteInstance();
//  if (d2) delete d2;
//  if (cc) delete cc;

  return errMessage;
}
