// RawDataMaker. Macro to make data containing RawHitBuffers out of files
// that have previously been converted using something like ConvApxToRoot.C
// 
//Begin_Html <A HREF="mailto:decowski@mit.edu">decowski@mit.edu</A>
//End_Html
//
// $Id: RawDataMaker.C,v 1.5 1998/07/01 18:20:47 decowski Exp $
{

  gROOT->Reset();

  TFile f("$PHATHOME/init/SensorDB.root");
  TPhSensorMapDB* SensDB = (TPhSensorMapDB*) f.Get("SensDB");

  TPhDetector *det = TPhDetector::ReadLatestGeometry();

  // this is file will be created by ConvApxToRoot.C
  //  TPhDST data("$PHATHOME/data/cv0_00000e1.apx.root");
  TPhDST data("$PHATHOME/data/cv0_00000e1.apx.root");
  TPhEvent *event;

  Int_t i, j;
  Int_t FEC, String, ChPos, BondPat;
  Char_t SensorName[10], FECName[10];

  TPhAbsDataContainer *cont;
  TPhHitArray *HitArray;
  TPhRawHitFECn0 *RawBuf;
  FECRawHit_t Hit;

  // This is the file that we will write into
  TFile fout("out.root", "recreate");

  // We keep track of all the spectrometer sensors that weren't found
  // in this hashtable (for debugging purposes).
  THashTable *NotFoundHitArrays = new THashTable;

  event = data.GetNextEvent();
  //while(event = data.GetNextEvent()) {

    // strip the event of the hits and tracks if they exist, we are only
    // interested in having the HitArrays and RawHitBuffers in the event.
    TPhObjectContainer *mchits = 
      (TPhObjectContainer *) event->GetObject("MCHits");
    if(mchits) {
      event->RemoveObject(mchits);
      mchits->GetObjectList()->Clear();
      delete mchits;
    }
    // strip tracks
    TPhObjectContainer *mctracks = 
      (TPhObjectContainer *) event->GetObject("MCTracks");
    if(mctracks) {
      event->RemoveObject(mctracks);
      delete mctracks;  
    }

    // get container with the HitArrays and make a new container to 
    // hold the rawdata
    cont = (TPhAbsDataContainer*) event->GetObject("MCHitArrays");
    TPhHashContainer *RawBufCont = new TPhHashContainer("MCRawData");

    TIter NextHitArray(cont->GetIterator());

    // iterate over all HitArrays and go over all rows and columns, 
    // converting the ADC values for each of them
    while(HitArray = (TPhHitArray*) NextHitArray()) {
      for(i = 0; i < HitArray->GetRows(); i++) {
	for(j = 0; j < HitArray->GetColumns(); j++) {
	  // TPhSensorMapDB::Lookup returns true if this is a valid entry
	  if(SensDB->Lookup(HitArray->GetName(), i, j, 
			    &FEC, &String, &ChPos)) {
	    // some sanity checks 
	    if(ChPos > 768)
	      printf("Error: %s %d %d %d %d %d\n", 
		     HitArray->GetName(), i, j, FEC, String, ChPos);
	    // find a RawHitBuf with the given FEC number. if not found, 
	    // create a new RawHitBuf
	    RawBuf = (TPhRawHitFECn0*) 
	      RawBufCont->GetObject(TPhRawHitFECn0::MakeFECName(FEC, FECName));
	    if(RawBuf == NULL) {
	      printf("Making FEC# %d\n", FEC);
	      if(FEC < 0) 
		printf("Error: %s %d %d %d %d %d\n", 
		       HitArray->GetName(), i, j, FEC, String, ChPos);
	      RawBuf = new TPhRawHitFECn0(FEC);
	      RawBufCont->Add((TPhDataObject*)RawBuf);
	    }
	    Hit.ADC = HitArray->GetADC(i, j);
	    Hit.Channel = ChPos;
	    RawBuf->Add(String, &Hit);
	  }
	  else {
	    // the sensor did not give a valid lookup entry:
	    // check if the sensor is a spectrometer (only spectrometers 
	    // are currently in the database)
	    if(det->GetSubDetector("SpectN")->IsThere(HitArray->GetName()) ||
	       det->GetSubDetector("SpectP")->IsThere(HitArray->GetName())) {
	      if(NotFoundHitArrays->FindObject(HitArray->GetName()) == NULL) {
		printf("%s: (%d, %d) is not valid! Ret: %d %d %d\n",
		       HitArray->GetName(), i, j, FEC, String, ChPos);
		NotFoundHitArrays->Add(HitArray);
	      }
	    }
	  }
	}
      }
    }
    // add the new RawBuf container to the event. 
    event->AddObject(RawBufCont);
    // write the event out
    fout.cd();
    event->Write();


    //show lengths of all strings
    // for debugging purposes
//     Char_t FECName[10];
//     for(FEC = 0; FEC < 32; FEC++) {
//       RawBuf = (TPhRawHitFECn0*) 
// 	RawBufCont->GetObject(TPhRawHitFECn0::MakeFECName(FEC, FECName));
//       printf("FEC# %d\n", RawBuf->GetFECNum());
//       for(i = 0; i < 8; i++) 
// 	printf("String %d => Length: %d\n", i, RawBuf->GetRawHitArraySize(i));
//       printf("------------------------\n");
//     }

    //    delete event;
    //  } // while(event = data.GetNextEvent())
  data.Close();
  //  f.Close;
  printf("%d Sensors were not found in the database!\n", 
	 NotFoundHitArrays->GetSize());
  fout.Close();

}
