// Show how to define a new SubDetector and add sensors to it. It then draws all the
// Sensors that got added to it. 
{
  gROOT->Reset();
  Char_t FileName[256];

  // Open the Detector DB and read the current geometry from it. 
  sprintf(FileName, "%s/init/test/Detector.root", gSystem->Getenv("PHATHOME"));
  TFile detDB(FileName);
  TPhDetector *det = (TPhDetector*) detDB->Get("PMC_V3.80");
  detDB.Close();

  // This is our new collection of Ring+Octagon sensors
  TPhSubDetector *MyNewSubDetector = new TPhSubDetector("Ring+Octagon");

  // We now need to iterate over the layerlists of the two SubDetectors in order
  // to get the sensors. First create a temporary TObjArray, where we will add the 
  // layerlists of both subdetectors.
  TObjArray *AllLayers = new TObjArray();
  AllLayers->AddAll(det->GetSubDetector("RingMult")->GetLayerList());
  AllLayers->AddAll(det->GetSubDetector("Octagon")->GetLayerList());

  // Iterate over all the layers in the temporary TObjArray
  TIter NextLayer( AllLayers );
  TPhLayer *Layer;
  TPhSensor *Sensor;
  while(Layer = (TPhLayer*) NextLayer()) {
    TIter NextSensorInLayer(Layer->GetSensorList());
  // Iterate over all sensors in the layer
    while(Sensor = (TPhSensor*) NextSensorInLayer()) {
      // Here we could decide to only take a subset of the already defined
      // sensors in the subdetector...
      printf("Got Sensor: %s\n", Sensor->GetName());
      // Add the sensor to the new subdetector. the layer information that is stored in 
      // the sensor will be used to create new layers. 
      MyNewSubDetector->AddSensor(Sensor);
    }
  }

  // Now display the result
  TCanvas c1("MyNewSubDetector View!!!!");

  // never mind the bricks and the nodes. this is simply to set everything up to 
  // display the sensors in the subdetector. this is normally done in TPhDetector.
  TBRIK Brik("MotherBRIK", "MotherBRIK", "void", 100, 100, 100);
  TNode Node("MotherNode", "MotherNode", "MotherBRIK");
  Node.cd();

  MyNewSubDetector->Draw();

  Node.SetVisibility(2);
  Node.Draw();
  // it will look somewhat funny in the canvas, but try viewing it with X3D, it is ok!
}
