// file phatonl.C  Overall event loop macro for Online analysis
//   Derived from phat.C used in MDC1 9/98       D. McLeod 08-Jan-1998 
//
// Usage:  phat phatonl.C (& simbatch = 0) for interactive use with gui; 
//         phat -b -q phatonl.C  for batch.
//
// What it is:  A combination of the functionality of macros provided by
//              phat program developers (RV -> mul, JK -> sensor monitors,
//              PD/LR -> vertex, KW -> spectr. tracks); in interactive
//              mode, added buttons & dialog boxes for ease of use.
//                 >>>> As of 1/17/99 still no dialog boxes; text "workaround".
//              The code in dialogs.C gives seg viol only in Phat (?!).
//
// Structure:   At a minimum, one can run this script in batch mode which
//              will use the (editable) defaults below, on one "processor",
//              one of macros Vtx.C, Mul.C, Onl.C, Spc.C etc.  You may 
//              also need include files such as Mul.h for global variables.
//              Out.C is needed for output files.   For interactive use,
//              needs several auxiliary macros (see below).
//              
// Comments:    This is and will remain a CINT script for ease in editing
//              and understanding; this is fast enough at the event loop
//              level.  phatonl.C calls other macros which may or may not
//              become compiled classes, as well as compiled phat & root.
//              Some object and variable names are assumed "standard" by
//              the macros and are so labeled below.  
//
// Warning:     This macro requires several "helper" macros as well as the
//              "processor" macros Vtx.C etc.:  phaticonl.C, phatsetonl.C,
//              Onl.C, onlgui.C, onlset.C, mx.C, mxx.C, sensor.C, views.C 
//              and infogv.C.  These may appear in a different directory
//              e.g. within Phat, in phatonl/macros.  If so, the macro
//              search path in .rootrc should contain this directory as
//              well as the directory containing phatonl.C, ugui.C, uset.C
//              and Onl.h (which must be here since there's no search for .h),
//              and the "processor" macros Vtx.C, Mul.C, Mul.h, Tof.C, Tof.h,
//              Out.C, Out.h and any other future processors.
//
// Environment: With setup_Online, .cshrc or some other script, setenv these:
//              INPUT0  = default pathname for the input file (top button)
//              INPUT1  =    "        "     "  alternate file (middle button)
//              INP0NUM = label for the top file selecting button 
//              INP1NUM = label for the middle file selecting button
//              OUTPUT0 = default pathname for the output
//              TEMPLATESn = filenames for templates
//              PHATHELP = command to browse top html of the online HELP
//              ONLHOME: Not essential, but useful (/phobos/u/phobos/online)
//    
/////////////////////////////////////////////////////////////////////////////
//                         INITIALIZATION
/////////////////////////////////////////////////////////////////////////////
//   General phatonl.C initialization:
//     The first section here may be edited by users for various purposes
{
  gROOT->Reset("a");
//  Load include files, if needed, for user macro global variables
//    You will have to select which lines go into the .h file and which
//    go into the initialization phase of the .C files below; note that
//    in the latter, quantities defined in the initialization phase will
//    not be in scope for the event loop, although changes in global
//    variables in initialization will be seen in the event loop.
//
//  Next line moved 2/11/99 from later in phatonl.C to satisfy Mul.h reference
Int_t    iev  =  0;      // We need the event loop counter to be global
#include "Mul.h"
#include "Onl.h"
#include "Spc.h"
#include "Tof.h"
#include "Out.h"
//*************************************************************************

//  Some important variables to edit, or set by interactive GUI if coded:

  Int_t simbatch    = 1;  //Set this to 1 to simulate batch, but with displays
  Int_t ReportLevel = 1;  // Verbosity of comments; 3 = highest
  Int_t ReportFreq  = 1; // every ReportFreq events
  ////Char_t *HitArrayName = "HitArraysfromRawData";// old Phat ver.
  ///Char_t *HitArrayName = "HitArraysFromHits";// May differ for old Phat ver.
  Char_t *HitArrayName = "PMCHitArraysFromHits";// May differ for old Phat ver.
  Int_t detflag = 2;   // 1,2 = get det from Detector.root or data (1/12/99)
  //  Note... some raw data files may not have the detector contained!

 // Number of events, runs etc.: (Edit or set via GUI)
  Int_t nEvents = 10000;
  Int_t nSpc    = 300;      // Max. number of events to run Spc on
  Int_t nRuns   = 10000;

//*************************************************************************
//  Flags to turn on or off the individual analysis processes.
//  These may be changed by interactive phatic.C, or for batch use
//  may be edited temporarily for the particular application.

  // Execution flags:
  Int_t doVtx = 1;  //  Convention: 0 =do not process, 1 = process
  Int_t doOnl = 1;  //  the corresponding data type.  These may be
  Int_t doMul = 1;  //  set to 1 by clicking buttons on the GUI.
  Int_t doSpc = 1;
  Int_t doTof = 0;
  Int_t doOut = 1;  //  Write output if not zero

  // Display flags:
  Int_t flGen = 0;  //  flag for use by ugui, uset to control canvases
  Int_t flVtx = 0;  //  Event loop processing flag: 0 = process only,
  Int_t flOnl = 0;  //  otherwise process & display.  The integer value
  Int_t flMul = 0;  //  may be used to select types of displays. (Also
  Int_t flSpc = 0;  //  controls final displays.)
  Int_t flTof = 0; 
  Int_t flOut = 0;  //  available for use to control output routine
  //  If in batch, process only, regardless of the state of fl*** above.

//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>End of the usual user editables

//  Load the user macros:
//      The macro name list may be changed by editing phatonl.C, phaticonl.C
//  and phatsetonl.C
//  At least "dummy" routines are recommended to avoid error messages here. 
  gROOT->LoadMacro("Vtx.C");
  gROOT->LoadMacro("Mul.C");
  gROOT->LoadMacro("Onl.C");
  gROOT->LoadMacro("Spc.C");
  gROOT->LoadMacro("Tof.C");
  gROOT->LoadMacro("Out.C");

//  Load the interactive control macros:
  if(!gROOT->IsBatch() && (simbatch == 0)) {  
     gROOT->LoadMacro("phaticonl.C");
     gROOT->LoadMacro("phatsetonl.C");
     gROOT->LoadMacro("onlset.C");
     gROOT->LoadMacro("onlgui.C");
     gROOT->LoadMacro("mx.C");
     gROOT->LoadMacro("mxx.C");
     gROOT->LoadMacro("ugui.C");
     gROOT->LoadMacro("uset.C");
     gROOT->LoadMacro("sensor.C");
     gROOT->LoadMacro("infogv.C");
     gROOT->LoadMacro("views.C");
  }
//  Analysis phase flags:
  const Int_t kNull = 0;     // can use to "comment out" parts of user macro
  const Int_t kIni1 = 1;     // initialization (only one stage now)
  const Int_t kRun1 = 11;    // first event loop execution
  const Int_t kRun2 = 13;    // second; for actions dep. on kRun1
  const Int_t kRun3 = 15;    // third, likewise. (Can add 4th etc. later)
  const Int_t kDis1 = 12;    // displays after first stage execution
  const Int_t kDis2 = 14;    // displays after second stage execution 
  const Int_t kDis3 = 16;    // displays after third stage execution
  const Int_t kDisf = 18;    // displays in final execution after ev. loop
  const Int_t kFin1 = 101;   // after run processing stage 1 (all for now)
  const Int_t kFinf = 201;   // final run processing, after displays

//  These are compared to the "phase" flag in user macros to generate the 
//  proper analysis sequence. 

  //  Loop control flags (irrelevant if in batch mode)
  Int_t doGo    = 0;     //  Leave 100 ms loop before event loop
  Int_t doStop  = 0;     //  End the event loop by user request
  Int_t doPause = 0;     //  while != 0, 100 ms delay loop within event loop
                         //  allowing interactive manipulation of last event
  Int_t doDisp  = 1;     //  Toggle set by any button choice, cleared after
                         //  first display in pause, set at pause exit.

//  default file names (in environment variables as per discussion 8/25/98)
//    phaticonl.C allows interactive change of InFile, OuFile:
  Char_t InFile[100];
  strcpy(InFile,"$INPUT0\0");
  printf("Default input file set to $INPUT0, expanded name = %s\n",
    gSystem->ExpandPathName(InFile));
  Char_t OuFile[100];
  strcpy(OuFile,"$OUTPUT0\0");
  printf("Default output file set to $OUTPUT0, expanded name = %s\n",
    gSystem->ExpandPathName(OuFile));

//  Major changes here for MDC2 to allow multiple templates.

  Char_t *TemplFile1 = "$TEMPLATES1";  //First template file
  printf("Template file #1 set to $TEMPLATES1, expanded name = %s\n",
    gSystem->ExpandPathName(TemplFile1));
  printf("Other template files, assumed in same directory:\n");
  Char_t *TemplFile2 = "$TEMPLATES2";
  printf("#2 = %s\n",TemplFile2);
  Char_t *TemplFile3 = "$TEMPLATES3";
  printf("#3 = %s\n",TemplFile3);
  Char_t *TemplFile4 = "$TEMPLATES4";
  printf("#4 = %s\n",TemplFile4);
  Char_t *TemplFile5 = "$TEMPLATES5";
  printf("#5 = %s\n",TemplFile5);
  Char_t *TemplFile6 = "$TEMPLATES6";
  printf("#6 = %s\n",TemplFile6);
//
// Declarations of objects; note that user routines (Mul.C etc.)
// must use these names to reference these objects instantiated by phat.C.

  TPhDetector *det;
  TPhDST      *data = NULL;   // Either raw or dst format
  TPhEvent    *ev  =  NULL;
  TPhVertex   *v1  =  NULL;  
  TCanvas*    barg =  NULL;   // Canvas for ugui.C buttons

// Create the detector.  ReadLatestGeometry depends on the existence
// of Detector.root in $PhatHome/init/test/.  Macros assume object "det".
// As A.O. points out, this may not correspond to the data!  See detflag above.
  if(detflag == 1) {
    if ((det = TPhDetector::GetInstance()) == NULL) { 
      det = TPhDetector::ReadLatestGeometry();
    }
    if(!det) {
       printf("FATAL.. maybe no $PHATHOME/init/test/Detector.root?")
       gSystem->Abort();
    }
  }
// Execute phase 1 of the interactive macro phatic.C, brings up GUI
// to alter various parameters from the defaults set here.  
  if(!gROOT->IsBatch() && (simbatch == 0)) {  
      phatic(1);       // Panel of initialization buttons
     // Now enter a loop while the buttons are pushed & dialog boxes
     // are filled; exit when "GO" is pushed.
     //     This loop times out after 10 minutes
     for (Int_t i = 0; i < 6000; i++) {
       gSystem->ProcessEvents(); // New ROOT 2.10 method restores interrupts
       if(doGo == 1) break;   // Continue on if "GO" button is pushed 
       if(flGen == 2) {
           barg->Close();  // Remove ugui canvas
          barg = NULL;
          flGen = 0;
       }
      gSystem->Sleep(100);   // 1/10 second delay 
     }
     printf("Exit setup loop\n");
  }     //end IsBatch
//   Any alterations by the initialization GUI are now in effect.
//
// Open input data file.  Macros assume object "data".
  data = new TPhDST(InFile);   // This is used for both raw and dst input
  if(!data) {
     printf("FATAL.. InFile, expanded name = %s, can't open!\n",
        gSystem->ExpandPathName(InFile));
     gSystem->Abort();
  }
  printf("Input data file opened, expanded name = %s\n",
        gSystem->ExpandPathName(InFile));

//  12-Jan-1999: On A.O.'s recommendation, get detector from the data file:
  if(detflag == 2) det = data->GetDetector();
  det->Print("1");
// The output file, if wanted, is opened by the Out.C macro.  The default
// name is set above, and may have been changed by the GUI.
// Initialize analysis modules:
  printf("Start of initialization, start event processing at\n");
  if(doVtx >= 1) Vtx(kIni1);
  if(doMul >= 1) Mul(kIni1);
  if(doOnl >= 1) Onl(kIni1);
  if(doSpc >= 1) Spc(kIni1);
  if(doSpc >= 1) Tof(kIni1);
  if(doOut >= 1) Out(kIni1);

  printf("End of initialization, start event processing at\n");
  gSystem->Exec("date");

  gBenchmark->Start("phat.C");
//////////////////////////////////////////////////////////////////////////
//                           EVENT LOOP
//////////////////////////////////////////////////////////////////////////
  if(!gROOT->IsBatch() && (simbatch == 0)) phatic(2);   
         // 2nd panel for event loop flags.
//  This continues to be active during the event loop and allows
//  asynchronous setting of flags for the event loop to read.

// Loop over all events
   for (iev = 0; iev < nEvents; iev++) {
     if(doStop >> 0) break;    // STOP button was pressed.  The effect will
                 // nevertheless usually appear to be delayed by one event.
     ev = data->GetNextEvent();
     if (!ev) break;   //EOF
     if(iev%ReportFreq==0 && ReportLevel >=1 && iev) printf("Event# %d\n",iev);
     det->ClearHits();
     det->SetEvent(ev);
//      changed MDB edit to allow raw/dst choice for use after reconstruction:
//   det->MakeHitArrays("MCRawHitFECn0s", HitArrayName);// chng 2/110/99 DMcL
     det->MakeHitArrays("PMCRawHitFECn0s", HitArrayName);
     det->FillWithEventHitArrays(HitArrayName);
//  Line below added 2/11 after spectrometer found empty of hits (P.D. changes)
     det->MakeHitLists("Hits");
     if(ReportLevel >= 3) printf("Hit arrays into detector\n");

// Step through three phases e.g. vertex first, then multiplicity, then
// through everything else.  Use phase e.g kRun2 to control the sequence.

     if(doVtx >= 1) {            // find vertex (or set to 0)
       Vtx(kRun1);
       if(v1 && (ReportLevel >= 3) ) {
         printf("Vertex at %f %f %f\n",v1->GetX(),v1->GetY(),v1->GetZ());
       }
     }
     else {
       v1 = new TPhVertex("MainVertex");
       ev->AddObject(v1);
       v1->SetX(0.0);   // MDB... default the vertex to 0 if
       v1->SetY(0.0);   // there is no vertex finder called by phat.C
       v1->SetZ(0.0);
     }
     if(doMul >= 1) Mul(kRun1);  // multiplicity analysis
     if(doOnl >= 1) Onl(kRun1);  // online monitoring
     if(doSpc >= 1 && iev < nSpc) Spc(kRun1);  // spectrometer tracking
     if(doTof >= 1) Tof(kRun1);
     if(doOut >= 1) Out(kRun1);  // output

     if(doVtx >= 1) Vtx(kRun2);  // more vertex processing if wanted
     if(doMul >= 1) Mul(kRun2);  // second stage processing if wanted
     if(doOnl >= 1) Onl(kRun2); 
     if(doSpc >= 1 && iev < nSpc) Spc(kRun2);  
     if(doTof >= 1) Tof(kRun2); 
     if(doOut >= 1) Out(kRun2); 

     if(doVtx >= 1) Vtx(kRun3);  // third stage processing
     if(doMul >= 1) Mul(kRun3);  
     if(doOnl >= 1) Onl(kRun3);  
     if(doSpc >= 1 && iev < nSpc) Spc(kRun3); 
     if(doTof >= 1) Tof(kRun3); 
     if(doOut >= 1) Out(kRun3); 

//  EVENT LOOP DISPLAYS
//  Execute the following at least once if interactive; loops if paused.
//  For now we assume only one display phase, at the end of event analysis.
//       (If simbatch = 1, we still get displays to follow the progress.)
     if(!gROOT->IsBatch()) {
       do {
         if(doStop >> 0) break;    // STOP button was pressed during PAUSE
         if(flGen == 2) {
            barg->Close();  // Remove ugui canvas
            barg = NULL;
            flGen = 0;
         }
         if(simbatch == 0) gSystem->ProcessEvents();
         if(doDisp == 1) {
	   //   We assume for now only one phase (kDis3) of displays.
	   if(doVtx >= 1 && flVtx != 0) Vtx(kDis3,flVtx);
	   if(doMul >= 1 && flMul != 0) Mul(kDis3,flMul);
	   if(doOnl >= 1 && flOnl != 0) Onl(kDis3,flOnl);
	   if(doSpc >= 1 && flSpc != 0 && iev < nSpc) Spc(kDis3,flSpc);
	   if(doTof >= 1 && flTof != 0) Tof(kDis3,flTof);
	   if(doOut >= 1 && flOut != 0) Out(kDis3,flOut);
           doDisp = 0;  // Turn off, comes back on if button push
  	 }   
         if(doPause != 0) gSystem->Sleep(100);   // Don't tie up CPU!
       }   while (doPause != 0);
       doDisp = 1;  // Turn the displays back on after pause
     }     //end !IsBatch 
 
     if (ReportLevel >= 2) printf("Delete event\n");
     delete ev;
   }       //end event loop
   printf("End of event loop at\n");
   gSystem->Exec("date");

///////////////////////////////////////////////////////////////////////////
//                             FINAL ACTIONS            
///////////////////////////////////////////////////////////////////////////
// End of run, store histograms etc.; for batch use, exit or clean up & 
// return to initialization but for interactive use allow user plots, 
// possible output of histograms, etc.

//    Run final actions before interactive displays:
   if(doVtx >= 1) Vtx(kFin1,flVtx);  // vertex
   if(doMul >= 1) Mul(kFin1,flMul);  // multiplicity 
   if(doOnl >= 1) Onl(kFin1,flOnl);  // online monitoring
   if(doSpc >= 1) Spc(kFin1,flSpc);  // spectrometer tracking
   if(doTof >= 1) Tof(kFin1,flTof);  // TOF analysis 
   if(doOut >= 1) Out(kFin1,flSpc);  // write output DST  

   printf("After Fin1 final actions \n");

//  END OF RUN DISPLAYS
//    Interactive display loop:
   if(!gROOT->IsBatch()) {
     if (simbatch == 0) phatic(3);   // Panel for final stage
     // Now enter a loop while the buttons are pushed & dialog boxes
     // are filled; exit when "EXIT" is pushed.
     //     This loop times out after 10 minutes
     for (Int_t i = 0; i < 6000; i++) {
       if(doGo == 10) break;   // Continue on if "EXIT" button is pushed,
                               // the button-selected options will execute.  
       if(flGen == 2) {
          barg->Close();  // Remove ugui canvas
          barg = NULL;
          flGen = 0;
       }
       if(simbatch == 0) gSystem->ProcessEvents();
       if(doDisp == 1) {
	 //   We assume for now only one phase (kDisf) of displays.
	 if(doVtx >= 1 && flVtx != 0) Vtx(kDisf,flVtx);
       	 if(doMul >= 1 && flMul != 0) Mul(kDisf,flMul);
	 if(doOnl >= 1 && flOnl != 0) Onl(kDisf,flOnl);
	 if(doSpc >= 1 && flSpc != 0) Spc(kDisf,flSpc);
	 if(doTof >= 1 && flTof != 0) Spc(kDisf,flTof);
         doDisp = 0;  // Turn off, comes back on if button push
       }
       if(simbatch !=0) break;   // Avoid looping in simulated batch
       gSystem->Sleep(100);   // Don't tie up CPU!
     }
   }
   printf("Before Finf final actions \n");
//    Run final actions after interactive displays:
   if(doVtx >= 1) Vtx(kFinf,flVtx);  // vertex final actions
   if(doMul >= 1) Mul(kFinf,flMul);  // multiplicity 
   if(doOnl >= 1) Onl(kFinf,flOnl);  // online monitoring
   if(doSpc >= 1) Spc(kFinf,flSpc);  // spectrometer tracking
   if(doTof >= 1) Tof(kFinf,flTof);  // TOF analysis
   if(doOut >= 1) Out(kFinf);        // write output #2 (if wanted)
      
   printf("RUN COMPLETED at\n");
   gSystem->Exec("date");
   gBenchmark->Show("phat.C");
   if(!gROOT->IsBatch()) gSystem->Exit(1);
   // Remove the above if you want ROOT to continue.
}  // End of phatonl.C









