// file phatrdon.C   Adaptation of onlgui.C (part of the phatonl system)
// to read saved plots; later may be extended to interprocess monitoring.
//   Requires other macros onlset.C, ugui.C, uset.C, mx.C, mxx.C, sensor.C,
// infogv.C, views.C (as used by phatonl.C) to implement button functions. 
//                                                D. McLeod 31-Jan-1999
{
   gROOT->Reset("a");

   Int_t ReportLevel = 1;  // Verbosity of comments; 3 = highest

/////////////////////////////////////////////////////////////////////
//  Load the interactive control macros:
/////////////////////////////////////////////////////////////////////

   gROOT->LoadMacro("onlset.C");
   gROOT->LoadMacro("mx.C");
   gROOT->LoadMacro("mxx.C");
   gROOT->LoadMacro("ugui.C");
   gROOT->LoadMacro("uset.C");
   gROOT->LoadMacro("sensor.C");
   gROOT->LoadMacro("infogv.C");
   gROOT->LoadMacro("views.C");

/////////////////////////////////////////////////////////////////////
//  Define some global objects:
/////////////////////////////////////////////////////////////////////
   Char_t InFile[100];  // Input file with TPhSensor object
   TPhDetector *det = TPhDetector::ReadLatestGeometry();
   Int_t   nchan;
   TCanvas* onlc1;   //  These handles are needed in phatset.C for onlgui
   TCanvas* onlc2;   // bar for layers
   TCanvas* onlc3;   // bar for sensors
   TCanvas* barg;    // miscellaneous modifier bars
   TCanvas* view;    // for 3-d views of the subdetectors
   Int_t bbflag = 2;  //  Blue button flag for onlgui display choices

/////////////////////////////////////////////////////////////////////////////
//   Open the input file.   If later there are many options to be chosen,
//   we will need to put a delay loop in here to suppress exit until done.
/////////////////////////////////////////////////////////////////////////////
   Char_t *datadir;  // Directory first appearing in file selector box
   if(!(datadir = gSystem->Getenv("DATAHOME"))) {   
      datadir = gSystem->Getenv("HOME"); // Default $HOME if no $DATAHOME
   }
   gSystem->ChangeDirectory(datadir);
   const char *gOpenAsTypes[] = {
     "ROOT files",   "*.root",
     "All files",    "*",
      0,              0
   };
   static TGFileInfo fi;
   fi.fFileTypes = gOpenAsTypes;
   new TGFileDialog(gClient->GetRoot(), gClient->GetRoot(), kFDOpen, &fi);
   if(fi.fFilename == NULL) {
     printf("Try again, no input file!\n");
   } // CANCEL button pushed
   strcpy (InFile,gSystem->WorkingDirectory());
   strcat (InFile,"/");
   strcat (InFile,fi.fFilename);
   printf("Input file now set to expanded name %s\n",InFile);
   gROOT.cd();
   TFile fin(InFile);
   TPhOnSensorContainer* tt = fin.Get("senshistos"); 
   //////TPhOnSensorContainer* tt = fin.Get("Sensors"); 

/////////////////////////////////////////////////////////////////////
//  Set up the canvas for selection of plots etc.; this is nearly a
//  clone of the Onl options in phatonl.C now (may be changed).
/////////////////////////////////////////////////////////////////////
   onlc1 = new TCanvas("online","online",-5,30,80,790);
   onlc1->SetEditable(0);    // equivalent to TDialogCanvas; note that
   onlc1->SetFillColor(38);  // TDialogCanvas has no 6 arg constructor.
   onlc1->SetFillStyle(3014);  // same as phat.C appearance
   onlc1->Draw();

   //  "HELP" button at top
   TButton *buth = new TButton("HELP","gSystem.Exec("$PHATHELP");",
         .02,.88,.98,.98);
   buth->SetFillColor(8);
   buth->SetTextColor(10);
   buth->SetTextSize(0.35);
   buth->Draw();

//  Text right on canvas
   TText *label = new TText(0.05,0.86,"  subdet");
   label->SetTextSize(0.22); 
   label->SetTextColor(5);
   label->Draw();

//    First group of 11 buttons.. y = .85 to .37
   Float_t ytop = 0.85; // Canvas coord. are 0.0 - 1.0 going up
   Float_t ybot = 0.12;
   Float_t d = (ytop - ybot)/11.0; // Button spacing
   Float_t s = d/20.0; // half-space between buttons
   Float_t ylow = ytop -d +s;  // initial button bottom
   Float_t yup = ytop -s;      // initial button top
//
   TButton *but1 = new TButton("Vertex","mx(1);onlset(\"Vertex\");",
   .01, ylow, .99, yup);
   but1->SetFillColor(4);
   but1->SetTextColor(10);
   but1->SetTextSize(0.50);
   but1->Draw();
//
   TButton *but2 = new TButton("RingMult","mx(1);onlset(\"RingMult\");",
   .01, ylow -d, .99, yup -d);
   but2->SetFillColor(4);
   but2->SetTextColor(10);
   but2->SetTextSize(0.40);
   but2->Draw();
//
   TButton *but3 = new TButton("Octagon","mx(1);onlset(\"Octagon\")",
   .01, ylow - 2.0*d, .99, yup -2.0*d);
   but3->SetFillColor(4);
   but3->SetTextColor(10);
   but3->SetTextSize(0.40);
   but3->Draw();
//
   TButton *but4 = new TButton("SpectP","mx(1);onlset(\"SpectP\");",
   .01, ylow - 3.0*d, .99, yup -3.0*d);
   but4->SetFillColor(4);
   but4->SetTextColor(10);
   but4->SetTextSize(0.50);
   but4->Draw();
//
   TButton *but5 = new TButton("SpectN","mx(1);onlset(\"SpectN\");",
   .01, ylow - 4.0*d, .99, yup -4.0*d);
   but5->SetFillColor(4);
   but5->SetTextColor(10);
   but5->SetTextSize(0.50);
   but5->Draw();
//
   TButton *but6 = new TButton("TOFP","mx(1);onlset(\"TOFP\");",
   .01, ylow - 5.0*d, .99, yup -5.0*d);
   but6->SetFillColor(4);
   but6->SetTextColor(10);
   but6->SetTextSize(0.50);
   but6->Draw();
//
   TButton *but7 = new TButton("TOFN","mx(1);onlset(\"TOFN\");",
   .01, ylow - 6.0*d, .99, yup -6.0*d);
   but7->SetFillColor(4);
   but7->SetTextColor(10);
   but7->SetTextSize(0.50);
   but7->Draw();
//
   TButton *but8 = new TButton("Trigger","mx(1);onlset(\"Trigger\");",
   .01, ylow - 7.0*d, .99, yup -7.0*d);
   but8->SetFillColor(4);
   but8->SetTextColor(10);
   but8->SetTextSize(0.50);
   but8->Draw();
//
   TButton *but9 = new TButton("ZDC","mx(1);",
   .01, ylow - 8.0*d, .99, yup -8.0*d);
   but9->SetFillColor(4);
   but9->SetTextColor(10);
   but9->SetTextSize(0.50);
   but9->Draw();
//
   TButton *but10 = new TButton("PIN","mx(1);",
   .01, ylow - 9.0*d, .99, yup -9.0*d);
   but10->SetFillColor(4);
   but10->SetTextColor(10);
   but10->SetTextSize(0.50);
   but10->Draw();
//
   TButton *but11 = new TButton("SLOW","mx(1);",
   .01, ylow - 10.0*d, .99, yup -10.0*d);
   but11->SetFillColor(4);
   but11->SetTextColor(10);
   but11->SetTextSize(0.50);
   but11->Draw();
//

  //  "EXIT" button at bottom
   TButton *but11 = new TButton("EXIT",
       "gSystem->Exec(\"date\");gSystem->Exit(1);",
       .02,.02,.98,.12);
   but11->SetFillColor(2);
   but11->SetTextColor(10);
   but11->SetTextSize(0.50);
   but11->Draw();
   //  "printf(\"COMPLETED at\n\");gSystem->Exec(\"date\");gSystem->Exit(1);",

///////////////////////////////////////////////////////////////////////////
//  After the canvas is drawn, buttons interrupt to carry out functions
///////////////////////////////////////////////////////////////////////////
}



