// A.O. 18.06.1998
// example macro to run one particle through Phobos setup with the PMC code.
// Warning: TPhPMC singleton object is allowed to be initialized by Init() function
//          only once so the macro (as a whole) can not be repeatedly executed.
//
// $Id: single_event.C,v 1.6 1999/03/09 07:49:54 olszewsk Exp $

// Run macro with a the input file name 
// (standard $PHATHOME/data directory is assumed)
runpmc(Char_t *mc_input=NULL, Int_t runno=1)
{
   gROOT->Reset();

   const Char_t *PhatDirName = gSystem->Getenv("PHATHOME");
   const Char_t *PMCDirName = gSystem->Getenv("PMCHOME");
   Char_t gfilename[100];

   TPhPMC *pmc = TPhPMC::CreateInstance();
   //   pmc->SetVerbose(2);
   TStopwatch timer;

   TPhEvent *event, *testeventp;
   TPhDetector *det;
   // The detector is created automatically from the UGCDATA file 
   // when TPhPMC object gets initialized,
   // and "writing to memory" (SMEM) option is turned on in the control-file.

   // For a correct initialization standard PMC I/O files must be set via 
   // soft links
   // They can be set under system shell or by calls to
   // TPhPMC::SetPathToFile(Char_t *filesymbol, Char_t *filename)
   // before Init() has been called.
   // "filesymbols" are:
   // -----------------------------------------------------
   // FLDASC;           //  path to magnetic field map file
   // GENEVS;           //  path to HIJET input data file
   //    the links to the above two file types can be 
   //    changed even after Init() has been called.
   // -----------------------------------------------------
   // GCDEFS;           //  path to geometry UGCDATA file
   // RHAOU;            //  path to output ascii AOU file
   // RHDATA;           //  path to control file with PMC parameters
   // RHASC;            //  path to console output file
   // RHHST;            //  path to output histogram file
   // RHNTU;            //  path to ntuple output file for tracks with hits
   // MCNTU;            //  path to ntuple output file for all tracks
   // RHZEB;            //  path to RZ-format Zebra bank output file
   //
   // If links to files are not set an attempt to connect "standard" files 
   // at locations PMCHOME, PHATHOME or . (current directory) is made.
   Char_t *basename[] = mc_input;
   // Input file name is set here
   // ---------------------------
   if(mc_input) {
     sprintf(gfilename,"%s/data/%s",PhatDirName,basename);
     pmc->SetPathToFile("GENEVS", gfilename);
   }

   // Output data file in aou format.
   // -------------------------------
//    if(!basename) basename = "output";
//    sprintf(gfilename,"%s/data/%s",PhatDirName,basename);
//    strcat(gfilename,".aou");
//    pmc->SetPathToFile("RHAOU", gfilename);

   // Output data file in root format.
   // -------------------------------
   sprintf(gfilename,"%s/data/%s",PhatDirName,basename);
   strcat(gfilename,".root");
   TFile *file = new TFile(gfilename,"RECREATE");
   if( !file->IsOpen() ) { 
      printf("Output file %s could not be opened!\n", gfilename);
      return;
   }

   Char_t parname[10];
   Int_t number = 0; Float_t values[5] = {0,0,0,0,0};
   // An "event at a time" mode (other modes available)
   // should be set in a RHDATA control-file to "GEN " other then 10. 
   // Some other parameters can also be changed interactively e.g.:

   if( !pmc->Init() ) return;
   det = TPhDetector::GetInstance();
   printf("\n(PHAT)PMC run with:\n GEANT ver.%6.3f\n PMC ver. %6.3f\n Geometry ver. %6.3f\n",
           pmc->GetGeantVer(),pmc->GetPmcVer(),pmc->GetPmcDetVer());

   
   // make sure to read tracks from file (GEN=0)
   strcpy(parname,"GEN "); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      return;
   }

   // Turn-off multiple scattering
   strcpy(parname,"MULS"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      return;
   }
   // Turn-off decays
   strcpy(parname,"DCAY"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      return;
   }
   // Turn-off hadronic interactions
   strcpy(parname,"HADR"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      return;
   }
   // Turn-off energy loss
   strcpy(parname,"LOSS"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      return;
   }
   // Turn-off pair production
   strcpy(parname,"PAIR"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      return;
   }
   // Turn-off bremstrahlung
   strcpy(parname,"BREM"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      return;
   }
   // Turn-off compton scattering
   strcpy(parname,"COMP"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      return;
   }
   // Turn-off photoelectric effect
   strcpy(parname,"PHOT"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      return;
   }
   // Turn-off delta-rays
   strcpy(parname,"DRAY"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      return;
   }
   // Turn-off annihilation
   strcpy(parname,"ANNI"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      return;
   }
   // Other parameters available to change:
   // CUTS MULS DCAY HADR LOSS PAIR BREM COMP PHOT DRAY ANNI PIXL 
   // SECT RBXX RBYY RBZZ RMOM RPID RETA RPHI RARM RMUL 
   // TRIG GEN SAOU SMEM 
   // their meaning is described inside the control-file, 
   // current values are printed by a call to pmc->PrintPMCPars().
   pmc->PrintPMCPars();

   // Make a new run object 
   // Handling of run and event objects is done now outside of the TPhPMC class
   TPhRun *run;
   run= new TPhRun(runno);
   run->SetTitle("Run generated by TPhPMC");
   // Set link to run object (temporary solution)
   pmc->SetRunLink(run);

   Int_t ntimes = 2;
   // ntimes by TRIG events will be simulated
   timer.Start();
   for(int i=0; i<ntimes; i++) {
     // TPhEvent event object is created by a user and then filled by PMC with a call
     // to pmc->SwimPmcEvent() or SwimPhatEvent(). 
     // The first call reads input tracks from the HIJET output file, while the second one
     // reads from object container in the TPhEvent object. 
     // TPhPMC creates and fills specified containers with (one or more (daughter)) particle
     // tracks in one container and their hits in the another one.
     event = new TPhEvent(runno,i);

     // The call will process a number of TRIG (set in control-file) events,
     // the availability of enough data in the input file can not be checked for so be carefull.
     // Tracks are assumed to be read from a file
     testeventp = pmc->SwimPmcEvent(event, "PMCTracks", "PMCHits"); 

     // below is an example where the same event is used as a source and as an output storage, 
     // where the simulated particles are to be put. 
     // It should contain at least a "inTrackContainer" container with TPhTrack objects.
     //     pmc->SwimPhatEvent(event, "inTrackContainer", event, "PMCTracks", "PMCHits");

     // creation of Phat TPhTracks and TPhHits occures only 
     // if SMEM value was set > 0 in the RHDATA file. 
     // SMEM value cannot be increased to > 0 after a call to Init() to prevent
     // from use of different geometries in PMC simulations and hit labeling.
     // abort if for some reason Swim****Event does not swim particles
     if(!testeventp) exit(0);

     // convert hits to hit arrays and then to raw hits
     det->SetEvent(event);
     det->MakeHitArraysOutOfHits("PMCHits","PMCHitArraysFromHits");
     det->MakeRawHitFECn0("PMCHitArraysFromHits", "PMCRawHitFECn0s");
 
     file->cd();
     event->Print("1");
     event->Write();
     run->AddEvent(event);

     delete event;
   }
   timer.Stop();
   timer.Print();

   run->Write();
   if(det) det->Write("LatestGeometry");

   file->Close();
}
