#ifndef _TPhBT98DST_INCLUDED_
#define _TPhBT98DST_INCLUDED_

#include "TObject.h"

#include "TPhEvent.h"
#include "TPhDetector.h"
#include "TPhRawHitFECn0TB98.h"

///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhBT98DST                                                     //
//                                                                   //
//    PHOBOS service class to access data on the                     //
//    binary Test Beam files                                         //
//                                                                   //
//    TPhBT98DST creates TPhEvents out of the Test Beam '98 binary   //
//    data written to files in a format of the packed data of        //
//    TPhRawHitFECn0 objects                                         //
//                                                                   //
//    Author  : Andrzej Olszewski (andrzej.olszewski@ifj.edu.pl)     //
//    Created : 10.1998                                              //
//    Version : 1.0                                                  //
//    Changed :                                                      //
//                                                                   //
//                                                                   //
// $Id: TPhBT98DST.h,v 1.3 1999/02/24 23:10:39 olszewsk Exp $
//                                                                   //
///////////////////////////////////////////////////////////////////////

// used buffer sizes
const Int_t    kTB98Channels   = 768;  // channels per string - 2 bytes info
const Int_t    kTB98StrRep     = 2;    // strings per repeater
//const Int_t    kTrailerLength = 20;  // trailer length (2 byte words) 
                                       // already defined in TPhRawHitFECn0
const Int_t    kTB98BufferSize = kTB98Channels*kTB98StrRep*2 + (kTrailerLength*2); // 3112 bytes

// the necessary transformation of FEC and string numbers to Phat convention
// for NA00,NA01,...,ND01 planes.
const Int_t kFECNo[16]      = {8,0,8,0,8,0,8,0,9,1,9,1,9,1,9,1}; 
const Int_t kStringRem[16]  = {5,0,4,1,5,0,4,1,5,0,4,1,5,0,4,1};
const Int_t kStringIn[16]   = {0,5,1,4,0,5,1,4,0,5,1,4,0,5,1,4};
const Int_t kStringOut[16]  = {0,1,1,0,2,3,3,2,0,1,1,0,2,3,3,2}; 

class TPhBT98DST : public TObject {
    
  public: 
    TPhBT98DST();
    TPhBT98DST(Char_t *FileName, Int_t skip = 0, Int_t runno = 0);
    virtual ~TPhBT98DST();

    Bool_t        Open(Char_t *FileName);
    Bool_t        Close();
    void          Reset();
    void          ls() { if(fFileName) printf("Beam Test file: %s\n", fFileName); }
    void          Print() { ls(); 
                            printf("  current event no: %d, %d bytes has been lost so far\n", 
                                   fNumEvent, fBytesLost); 
                          }
    TPhEvent     *GetNextEvent(Int_t skip=0);
    Bool_t        SkipEvent(Int_t skip);  

    void          SetFileName(Char_t *name) { strncpy(fFileName,name,100); }
    void          SetRunNo(Int_t runno) { fRunNo = runno; }
    void          SetVerbose(Int_t flag) { fVerbose = flag; }

     Int_t        GetCurrentEventNumber() { return fNumEvent; }
     Int_t        GetBytesRead() { return fBytesRead; }
     Int_t        GetBytesLost() { return fBytesLost; }
     Int_t        GetVerbose() { return fVerbose; }
    Bool_t        IsInitialized() { return fInitialized; }   

  private:
    void        ClearBuffer();
    Bool_t      ReadRepeaterBlock();

    Char_t      fFileName[100];           // Test Beam '98 DST file name
     Int_t      fNumEvent;                // Current event counter 
     Int_t      fRunNo;                   // Run number for created events
     Int_t      fBytesRead;               // Total no of bytes read 
     Int_t      fBytesLost;               // No of bytes skipped due to read failures
    UChar_t     fBuffer[kTB98BufferSize]; //!read buffer
    FILE       *fDInput;                  //!Test Beam '98 DST file
    Int_t       fVerbose;                 //!Control debugging output
   Bool_t       fInitialized;             //!Flag to set if initalization was succesfull

  public:
    ClassDef(TPhBT98DST,1)  // PHOBOS Test Beam '98 DST interface class    
};

#endif

