#ifndef _TPhEventHeader_INCLUDED_
#define _TPhEventHeader_INCLUDED_

///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhEventHeader                                                 //
//                                                                   //
//    Phobos event class                                             //
//                                                                   //
//    TPhEventHeader containing general information on event ID      //
//    and type                                                       // 
//                                                                   //
//    Author  : Gunther Roland (rolandg@vxcern.cern.ch)              //
//    Created : 7/12/97                                              //
//    Version : 1.0                                                  //
//    Changed : Andrzej Olszewski (andrzej.olszewski@ifj.edu.pl)     //
//              A list of TPhParKey objects, parameters accessed     //
//              by their name, added.                                //
//                                                                   //
// $Id: TPhEventHeader.h,v 1.3 1998/12/11 14:18:05 olszewsk Exp $                                                              //
//                                                                   //
///////////////////////////////////////////////////////////////////////

// Root Classes

// Phat classes
#include "TObject.h"
#include "TClass.h"
#include "TPhParManager.h"

class TPhEventHeader : public TObject {

public:

    TPhEventHeader();
    TPhEventHeader(Int_t run, Int_t event);

    virtual ~TPhEventHeader();

    Int_t     GetNRun()          { return fNRun;          }
    Int_t     GetNEvent()        { return fNEvent;        }
    Int_t     GetTriggerMask()   { return fTriggerMask;   }
    Int_t     GetDate()          { return fDate;          }
    Int_t     GetTime()          { return fTime;          }
    Int_t     GetDetStatus()     { return fDetStatus;     }

    void      SetNRun(Int_t NRun)     { fNRun = NRun;     }
    void      SetNEvent(Int_t NEvent) { fNEvent = NEvent; }
    void      SetTriggerMask(Int_t TriggerMask) { fTriggerMask = TriggerMask; }
    void      SetDate(Int_t Date)     { fDate = Date;     }
    void      SetTime(Int_t Time)     { fTime = Time;     }
    void      SetDetStatus(Char_t status)     { fDetStatus = status;     }

    TPhParManager *GetParManager()            { return fParMgr;          }
    void           AddParameter(TPhParKey *par);
    Int_t          GetNPars();
    TPhParKey     *GetParameter(Int_t pos);

    void           Print();

private:

    Int_t       fNRun;            //  run number
    Int_t       fNEvent;          //  event number
    Int_t       fTriggerMask;     //  trigger mask
    Int_t       fDate;            //  date of the trigger
    Int_t       fTime;            //  time of the trigger
    Char_t      fDetStatus;       //  detector status flag
    TPhParManager *fParMgr;       //  pointer to TPhParManager object 
                                  //    with parameters accessed by name
    
public:

    ClassDef(TPhEventHeader,2)    // PHOBOS event header class

};

#endif




