#ifndef _TPhHashContainer_INCLUDED_
#define _TPhHashContainer_INCLUDED_

///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhHashContainer                                               //
//                                                                   //
//    Phobos hash container class. Store objects that inherit from   //
//    TNamed in this container. This container in turn is managed by // 
//    TPhEvent.                                                      //
//                                                                   //
//    Author  : M. Patrick Decowski                                  //
//    Created : September 10 1997                                    //
//    Version : 1.0                                                  //
//    Changed :                                                      //
//                                                                   //
//    $Id: TPhHashContainer.h,v 1.2 1998/05/02 17:53:59 olszewsk Exp $                                                           //
//                                                                   //
///////////////////////////////////////////////////////////////////////

// Root Classes

#include "THashTable.h"

// PHOBOS Classes

#include "TPhAbsDataContainer.h"

class TPhHashContainer : public TPhAbsDataContainer {

private:

  THashTable *fHashTable;     // Hash Table of objects
  Int_t fInitCap;             // Initial capacity
  Int_t fReHash;              // Collision rate rehash trigger

public:

  TPhHashContainer();
  TPhHashContainer(Char_t *Name,Char_t *Title = NULL);
    
  virtual ~TPhHashContainer();

  virtual void Add(TObject* Object);
  virtual void Add(TPhDataObject *object);

  virtual TPhDataObject *GetObject(const Char_t *name)   
    { return (TPhDataObject*) fHashTable->FindObject(name); }

  virtual TIterator* GetIterator(Bool_t dir = kIterForward) 
    { return fHashTable->MakeIterator(dir); }

  virtual Int_t GetEntries() 
    { return fHashTable->GetSize(); }

  virtual void SetInitCap(Int_t InitCap) { fInitCap = InitCap; }
  virtual void SetReHash(Int_t ReHash) { fReHash = ReHash; }

  virtual void ls(Option_t *option = "");
  virtual void Print(Option_t *option = "");
  virtual void Draw(Option_t *option = "");

  ClassDef(TPhHashContainer,1)  // Hash Table container class for classes inheriting from TNamed

};

#endif /* _TPhHashContainer_ */




