#ifndef _TPhParKey_INCLUDED_
#define _TPhParKey_INCLUDED_


//////////////////////////////////////////////////////////////////////////
//                                                                      //
//    TPhParKey                                                         //
//                                                                      //
//    Phobos class for storing parameters used in data processing       //
//    software chain                                                    //
//                                                                      //
//    TPhParKey provides storage for notes of values of various         //
//    parameters and names of auxiliary files that have been used       //
//    on the path of data generation/simulation                         //
//                                                                      //
//                                                                      //
//    Author  : Andrzej Olszewski (andrzej.olszewski@ifj.edu.pl)        //
//    Created : 01/12/98                                                //
//    Version : 1.0                                                     //
//    Changed :                                                         //
//                                                                      //
// $Id: TPhParKey.h,v 1.2 1998/12/15 22:53:46 decowski Exp $
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#include <string.h>

#include "TObject.h"
#include "TString.h"

class TPhParKey: public TObject {

  public: 
    TPhParKey();
    TPhParKey(Char_t *source, Char_t *parname, Char_t *value, 
	      Char_t partype=-1);
    virtual ~TPhParKey() {}  
  
    void  SetSource(Char_t *line)       { fSource    = line; }
    void  SetParName(Char_t *line)      { fParName   = line; }
    void  SetParValue(Char_t *line)     { fParValue  = line; }
    void  SetParValueType(Char_t type)  { fParValueType = type; }

    const Char_t   *GetSource()      { return fSource.Data();   }
    const Char_t   *GetName()        { return fParName.Data();  }
    Char_t    GetParValueType()      { return fParValueType;    }

    Bool_t  IsFrom(Char_t *line)     { return ! fSource.CompareTo(line,
                                        TString::kIgnoreCase);  }
    Bool_t  HasName(Char_t *line)    { return ! fParName.CompareTo(line,
                                        TString::kIgnoreCase);  }

    // Get the parameter text value field 
    Bool_t GetParTextValue(Char_t *value);

    // Attempt to read fParValueType numbers (1 if fParValueType=-1 - unknown) 
    // from the parameter text value field, returns true if the number 
    // of parameters found is in agreement with the fParValueType
    Bool_t GetParNumValue(Float_t *values, Int_t *numvalues);

    virtual void  ls();
    virtual void  Print();
    void  PrintNoKey();

  private:
    
    TString fSource;                      // Software source name
    TString fParName;                     // Parameter name
    TString fParValue;                    // Parameter value as text
    Char_t  fParValueType;                // 1-9 (numbers), 0 (string), -1 (unknown)

  public:

    ClassDef(TPhParKey,1)  // PHOBOS parameter storage class
};

#endif

