#ifndef __TPhRawHitFECs0__
#define __TPhRawHitFECs0__

/////////////////////////////////////////////////////////////////////////
//
// Implementation of the zero-suppressed data.
//
// This currently implements the following format (P. Sarin: Nov. 28, '98)
//
//
// |--CMN block--|-------------data-------------------|
//
// ---         -----            ---------       -----------  ---
// | 0 ....... 0 | 0 .......... 0 | 1 | 0 ..... 0 | 1 | 1 |   |
// -----------------            ---------       -----------  2-bit
// | 0 ....... 0 | 1 .......... 1 | 0 | 1 ..... 1 | 0 | 1 |   |
// ---         -----            ---------       -----------  ---
// |             |                |Str|           |Str|   |   |
// |             |                | # |           | # | F |   |
// |             |                |   |           |   | E |   |
// |  32x        |   Up to        |----  10bit    |---- C |   |
// |  11bit      |     8 Strings  | # |  Channel  | # |   |   |
// |   CMN Val   |                | o |   Number  | o | # |   |
// |             |                | f |           | f |---|   |
// |             |                |   |           |   |   |   |
// |             |                | W |           | W | I |   |
// |             |                | O |           | O | n |   |
// |             |-- .......... --| R |-- ..... --| R | f |   |
// |-- ....... --|                | D |           | D | o |  22-bit
// |             |                | S |           | S |   |   |
// |             |                |   |           |   | W |   |
// |             |                | t |           | t | o |   |
// |             |                | o |           | o | r |   |
// |  32x        |                |   | 12bit     |   | d |   |
// |  11bit      |                | f |  ADC      | f |   |   |
// |   CMN Value |                | o |   Value   | o | 1 |   |
// |             |                | l |           | l | 6 |   |
// |             |                | l |           | l | b |   |
// |             |                | o |           | o | i |   |
// |             |                | w |           | w | t |   |
// --- ....... ----- .......... --------- ..... -----------  ---
//
//                                               Start ---^
//
// Created by: Patrick Decowski
//
// $Id: TPhRawHitFECs0.h,v 1.3 1999/02/24 23:27:55 olszewsk Exp $
//
///////////////////////////////////////////////////////////////////////


// Root classes
#include "TObjArray.h"

// Phobos classes
#include "TPhAbsRawHitFEC.h"

const Int_t kCMNLength = 32; // len of common mode noise block in 24-bit words

class TPhRawHitFECs0 : public TPhAbsRawHitFEC {

//protected:
public:
  void                 Pack();
  void                 UnPack(const UChar_t *ZeroSupBuffer, 
			      const Int_t Length);
  Int_t                CalcTotal24Words() const; 

public:

  TPhRawHitFECs0();
  TPhRawHitFECs0(Int_t FECNum);
  virtual ~TPhRawHitFECs0();

  Int_t                 GetInfoWord() { return fInfoWord; }
  UInt_t*               GetCMNBlock() { return fCMN; }

// Debugging
  virtual void          PrintPacked(Option_t *option = "") const;

protected:

  UChar_t*     fZeroSupPacked;     // Packed 24-bit Zero Suppressed RawHits
  Int_t        fInfoWord;          // FIXME:info word in header?????
  UInt_t       fCMN[2*kCMNLength]; // Common mode noise block

public:

  ClassDef(TPhRawHitFECs0,1)	// ZERO suppressed FEC data storage
};


#endif // __TPhRawHitFECs0__
