#ifndef _PhTrack_INCLUDED_
#define _PhTrack_INCLUDED_

///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhTrack                                                       //
//                                                                   //
//    Author  : Gunther Roland (rolandg@vxcern.cern.ch)              //
//    Created : 16/01//96                                            //
//    Version : 1.0                                                  //
//    Changed : 30.09.1997 Andrzej Olszewski                         //
//                                                                   //
///////////////////////////////////////////////////////////////////////

// Root Classes

#include "TObject.h"
#include "TObjArray.h"
#include "TPolyLine3D.h"
#include "TClass.h"

// Phobos Classes

#include "TPhHit.h"
#include "TPhMomentumVector.h"


class TPhTrack : public TPhMomentumVector {

public:

    TPhTrack();

    virtual ~TPhTrack();

    virtual Float_t    GetXProd()   { return fXProd;   }
    virtual Float_t    GetYProd()   { return fYProd;   }
    virtual Float_t    GetZProd()   { return fZProd;   }

    virtual Float_t    GetEnergy();
    virtual Float_t    GetRap(Float_t mass);
    virtual Float_t    GetBeta();
    virtual Float_t    GetTheta();
    virtual Float_t    GetDip();


    virtual void       SetXProd(Float_t x) { fXProd = x;      }
    virtual void       SetYProd(Float_t x) { fYProd = x;      }
    virtual void       SetZProd(Float_t x) { fZProd = x;      }
    virtual void       SetMass(Float_t x)  { fMass = x;       }
    virtual void       SetGPid(Int_t n)    { fGPid = n;       }

    virtual void       AddHit(TPhHit *hit);
    virtual TObjArray *GetHitList()        { return fHitList;    }
    virtual Int_t      GetNHits();

    virtual Float_t    GetMass()           { return fMass;       }
    virtual Int_t      GetGPid()           { return fGPid;       }
    virtual TPhHit    *GetNextHit();
    virtual void       ResetHitCounter()   { fHitCounter = 0;    }
    virtual void       MakeHitList();
    virtual void       Draw(Option_t *Option = "");
    virtual void       ListHits();

    virtual void       ls(Option_t *option);
    virtual void       Print(Option_t *option);

protected:
     
    Float_t      fXProd;        //  X-origin of the track
    Float_t      fYProd;        //  Y-origin of the track
    Float_t      fZProd;        //  Z-origin of the track
    Int_t        fGPid;         //  Particle GEANT ID
    Float_t      fMass;         //  Particle mass
    TObjArray   *fHitList;      //  List of hits on the track
    Int_t        fHitCounter;   //! Counter for GetNextHit member function
    TPolyLine3D *fPolyLine;     //! PolyLine for track drawing

public:
    ClassDef(TPhTrack,2) // PHOBOS track class

};

#endif

