#include "TPhNotes.h"

ClassImp(TPhNotes)
/////////////////////////////////////////////////////////////////////////////
//                                                                         //
// TPhNotes is a class to store user notes on a ROOT TFile file.           //
//                                                                         //
// $Id: TPhNotes.cxx,v 1.3 1998/12/09 08:21:10 olszewsk Exp $
//
/////////////////////////////////////////////////////////////////////////////

TPhNotes::TPhNotes() 
{
  // Default constructor
  fcurrLine = NULL;
}

TPhNotes::TPhNotes(Text_t *name, Text_t *title) : TNamed(name,title) 
{
  // Another constructor
  fLines = new TList();
  fcurrLine = NULL;
}

TPhNotes::~TPhNotes()
{
  // Default destructor
  SafeDelete(fLines);
}

void  TPhNotes::AddLine(Char_t *line)
{
  // Convert a char string to a TObjString and add it to the list

  TObjString *str = new TObjString(line);
  fLines->AddLast(str);
  fcurrLine = str;

  return;
}

void  TPhNotes::AddLineAt(Char_t *line, Int_t pos)
{
  // Convert a char string to a TObjString and add it to the list at a given position

  TObjString *str = new TObjString(line);
  fLines->AddAt(str,pos);
  fcurrLine = str;

  return;
}

const Char_t *TPhNotes::GetFirstLine()
{
  // Get first line from the list 

  if(!fLines) return NULL;

  fcurrLine = (TObjString *) fLines->First();
  if(fcurrLine) 
    return fcurrLine->GetString().Data();
  return NULL;
}

const Char_t *TPhNotes::GetNextLine()
{
  // Get first line from the list 

  if(!fLines) return NULL;

  TObjString *oldcurr;
  oldcurr = fcurrLine;
  if(!fcurrLine) 
     fcurrLine = (TObjString *) fLines->First();
  else {
     if(fcurrLine == (TObjString *) fLines->Last()) return NULL;
     fcurrLine = (TObjString *) fLines->After(oldcurr);
  }

  return fcurrLine->GetString().Data();
}

void TPhNotes::Print()
{
  // Print the contents of the list of strings
  const Char_t *line = NULL;

  printf("TPhNotes %s object, Title: %s\n\n",GetName(), GetTitle());
  line = GetFirstLine();
  if(line) printf("%s\n",line);
  while(line = GetNextLine()) 
    printf("%s\n",line);
  return;
}
