#include "TPhRawHitFECn0TB98.h"
#include <string.h>

ClassImp(TPhRawHitFECn0TB98)
//////////////////////////////////////////////////////////////////////////
//
// This is a specialized implementation of the FEC Raw Hit buffer, 
// specifically designed for the 98 Test Beam. 
//
// $Id: TPhRawHitFECn0TB98.cxx,v 1.2 1998/11/20 23:49:19 decowski Exp $
//
//////////////////////////////////////////////////////////////////////////

TPhRawHitFECn0TB98::TPhRawHitFECn0TB98()
{
  // Default constructor

}

TPhRawHitFECn0TB98::TPhRawHitFECn0TB98(UChar_t *packedrawhits, Int_t totalbytes) 
{
  // Constructor called by routines that provide packed data

  Int_t i;
  for(i = 0; i < 8; i++)
    fRawHits[i] = NULL;
  
  if(totalbytes%2) {
    Error("TPhRawHitFECn0TB98","Wrong buffer size - has odd number of bytes\n");
    exit(0);
  }

  // copy info to local buffer
  fTotalWords = totalbytes/2;
  fRawHitsPacked = new UChar_t[totalbytes];
  memcpy(fRawHitsPacked,packedrawhits,totalbytes);

  // unpack data into arrays of FECRawHit_t structures
  UnPack();
}

TPhRawHitFECn0TB98::~TPhRawHitFECn0TB98()
{
  // Default destructor
}

UShort_t TPhRawHitFECn0TB98::GetEventNumber()
{
  // Get event number from the trailer structure

  Int_t event;
  event = (fTrailer[0] & 0x7fff) + ((fTrailer[1] & 0x7fff) << 15);

  return event;
}

Bool_t TPhRawHitFECn0TB98::RemoveString(UChar_t iString)
{
  // Remove the string from the fRawHits[8] table and free the memory

  // check if the source position points to some data at all
  if(!fRawHits[iString]) {
    Warning("RemoveString","Zero pointer at the %d input position, no change\n",
             iString);
    return kFALSE;
  }

  delete fRawHits[iString];
  fRawHits[iString] = NULL;
  fRawHitBufSize[iString] = 0;

  return kTRUE;
}

Bool_t TPhRawHitFECn0TB98::MoveString(UChar_t iString, UChar_t oString)
{
  // Change of the string number means that a pointer to the data structure
  // has to be moved from one position in the fRawHits[8] table to another one

  if(iString == oString) return kTRUE;

  // check if the source position points to some data at all
  if(!fRawHits[iString]) {
    Warning("MoveString","Zero pointer at the %d input position, no change\n",
             iString);
    return kFALSE;
  }
  // check if the destination position is available (NULL) at the moment
  if(fRawHits[oString]) {
    Warning("MoveString","Non zero pointer at the %d position, data will be lost\n",
             oString);
    delete [] fRawHits[oString]; 
  }

  fRawHits[oString] = fRawHits[iString];
  fRawHits[iString] = NULL;
  fRawHitBufSize[oString] = fRawHitBufSize[iString];
  fRawHitBufSize[iString] = 0;

  return kTRUE;
}

void TPhRawHitFECn0TB98::SetFECNo(UChar_t fECNo)
{
  // Set FEC number 
  // Needed for two reasons:
  // 1. Wrong FEC numbers are assigned in the Test Beam output
  // 2. Procedure Unpack() does not fill this data member

  fFECNumber = fECNo;
}

void TPhRawHitFECn0TB98::Streamer(TBuffer &R__b)
{
  // Stream an object of class TPhRawHitFECn0.

  TPhRawHitFECn0::Streamer(R__b);
}
