#ifndef _PhLayer_INCLUDED_
#define _PhLayer_INCLUDED_

///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhLayer                                                       //
//                                                                   //
//    PHOBOS detetctor layer class                                   //
//                                                                   //
//    TPhLayer manages information and operations related to the     //
//    sensors contained in one layer of silicon                      //
//                                                                   //
//    Author  : Gunther Roland (rolandg@vxcern.cern.ch)              //
//    Created : 15/01/96                                             //
//    Version : 1.0                                                  //
//    Changed :                                                      //
//                                                                   //
// $Id: TPhLayer.h,v 1.5 1999/02/25 00:39:11 olszewsk Exp $
//                                                                   //
///////////////////////////////////////////////////////////////////////

// ROOT classes

#include "TObject.h"
#include "TObjArray.h"

// Phobos classes

class TPhHit;

#include "TPhSensor.h"

class TPhLayer : public TObject {

public:

    TPhLayer();

    virtual ~TPhLayer();

    virtual Int_t      GetLabel()              { return fLabel;   }
    virtual Int_t      GetNSensors()           
      { return fSensorList->GetEntries();  }
    virtual TObjArray *GetSensorList()         { return fSensorList;   }
    virtual TPhSensor *GetSensor(Int_t i)      
      { return (TPhSensor *) fSensorList->At(i); }

    virtual void       SetLabel(Int_t i)       { fLabel = i;   }
    virtual void       AddSensor(TPhSensor *s);
    virtual void       AddHit(TPhHit *hit);

    virtual void       Draw(Option_t *option = "");
    virtual Int_t      MakeHitArraysOutOfHits(Char_t *HitConName, 
					      Char_t *HitArrayConName);
    virtual Int_t      MakeHitsOutOfHitArrays(Char_t *HitArrayConName, 
					      Char_t *HitConName);
    virtual void       DeleteHits();
    virtual void       ClearHits();

private:

    Int_t      fLabel;           //  Layer ID label
    TObjArray *fSensorList;      //  List of silicon sensors in layer

public:

    ClassDef(TPhLayer,1) // Geometrical layer of several sensors

};

#endif



