#ifndef _TPhSensor_
#define _TPhSensor_

///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhSensor                                                      //
//                                                                   //
//    Phobos Base Sensor class                                       //
//                                                                   //
//  Created 971219 by P. Decowski                                    //
//                                                                   //
// $Id: TPhSensor.h,v 1.5 1999/01/13 18:12:56 decowski Exp $                                                              //
//                                                                   //
///////////////////////////////////////////////////////////////////////

// Root Classes
#include "TNode.h"
#include "TObjArray.h"

// Phobos Classes
class TPhHit;
class TPhHitArray;
class TPhDetector;

class TPhSensor : public TNode {

public:
  TPhSensor();
  virtual ~TPhSensor();

// These are all the virtual functions to force a given behavior
  virtual Int_t       MakeHitArraysOutOfHits(Char_t *HitConName, 
                                              Char_t *HitArrayConName) = 0;

  virtual void        SetRows(Int_t Rows) = 0;
  virtual void        SetColumns(Int_t Columns) = 0;
  virtual Int_t       GetRows() = 0;
  virtual Int_t       GetColumns() = 0;

///// The implemented members start here
  virtual void SetRotMtx(Double_t theta1, Double_t phi1, Double_t theta2, 
			 Double_t phi2, Double_t theta3, Double_t phi3);
  virtual void        Draw(Option_t *Option = "");
  virtual void        Print();
  virtual void        AddHit(TPhHit *hit);
  virtual void        DeleteHits();
  virtual void        ClearHits();
  virtual TObjArray*  GetHitList()     { return fHitList;    } 
  
  virtual Int_t       GetLabel()       { return fLabel;      }
  virtual Int_t       GetLayer()       { return fLayer;      }
  virtual Int_t       GetWing()        { return fWing;       }
  virtual Int_t       GetVerbose()     { return fVerbose;  }

  virtual const Char_t *GetChLabel()     { return GetName();   } // Obsolete!
  
  virtual void       SetLabel(Int_t num)     { fLabel = num;     }
  virtual void       SetLayer(Int_t num)     { fLayer = num;     }
  virtual void       SetWing(Int_t num)      { fWing  = num;     }
  virtual void       SetVerbose(Int_t i)       { fVerbose = i;  }

  virtual void       SetChLabel(Char_t *chlabel) 
    { SetName(chlabel); } // Obsolete!
  
protected:

  Int_t        fLabel;          //  Sensor label       
  Int_t        fLayer;          //  Sensor layer number
  Int_t        fWing;           //  Sensor wing ID      
  Int_t        fVerbose;        //! Control debugging output
  TObjArray   *fHitList;        //! List of all hits on sensor
  TPhDetector *fDetector;       //! The detector object

public:

  ClassDef(TPhSensor,2)  // Base class for all Phobos sensor classes

};

#endif


