#ifndef __TPhStringCalAbs__
#define __TPhStringCalAbs__


///////////////////////////////////////////////////////////////////////////
//                                                                       //
//    TPhStringCalAbs                                                    //
//                                                                       //
//    Phobos class to store calibration data for a single string         //
//                                                                       //
//    TPhStringCalAbs is an abstract class for classes storing           //
//    calibration information for a single string.                      //
//                                                                       //
//    Author  : Andrzej Olszewski (andrzej.olszewski@ifj.edu.pl)         //
//    Created : 22/02/99                                                 //
//    Version : 1.0                                                      //
//    Changed :                                                          //
//                                                                       //
// $Id: TPhStringCalAbs.h,v 1.1 1999/02/24 22:43:20 olszewsk Exp $
//                                                                       //
///////////////////////////////////////////////////////////////////////////

// ROOT classes
#include "TObject.h"
#include "TArrayF.h"

// Phat classes
//#include "TPhString.h"

struct FECRawHit_t;
class TPhChannelCalAbs;
const Int_t kMaxStrLength = 768;      // The (maximum) length of one string

class TPhStringCalAbs : public TObject {

public:

  TPhStringCalAbs();
  TPhStringCalAbs(Int_t parno, Int_t parpointsno, Float_t boardlinearity,
		   Float_t *pars, Float_t *calpoints, Float_t *parerrors=NULL);
  virtual ~TPhStringCalAbs();

  virtual Bool_t Add(Int_t channo, TPhChannelCalAbs *chancal);
  virtual Bool_t Calibrate(FECRawHit_t *hitstring, Int_t strlength)=0;

  virtual void   ls(Option_t *option = "");
  virtual void   Print(Option_t *option = "");

  virtual void   SetVerbose(Int_t verbose)   { fVerbose = verbose;             }
  virtual Int_t  GetVerbose()                { return fVerbose;                }

protected:

  Float_t        fCalLinearity;           // Calibration board linearity
  Char_t         fParNo;                  // Number of calibration method parameters
  Char_t         fParPointsNo;            // Number of calibration points
  TArrayF        fPars[kMaxStrLength];       // Array of calibration parameters
  TArrayF        fParErrors[kMaxStrLength];  // Errors of calibration parameters
  TArrayF        fCalPoints;              // ADC points where calibration was done
  Int_t          fVerbose;                //!Verbose flag

public:

  ClassDef(TPhStringCalAbs,1)	// Calibrate ADC for each channel
};


#endif // __TPhStringCalAbs__
