#include "math.h"
#include "TPhEDigi.h"

ClassImp(TPhEDigi)

////////////////////////////////////////////////////////////////
//                                                            //
// The TPhEDigi class provides functions for conversion       // 
// from the energy deposited in the silicon to ADC values     //
// and from ADC values to the deposited energy                //
//                                                            //
// $Id: TPhEDigi.cxx,v 1.3 1998/07/01 21:38:28 decowski Exp $
//
////////////////////////////////////////////////////////////////

TPhEDigi::TPhEDigi()
{
// Constructor. 
// Sets the parameters describing the conversion of analog signal 
// to default values
//
// Energy has to be given in GeV

  fFullADCRange = 4096; // 2^12 ADC counts
  fFullERange = 80. * 0.000130; // 80 Mips @ 130keV per Mip
  fPedestal = 0;
}

TPhEDigi::~TPhEDigi()
{
// Default destructor.
// no action required
}

void TPhEDigi::Print()
{
// Listing of data members
   printf("TPEDigi provides conversion of analog to digital signal\n");
   printf("Full ADC range: %d, Pedestal: %d\n", fFullADCRange, fPedestal);
   printf("Full E range: %f\n", fFullERange);
}


Short_t TPhEDigi::EtoADC(Double_t energy)
{
// translate energy to digital ADC signal 

  Short_t ADC;

  ADC = Short_t((energy/fFullERange)*fFullADCRange + fPedestal);
  return ADC;  
}

Double_t TPhEDigi::ADCtoE(Short_t ADC)
{
// Translate digital ADC signal to energy

  Double_t Energy;
  Energy = (Float_t(ADC - fPedestal)/fFullADCRange) * fFullERange;
  return Energy;
}


