#include <iostream.h>
#include <iomanip.h>
#include "TClass.h"
#include "TPhStringCalAbs.h"
#include "TPhString.h"

ClassImp(TPhStringCalAbs)

/////////////////////////////////////////////////////////////////////////////
//                                                                         //
// Methods to calibrate ADC signals in each separate string.               //
//                                                                         //
// $Id: TPhStringCalAbs.cxx,v 1.2 1999/03/05 12:10:18 olszewsk Exp $
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

TPhStringCalAbs::TPhStringCalAbs()
{
  // Default constructor
  Int_t i;

  fParNo = 0;
  fParPointsNo = 0;
  fCalLinearity = 1.0;
  fCalPoints.Set(0);      
  for(i=0; i<kMaxLength; i++) {
    fPars[i].Set(0); 
    fParErrors[i].Set(0);            
  }
}  

TPhStringCalAbs::TPhStringCalAbs(Int_t parno, Int_t parpointsno, Float_t boardlinearity,
				   Float_t *pars, Float_t *calpoints, Float_t *parerrors)
{
  // Normal constructor
  Int_t i;

  fParNo = parno;
  fParPointsNo = parpointsno;
  fCalLinearity = boardlinearity;
  fCalPoints.Set(parpointsno, calpoints); 
  for(i=0; i<kMaxLength; i++) {
    fPars[i].Set(parno, pars+i*parno); 
    if(parerrors) fParErrors[i].Set(parno,parerrors+i*parno);
    else          fParErrors[i].Set(0); 
  }
}  

TPhStringCalAbs::~TPhStringCalAbs()
{
  // Default destructor
}  

Bool_t TPhStringCalAbs::Add(Int_t channo, TPhChannelCalAbs *chancal)
{
  //
  Bool_t status = kTRUE;
  Int_t arrsize;
  Float_t *arrf;

  if( !(fParNo || fParPointsNo) ) {
    fParNo      = chancal->GetParNo();
    fParPointsNo  = chancal->GetParPointsNo();
    fCalLinearity = chancal->GetCalLinearity();
    arrsize = chancal->GetCalPoints()->GetSize();
    if(arrsize) {
      arrf = chancal->GetCalPoints()->GetArray();
      fCalPoints.Set(arrsize, arrf); 
    }
  }
  arrsize = chancal->GetPars()->GetSize();
  if(arrsize) {
    arrf = chancal->GetPars()->GetArray();
    fPars[channo].Set(arrsize, arrf); 
  } else 
    return kFALSE;
  arrsize = chancal->GetParErrors()->GetSize();
  if(arrsize) {
    arrf = chancal->GetParErrors()->GetArray();
    fParErrors[channo].Set(arrsize, arrf); 
  } else 
    fParErrors[channo].Set(0); 

  return status;
}
  
void  TPhStringCalAbs::ls(Option_t *option) 
{
  //
    Int_t i, j;
    IndentLevel();
    cout << "Class: " << IsA()->GetName() << endl << " Calibration points:   ";
    for(i=0; i<fParPointsNo; i++) 
      cout << fCalPoints[i] << "  ";
    cout << endl << " Parameters: " << endl;
    for(j=0; j<kMaxLength; j++) {
      cout << "Channel:" << setw(5) << j << " Pars: "; 
      for(i=0; i<fParNo; i++) 
	cout << setw(10) << fPars[j][i] << "  ";
      cout << endl;
    }
    TObject::IncreaseDirLevel();
}

void  TPhStringCalAbs::Print(Option_t *option) 
{
  //
  ls("");
}
