{ 
// plotmul.C: Phat macro to make standard multiplicity display
// Use: after TPhMul has processed one or more events,
//   .x plotmul.C
// and follow prompts.
// 30-Jan-99: Update for new TPhMulDisp methods (RV)
// $id$
//////////////////////////////////////////////////////////

// 05-Jan-99: Update for new TPhMulDisp methods (RV)
// 25-Dec-98: Remove enums to prevent CInt crashes; add help and numerics
// 21-Nov-98: Shift display order for use with data
// 11-Oct-98: Complete revision using TPhMulDisp class (Robin Verdier)
  Int_t MaxReply = 96;
  Int_t MaxDisplays = 12; // max # saved on page
  Int_t frameBits = 15; // bit mask for frames to be plotted
  Int_t frames[4];
  Int_t reportLevel = 1;
  Char_t *macroName = "plotmul.C";
  Char_t reply[MaxReply+1];
  Int_t
    Black = 1, Red    = 2, Green = 3, Blue  =  4, Yellow =  5, Magenta =  6,
    Cyan  = 7, Green2 = 8, Blue2 = 9, White = 10, Gray   = 11, Black2  = 12;
// Set colors and marker styles for reconstructed, generated, and global 
// average values:
  Int_t recColor = Red, genColor = Blue, avgColor = Green2;
  Int_t recStyle = 21,  genStyle = 22,   avgStyle = 25;

  TPhMul &mul = mp;
  if (!mul)
  { printf ("Please execute runmul.C or equivalent script to create "
      "TPhMul mp.\n");
    exit;
  }

  Char_t sel[MaxReply+1];
  strncpy (sel, "*+in", MaxReply);
  Int_t mType = 0; // 0 for physics display, 1 for sensor map
  Int_t dType = 0; // 0 for TGraphErrors, 1 for TH2F histogram
  Bool_t exit = kFALSE;

  while (!exit)

// Ask what map set to use...
  { Char_t mTypeId = (!mType)? 'p' : 's';
    printf ("Type of display (p = physics, s = sensor, e = exit"
      " %s) [CR = %c]? ", macroName, mTypeId);
    if (fgets(reply, MaxReply, stdin))
      if (reply[0] == 'e' || reply[0] == 'E') exit = kTRUE;
      else if (reply[0] != '\n') mType = (reply[0] != 's')? 0 : 1;
    if (exit) break;

// type of display...
    Char_t dTypeId = (!dType)? 'g' : 'h';
    printf ("Type of plot (g = graph, h = histogram [CR = %c]? ", dTypeId);
    if (fgets(reply, MaxReply, stdin))
      if (reply[0] != '\n') dType = (reply[0] != 'h')? 0 : 1;

// Sensor map or display name or number
    Char_t *help = "l";
    strcpy (reply, help);
    while (!strcmp (reply, help))
    { printf ("Name or number of display, %s to list all [CR = %s]? ",
        help, sel);
      if (fgets(reply, MaxReply, stdin))
      { reply[strlen(reply)-1] = 0;
        if (reply[0])
        { if (!strcmp(reply,help))
            if (!dType) mp.ListDisplays();
            else mp.ListSensors();
          else strcpy(sel, reply);
        }    
      }    
    }
// Look up name if reply was a number-- soon the number will be used directly
    Int_t iDisp = 0;
    Int_t nn;
    for (nn = 0; nn < strlen(sel); nn++)
      if (sel[nn] < '0' || sel[nn] > '9') iDisp = -1;
    if (!iDisp) sscanf (sel, "%d", &iDisp);
//    if (iDisp >= 0) strcpy (sel, mp.GetDisplay(iDisp)->GetName()); // Temp **

// Ask what variables to display
    printf ("Frames to display (add 1 for occ, 2 for edep, 4 for mult, 8 "
      "for fsec [%d]? ", frameBits);
    if (fgets(reply, MaxReply, stdin))
      if (reply[0] != '\n') sscanf (reply, "%d", &frameBits);
    Int_t nFrames = 0;
    for (Int_t nn = 0; nn < 4; nn++)
      if (frames[nn] = frameBits & pow (2, nn)) nFrames++;

    TPhMulDisp *d[MaxDisplays];
    TCanvas *c1 = new TCanvas("c1","c1",0,0,600,600);
    if (nFrames >= 3) c1->Divide(2,2);
    if (nFrames == 2) c1->Divide(1,2);

    Int_t id = -1;
    Int_t iFrame = 0;
// We need to locate frames sequentially ***

// Upper left pad: occupancy
    Char_t *GEta = "Eta"; // "`h#"; ROOT displays this as-- guess? `h#
    if (frames[iFrame++]) 
    { c1->cd(iFrame); // 1 for frame 0, ...
      if (reportLevel >= 3) printf ("Drawing frame %d\n", iFrame);
      d[++id] = new TPhMulDisp(mul, sel, iDisp, 11, 12, 1, dType, mType,
        GEta, "Tracks/pad", "Reconstructed");
      if (!d[id]) printf ("%s: no display named \'%s\' found.\n", macroName,
        sel);
      if (!d[id]) return;
      d[id]->fMColor = recColor;
      d[id]->fMStyle = recStyle;
      d[id]->Draw();
      d[id]->fMColor = genColor;
      d[id]->fMStyle = genStyle;
      d[id]->Draw(21, 22, 1, "Generated");
    }

// Upper right pad: Energy deposited per track
      if (frames[iFrame++]) 
      { c1->cd(iFrame);
      if (reportLevel >= 3) printf ("Drawing frame %d\n", iFrame);
      d[++id] = new TPhMulDisp(mul, sel, iDisp, 26, -1, 1, dType, mType,
        "Eta", "Energy/track, GeV", "Global fit");
      d[id]->fMColor = avgColor;
      d[id]->fMStyle = avgStyle;
      d[id]->Draw();
      d[id]->fMColor = genColor;
      d[id]->fMStyle = genStyle;
      d[id]->Draw(15, 16, 1, "Generated");
    }

// Lower left pad: charged multiplicity
    if (frames[iFrame++]) 
    { c1->cd(iFrame);
      if (reportLevel >= 3) printf ("Drawing frame %d\n", iFrame);
      d[++id] = new TPhMulDisp(mul, sel, iDisp, 13, 14, 360, dType, mType,
        "Eta", "dNch/dEta", "Reconstructed");
      d[id]->fMColor = recColor;
      d[id]->fMStyle = recStyle;
      d[id]->Draw();
      d[id]->fMColor = genColor;
      d[id]->fMStyle = genStyle;
      d[id]->Draw(23, 24, 360, "Generated");
    }

// Lower right pad: Total / primary tracks
    if (frames[iFrame++]) 
    { c1->cd(iFrame);
      if (reportLevel >= 3) printf ("Drawing frame %d\n", iFrame);
      d[++id] = new TPhMulDisp(mul, sel, iDisp, 27, -1, 1, dType, mType,
        "Eta", "Total/primary tracks", "Global fit");
      d[id]->fMColor = avgColor;
      d[id]->fMStyle = avgStyle;
      d[id]->Draw();
      d[id]->fMColor = genColor;
      d[id]->fMStyle = genStyle;
      d[id]->Draw(19, 20, 1, "Generated");
    }
  }
}  
