{ // writemaps.C: Phat macro to write TPhMapSets
// Use: after executing runmul.C to fill TPhMul mp with data,
//  .x writemaps.C
// 31-Jan-99 First version, creates TPhMapSets from TPhMul (RV)
// Status: sensor maps report none active; get segv when rerun

  TPhMul &mu = mp;
  Int_t   maxReply = 128;
  Int_t   maxMName = 20;
  Char_t  outFile[maxReply]; // output .root file
  Char_t  dispMapName[maxMName]; // name of display MapSet object in file
  Char_t  sensMapName[maxMName]; // name of sensor MapSet object in file

  strncpy (outFile, "hij50maps.root", maxReply);
  strncpy (sensMapName, "sensors", maxMName);
  strncpy (dispMapName, "displays", maxMName);

  Int_t   nDispMaps = 0;
  Int_t   nSensMaps = 0;
  Char_t *openOpt = "recreate"; // new or create, recreate, update, read

  Int_t   iPrint = 0;
  Int_t   nErrs = 0;
  Int_t   reportLevel = 3;
  Int_t   nPrint = 10;
  Char_t  reply[maxReply+1];
  Bool_t  interact = kTRUE;

// Set options interactively

  if (interact)
  { printf ("Output file name (cr = %s)? ", outFile);
    if (fgets(reply, maxReply, stdin))
    { if (reply[0] != '\n')
      { strncpy(outFile, reply, maxReply);
        outFile[strlen(reply)-1] = 0;
      }        
    }
    printf ("Display mapset object name (cr = %s)? ", dispMapName);
    if (fgets(reply, maxMName, stdin))
    { if (reply[0] != '\n')
      { strncpy(dispMapName, reply, maxMName);
        dispMapName[strlen(reply)-1] = 0;
      }        
    }
    printf ("Sensor mapset object name (cr = %s)? ", sensMapName);
    if (fgets(reply, maxMName, stdin))
    { if (reply[0] != '\n')
      { strncpy(sensMapName, reply, maxMName);
        sensMapName[strlen(reply)-1] = 0;
      }        
    }
  } // if (interact

// Create display and sensor MapSets

  TPhMapSet *dms1 = new TPhMapSet();
  if (dms1)
  { nDispMaps = dms1->GetNMapsAct();
    if (reportLevel >= 3)
      printf ("writemaps.C: found %d active display maps.\n", nDispMaps);
  }
  else
    if (reportLevel > 0)
      printf ("writemaps.C: unable to create display MapSet.\n");

  TPhMapSet *sms1 = new TPhMapSet(1);
  if (sms1)
  { nSensMaps = sms1->GetNMapsAct();
    if (reportLevel >= 3)
      printf ("writemaps.C: found %d active sensor maps.\n", nSensMaps);
// Kludge until we find out why there are no sensor maps
    nSensMaps = sms1->GetNMaps();
    if (reportLevel >= 3)
      printf ("writemaps.C: found %d active sensor maps.\n", nSensMaps);
  }
  else
    if (reportLevel > 0)
      printf ("writemaps.C: unable to create sensor MapSet.\n");

// Copy TPhPadMaps from TphMul to TPhMapSet

  TPhPadMap *pm;
// CInt will fail if we call iEr1 iErr!
  if (dms1)
    for (Int_t imap = 0; imap < nDispMaps; imap++)
    { pm = mu.GetDisplay(imap);
      Int_t iEr1 = dms1->Replace(pm);
      if (iEr1) nErrs++;
      if (reportLevel >= 3 && (iPrint < nPrint || imap == nDispMaps-1))
      { printf ("writemaps.C: replaced %s, error = %d.\n",pm->GetName(), iEr1);
        iPrint++;
      }
    }

  if (sms1)
    for (Int_t imap = 0; imap < nSensMaps; imap++)
    { pm = mu.GetSensMap(imap);
      Int_t iEr1 = sms1->Replace(pm);
      if (iEr1) nErrs++;
      if (reportLevel >= 3 && (iPrint < nPrint || imap == nDispMaps-1))
      { printf ("writemaps.C: replaced %s, error = %d.\n",pm->GetName(), iEr1);
        iPrint++;
      }
    }

  if (reportLevel >= 1 ) // && nErrs)
    printf ("writemaps.C: %d errors replacing TPhPadMaps.\n", nErrs);

// Write TPhMapSet into output file

  TFile ff(outFile, openOpt);

  if (dms1) dms1->Write(dispMapName);
  if (sms1) sms1->Write(sensMapName);

  if (reportLevel >= 2) printf ("writemaps.C: wrote %s containing %d maps.\n",
    outFile, nDispMaps + nSensMaps);

// Write output file screws things up, gives
// mmuzip: error in header


//  ff.Write();

  ff.Close();

}
