{
// writemul.C: Phat macro to write Multiplicity objects
// Use: assume that a TPhMul has been instantiated and an output file
// is open, and necessary variables are defined by statements such as
// the following, for MDC use:
/* 
// Additions for MDC2, from Mul.h, R.Verdier, 9-Feb-99.
TPhMul     mp;             // The Mul object
Char_t    *mMacroName = "$PHATHOME/phatmul/macros/writemul.C";
Int_t      dataSource = 2; // 0 = data, 1 = MC data, 2 = MCHits;
// The following names are used by writemul.C and must be changed here,
// and in writemul.C, if they conflict with other names.
TPhMapSet *dms1            // Display mapset pointer
TPhMapSet *sms1            // Sensor mapset pointer
// Bits set in mOpts to write histos, multi and single event maps:
Int_t      EnCh = 1, Ehists = 2, EdispNEv = 4, Edisp1Ev = 8, Esens1Ev = 16;
Int_t      mOpts = 31;     // 1 + 2 + 4 + 8 + 16;
Int_t     @mEvIndex = iev; // Event-loop index for identifying objects
TFile     *outFile = (TFile *)0; // If non-null, specified objects will be
//                                  streamed directly to this file.
// Reconstructed and generated total charged multiplicity
Float_t    nChRec = 0;     // Reconstructed from *+in 
Float_t    nChRecRMS = 0;  // RMS error in ""
Float_t    nChGen = 0;     // Generated (if MC tracks and hits used)
Float_t    nChGenRMS = 0;  // RMS error in ""
// 2-d histograms.  TypeNames must begin with 's' to specify a SensMap;
// anything else means a display
Int_t      nMHists = 2;    // Number of 2-d histograms filled on each event
TH2F      *mHists[nMHists] = {0, 0};// Array of pointers to histograms
Char_t    *histSelect[2]  = {"*+hr-1", "*+in-1"};
Char_t    *hTypeNames[2]  = {     "h",      "h"};
// 1-d plots
Int_t      nMGraphs = 2;   // Number of 1-d plots filled on each event
TGraphErrors *mGraphs[nMGraphs] = {0, 0}; // Array of pointers to plots
Char_t    *graphSelect[2] = {"*+hr", "*+in"};
Char_t    *gTypeNames[2]  = { "g",      "g"};
// End of MDC2 interface ////////////////////////////////////////////
*/

// After reconstructing one or more events, the statement
//   gROOT->Macro("writemul.C");
// will write objects specified by mOpts.
// Status: This should be a macro with arguments, to eliminate possible
// conflicts with other names in the common CInt namespace.
// $Id: writemul.C,v 1.7 1999/02/13 23:16:17 verdier Exp $
// 13-Feb-99 Use MDC-2 interfaces defined in Mul.h (RV)
// 07-Feb-99 Creation from writemaps.C; creates TPhMapSets from TPhMul (RV)

  TPhMul &mu = mp;
  Int_t   mErr = 0;
  if (!mu)
    return mErr = -1;

  else
  { Int_t   maxMName = 8;
    Char_t  objName[maxMName]; // name prefix for objects in file
    sprintf (objName, "e%d", mEvIndex);
    Char_t  mapName[maxMName+8]; // full name of object in file
    Int_t   nDispMaps = 0;
    Int_t   nMapsUsed = 0;
    Int_t   nSensMaps = 0;
    Int_t   iPrint = 0;
    Int_t   nErrs = 0;
    Int_t   opts = mOpts;
    Int_t   reportLevel = 3; // Fatal, warnings, summary
    Int_t   nPrint = 30;
// TPhPadMap variable indices
    Int_t   idNdE      = 13; // iVar for reconstructed d2N/dEdPhi in map
    Int_t   idNdERMS   = 14; // iVar for assigned error in ""
    Int_t   idNdEMC    = 23; // iVar for generated d2N/dEdPhi in map
    Int_t   idNdEMCRMS = 24; // iVar for assigned error in ""
    Float_t scale      = 360.; // Scale for integrating on phi

    if (opts && outFile) outFile->cd();

    if (opts && outFile && reportLevel >= 4 && iPrint < nPrint)
    { printf ("writemul.C processing event %d for opts = %d\n",
        mEvIndex, opts);
      iPrint++;
    }

// Get the integral multiplicity, reconstructed and generated (if available)
// Clear stats in case for some reason the padmap isn't available

    if (opts & EnCh)
    { nChRec = 0;
      nChRecRMS = 0;
      nChGen = 0;
      nChGenRMS = 0;
      Int_t iInteg     = 0; // location of integral in stats array
      Int_t iIntegRMS  = 7; // location of RMS error assigned to integral
      Float_t stats[20];
      TPhPadMap *mw1 = mp.GetDisplay("*+in");
      if (mw1)
      { mw1->GetStats(idNdE, idNdERMS, 10, stats);
        nChRec = stats[iInteg];
        nChRecRMS = stats[iIntegRMS];
        mw1->GetStats(idNdEMC, idNdEMCRMS, 10, stats);
        nChGen = stats[iInteg];
        nChGenRMS = stats[iIntegRMS];
      }
      if (reportLevel >= 3 && iPrint < nPrint)
      { printf ("writemul.C: Reconstructed nCh =%7.1f +-%5.1f,"
          " generated =%7.1f +-%5.1f.\n",
          nChRec, nChRecRMS, nChGen, nChGenRMS);
        iPrint++;
      }
    }

// Extract graphs and histograms; write them in outFile if open

    if (opts & Ehists)
    { for (Int_t ih = 0; ih < nMHists; ih++)
      { if (mHists[ih]) delete mHists[ih];
        mHists[ih] = mu.GetHist(hTypeNames[ih], histSelect[ih],
          idNdE, idNdERMS, scale);
        if (mHists[ih])
        { sprintf (mapName, "%s%s%s", objName, hTypeNames[ih],
            histSelect[ih]);
          if (outFile) mHists[ih]->Write(mapName);
        }
        else
          mErr = -2;
      } // for (ih

      for (Int_t ig = 0; ig < nMGraphs; ig++)
      { if (mGraphs[ig]) delete mGraphs[ig];
        mGraphs[ig] = mu.GetGraph(gTypeNames[ig], graphSelect[ig],
          idNdE, idNdERMS, scale);
        if (mGraphs[ig])
        { sprintf (mapName, "%s%s%s", objName, gTypeNames[ig], 
            graphSelect[ig]);
          if (outFile) mGraphs[ig]->Write(mapName);
        }
        else
          mErr = -4;
      } // for (ig
    }

// Create display MapSet only once.
// For the MDC-2, valid TPhMapSets must be present, even if not filled,
// for higher-level decisions whether to write the ROOT tree.
 
    Bool_t createDMS = !dms1;
    if (createDMS) dms1 = new TPhMapSet();
    if ((opts & (EdispNEv | Edisp1Ev)) && createDMS)
    { if (dms1)
      { nDispMaps = dms1->GetNMapsAct();
        if (reportLevel >= 5 && iPrint < nPrint)
        { printf ("writemul.C: found %d active display maps.\n", nDispMaps);
          iPrint++;
        }
      }
      else
      { mErr = mErr || 1;
        if (reportLevel > 0 && iPrint < nPrint)
        { printf ("writemul.C: unable to create display MapSet.\n");
          iPrint++;
        }
      }
    } // if ((opts

// Create sensor MapSet once-only
  
    Bool_t createSMS = !sms1;
    if (createSMS) sms1 = new TPhMapSet(1);
    if ((opts & Esens1Ev) && createSMS)
    { if (sms1)
      { nSensMaps = sms1->GetNMapsAct();
        if (reportLevel >= 5 && iPrint < nPrint)
        { printf ("writemul.C: found %d active sensor maps.\n", nSensMaps);
          iPrint++;
        }
      }
      else
      { mErr = mErr || 2;
        if (reportLevel > 0 && iPrint < nPrint)
        { printf ("writemul.C: unable to create sensor MapSet.\n");
          iPrint++;
        }
      }
    } // if ((opts

    if (!mErr && opts)

// Once only, copy TPhPadMaps from TphMul to TPhMapSets.  The Replace
// function presently only copies pointers, not data, and they must be 
// preserved until their creator, TPMul, deletes them.

    { TPhPadMap *pm;
      if (dms1 && createDMS && (opts & (EdispNEv | Edisp1Ev)))
      { for (Int_t imap = 0; imap < nDispMaps; imap++)
        { pm = mu.GetDisplay(imap);
          if (pm)
// CInt will fail if we call iEr1 iErr!
          { Int_t iEr1 = 0;
// Enable only maps selected by opts
            if  (((opts & Edisp1Ev) && pm->GetId() == 1)
              || ((opts & EdispNEv) && pm->GetId() == -1))
              iEr1 = dms1->Replace(pm);
            if (iEr1) nErrs++;
            if (reportLevel >= 6 && (iPrint < nPrint || imap == nDispMaps-1))
            { printf ("writemul.C: replaced %s, error = %d.\n",
                pm->GetName(), iEr1);
              iPrint++;
            }
            nMapsUsed++;
          }
          else
          { pm = dms1->GetMap(imap);
            if (pm) pm->SetNCells (1, 1, 0);
          }
        } // for (imap
      } // if (dms1

// Sensor selection should also be by nEvents

      if (sms1 && createSMS && (opts & Esens1Ev))
      { for (Int_t imap = 0; imap < nSensMaps; imap++)
        { pm = mu.GetSensMap(imap);
          if (pm)
          { Int_t iEr1 = 0;
            if ((opts & Esens1Ev) && pm->GetId() == 1)
              iEr1 = sms1->Replace(pm);
            if (iEr1) nErrs++;
            if (reportLevel >= 6 && (iPrint < nPrint || imap == nDispMaps-1))
            { printf ("writemul.C: replaced %s, error = %d.\n",
                pm->GetName(), iEr1);
              iPrint++;
            }
            nMapsUsed++;
          }
          else
          { pm = sms1->GetMap(imap);
            if (pm) pm->SetNCells (1, 1, 0);
          }
        } // for (imap
      } // if (sms1

    } // if (!mErr

    if (reportLevel >= 1 && nErrs && iPrint < nPrint)
    { printf ("writemul.C: %d errors replacing TPhPadMaps.\n", nErrs);
      iPrint++;
    }

// Write TPhMapSets into output file

// Clear tallies
    nSensMaps = 0;
    nDispMaps = 0;
    nMapsUsed = 0;

    if (dms1 && (opts & (Edisp1Ev | EdispNEv)))
    { sprintf (mapName, "%sdisp", objName);
      if (reportLevel >= 4 && iPrint < nPrint)
      { printf ("writemul.C: Write %d / %d active displays.\n", 
          dms1->GetNMapsAct(), dms1->GetNMaps());
        iPrint++;
      }
      if (outFile) dms1->Write(mapName);
      nDispMaps = dms1->GetNMaps();
      nMapsUsed += dms1->GetNMapsAct();
    }
    if (sms1 && (opts & Esens1Ev))
    { sprintf (mapName, "%ssens", objName);
      if (reportLevel >= 4 && iPrint < nPrint)
      { printf ("writemul.C: Write %d / %d active sensMaps.\n",
          sms1->GetNMapsAct(), sms1->GetNMaps());
        iPrint++;
      }
      if (outFile) sms1->Write(mapName);
      nSensMaps = sms1->GetNMaps();
      nMapsUsed += sms1->GetNMapsAct();
    }

    if (reportLevel >= 3 && (opts & (Esens1Ev | Edisp1Ev | EdispNEv))
      && outFile && iPrint < nPrint) 
    { printf ("writemul.C wrote %d / %d maps into %s.\n",
        nMapsUsed, nDispMaps + nSensMaps, outFile->GetName());
      iPrint++;
    }

    return mErr;

  } // end of writemul.C
}
