#include "TObject.h"
#include "TPhOpts.h"

ClassImp(TPhOpts)
////////////////////////////////////////////////////////////////////////////
//
// TPhOpts is a Phobos class that provides access to ASCII files 
// containing embedded numerical fields specifying analysis options,
// such as TPhDeptV.h, a file containing default values for the single-track
// energy deposition parameters for class TPhDept in the form
/*
...
  104.3e-6, -6.0, 111.7e-6, -4.0, 100.0e-6, 6.0, 100.0e-6, 9.0, // MNC2
  120.0e-6,  9.9, 120.0e-6,  9.9, 120.0e-6, 9.9,                // MNC2
...
*/
// Class methods read such a file and return successive data fields  
// on repeated calls, optionally writing a new version of the file
// with user-modified values of the data fields.
// Data fields are character arrays or numeric, with field separators 
// and selectors defined in key strings specified by the user.
//
// File: TPhOpts.cxx
//
// Date     Vers  Description
// 16-Dec-98 1.0  Minor changes to eliminate compiler warnings (RV)
//
// $Id: TPhOpts.cxx,v 1.4 1998/12/21 21:34:58 verdier Exp $
//
////////////////////////////////////////////////////////////////////////////

// 15-Aug-98 1.0  Change default values filename from .v to V.h (RV)
// 04-Jun-98 1.0  Changed Char_t *cres = ... syntax for OSF1 (RV)
// 20-May-98 0.2  Move output write to precede newline read (RV)
// 28-Apr-98 0.2  Add GetField, GetLine, GetFloat, GetText, ... (RV)
// 28-Mar-98 0.1  Creation (Robin Verdier)

// Variables requiring initialization.  These should be moved into the
// constructor, but there are problems declaring Char_t *xxx[] there;
// if it's Char_t **, how does the initialization work?

Char_t  *fBufc;         //! Pointer to current position in buffer
Char_t  *&fBufr = fBufc;// Reference for returning updated pointer
                        // Field selectors for use with existing TPhxxxV.h 
                        // files.  THE TERMINAL ';' IS REQUIRED:
Char_t  *fDecKeys[]   = { "0-9", "+", "-", "." }; // Start of dec. num.
Char_t  *fDecEnd[]    = { ",", " ", "}", "]" }; // End of decimal number
                        // fDecEnd does not contain the comment indicator 
                        // '//', newline '\n', or end-of-string null "",
                        // assuming they have already been located and 
                        // used to set the buffer limit for efficiency;
                        // otherwise, they should be added to the decEnd keys.
Char_t **fEndKeys     = fDecEnd; // Current field end keys
Char_t  *fEOLNumKeysD[]= {"\n","//"}; // Default end-of-line keys for num flds
Char_t  *fEOLTextKeysD[]= {"\n"}; // Default end-of-line keys for text fields
Char_t  *fEx          = { "/home/verdier/mul/inc/TPhDeptV.h" }; // Example file
                        // Keys for multiplicity sensor names presently
                        // in comment fields:
Int_t    fGetFieldReq[2] = { eTextBit, 0 }; // Require masks for new field
Int_t    fGetFieldSel[2] = { eTextBit, eTextBit }; // Select masks for new fld
Int_t    fGetLineReq  = eTextBit + eNumbBit; // Require mask for new line
Int_t    fGetLineSel  = eTextBit + eNumbBit; // Selection mask for new line
Int_t    fNDecKeys    = { sizeof(fDecKeys) / sizeof(Char_t *) };
Int_t    fNEOLNumKeysD= { sizeof(fEOLNumKeysD) / sizeof(Char_t *)};//# EOL keys
Int_t    fNEOLTextKeysD={ sizeof(fEOLTextKeysD) / sizeof(Char_t *)};
Int_t    fNStartKeys  = { fNDecKeys };
Int_t    fNEndKeys    = { sizeof(fDecEnd) / sizeof(Char_t *) };
Char_t  *fSensKeys[]  = { "// MO", "// MP", "// MN", "// VA", "// VB" };
Char_t **fStartKeys   = fDecKeys; // Current field start keys
Char_t **fTextKeys    = fSensKeys; // Current text search keys
// One-pass compiler...
Char_t **fEOLNumKeys  = fEOLNumKeysD; // Current end-of-line keys for numbers
Int_t    fNEOLNumKeys = fNEOLNumKeysD; // Current # end-of-line keys for numb
Char_t **fEOLTextKeys = fEOLTextKeysD; // Current end-of-line keys for text
Int_t    fNEOLTextKeys= fNEOLTextKeysD; // Current # end-of-line keys for text
Int_t    fNSensKeys   = { sizeof(fSensKeys) / sizeof(Char_t *) };
Int_t    fNTextKeys   = fNSensKeys;

// PHOBOS Classes

TPhOpts::TPhOpts ()
{
// TO BE MOVED TO HELP METHOD
/* if (ReportLevel)
    printf ("TPhOpts::TPhOpts() invalid call; the correct syntax is \n"
    "TPhOpts op(Char_t *inFile, [Char_t *outFile = \"\"], [Int_t type = 0],\n"
    "         [Int_t maxChars = 96])\n");
    printf ("Filenames must be enclosed in quotes [\"name\"].\n");
*/
}

TPhOpts::TPhOpts (const Char_t *inFile, const Char_t *outFile,
                  const Int_t repLev, const Int_t maxChars)
{ ReportLevel = repLev;
// Open input file if specified in instantiation call.
  if (strlen(inFile) > 0) Open(inFile);
// Open optional output file:
  if (strlen(outFile) > 0) Open(outFile, "w");
  else
    fOut = 0;

// Allocate the line buffer and initialize fields not already set by Open:
  fBufp = new Char_t[maxChars+2]; // space for possible exponent conversion
  fBufp[maxChars] = 0; // fence to end too-long strings
  fLinesPerCall = 9999999; // May take this as -1
  fFlags[eText] = 1; // Default mode: one text key per line
  fMaxChars = maxChars;
  fEOLNumKeys  = fEOLNumKeysD;
  fNEOLNumKeys = fNEOLNumKeysD;
  fEOLTextKeys  = fEOLTextKeysD;
  fNEOLTextKeys = fNEOLTextKeysD;
// Public class variables for debugging
  iprint = 0;
  Nprint = 100;
}

TPhOpts::~TPhOpts() 
{ delete[] fBufp; // MAKES CINT CRASH
  if (fIn)
  { Int_t res = fclose(fIn);
    if (ReportLevel >= 3)
    { printf ("TPhOpts: fclose input returned %d.\n", res);
      iprint++;
    }
  }
  if (fOut)
  { Int_t res = fclose(fOut);
    if (ReportLevel >= 3)
    { printf ("TPhOpts: fclose output returned %d.\n", res);
      iprint++;
    }
  }
}

Float_t TPhOpts::ConvFloat()
// Return current field converted to floating point
{ Float_t result;
  Int_t off = fOffsets[eNumb];
  Int_t len = fLengths[eNumb];
  Char_t save = fBufp[off + len];
  fBufp[off + len] = 0; // Check for EOB!
  sscanf(&fBufp[off], "%f", &result);
  fBufp[off + len] = save;
  return result;
}

Char_t *TPhOpts::CurrField (Int_t type)
// Return pointer to most recently found field
{ return (type >= 0 && type < 2)? fBufp + fOffsets[type] : 0;
}

Int_t TPhOpts::Find (Int_t targets = eTextBit + eNumbBit)
{
// Find next fields specified by target bits in a range of lines in inFile,
// return fStatus.

  if (!fIn)
  { if (ReportLevel >= 1)
      printf ("TPhOpts::Find: No input file is open.\n");
    fStatus = -2;
    return fStatus;
  }

// Loop until requested targets are found within line range or end of file:

  fLinesRead = 0;

  Bool_t contin = fStatus >= 0;
// Note: naming the variable 'continue' causes fatal compiler errors with
// unspecific diagnostic: 'syntax error'

  while (contin)

  { if (ReportLevel >= 6 && iprint < Nprint)

    { printf ("TPhOpts::Find entered loop with fStatus = %d, fOffsets = "
        "%d, %d, fLengths = %d, %d\n", fStatus, fOffsets[0], fOffsets[1], 
        fLengths[0], fLengths[1]);
      iprint++;
    }

// Read a new line if status is OK and desired targets were not found and
// allowed number of lines is not exceeded:
// Temporary: for "desired targets" read "numeric field"
    if (fStatus >= 0
        && (fStatus & fGetLineSel) != fGetLineReq
        && (fLinesRead < fLinesPerCall))
// Write previous (possibly modified) line to output file if one is open.
// This will skip null lines; do we want them echoed in output?
// Also, there must be an error check here.

    { if (fLine > 0 && fOut)
      { Char_t *loc = fBufp;
        while (*loc) fprintf (fOut, "%c", *loc++);
      }
      fStatus = GetLine();
      for (int nn = 0; nn < 2; nn++)
      { fFields[nn] = 0;
        fLengths[nn] = 0;            
        fLineFields[nn] = 0;            
        fOffsets[nn] = 0;
      }
      if (fStatus == 0) fLinesRead++;
    }

// Search for next text field if required and if status is OK
    if (fStatus >= 0 && (targets & eTextBit)
        && ((fStatus & fGetFieldSel[eText]) != fGetFieldReq[eText]))
    { if (fLengths[eText] > 0) fOffsets[eText] += fLengths[eText] + 1;
      fLengths[eText] = 0;
      if (GetText(fTextKeys, fNTextKeys) >= 0) fStatus |= eTextBit;
      else fStatus &= -eTextBit-1; // (unnecessary)
    }

// Search for next numeric field if required and if status is OK
// This could be flagged not to search unless a text key was found
// Temporary: require a text field found
    if (fStatus >= 0 && (targets & eNumbBit)
        && ((fStatus & fGetFieldSel[eNumb]) != fGetFieldReq[eNumb]))
    { if (fLengths[eNumb] > 0) fOffsets[eNumb] += fLengths[eNumb] + 1;
      fLengths[eNumb] = 0;
      if (GetNum() >= 0) fStatus |= eNumbBit;
      else fStatus &= -eNumbBit-1;
    }

    if (fStatus > 0)
// Field found:
    { for (int nn = 0; nn < 2; nn++)
      { if (fStatus & (nn+1)) // Kludge: should use enum bits
        { if (nn == eNumb) fNumValue = ConvFloat(); // ANOTHER KLUDGE
          fLineFields[nn]++;
          fFileFields[nn]++;
          if (ReportLevel >= 5 && iprint < Nprint)
          { printf ("TPhOpts::Find found field '");
            for (int mm = 0; mm < fLengths[nn]; mm++)
              printf ("%c", fBufp[fOffsets[nn]+mm]);
            printf ("', status %d, in line %d at offset %d",
              fStatus, fLine, fOffsets[nn]);
            if (nn == 1) printf (", d.p. at %d, exp at %d",
              fDecOffs, fExpOffs);
            printf ("\n");
            iprint++;
          }
        }
      }
    } // if (fStatus

    contin = (fStatus >= 0 && (fStatus & targets) != targets);

  } // while (contin

// Close input and optional output files at EOF

  if (fStatus < 0)
  { Int_t res = fclose(fIn);
    if (ReportLevel >= 3)
    { printf ("TPhOpts: closed %s after %d lines, result %d.\n",
        fInFile, fLine, res);
      iprint++;
    }
    if (fOut)
    { res = fclose(fOut);
      if (ReportLevel >= 3)
      { printf ("TPhOpts: closed %s with result %d.\n", fOutFile, res);
      iprint++;
      }
    }
  }
  return fStatus;
}

Int_t TPhOpts::GetField (Char_t * const keys[], Int_t nKeys, Char_t *& chars,
            const Char_t *charLim)
{
// GetField is a text search utility function somewhat like grep.
// It returns the index (0...nKeys-1) of the first of nKeys selector
// keys for which a match occurs in the character array pointed to by 
// the reference chars, up to but excluding the character at charLim.
// It returns -1 if a match is not found, or -2 if a match is not
// found in the first MaxKeys keys, where the local storage limit 
// MaxKeys is arbitrarily chosen as 20.  If more than 20 selectors are
// needed, recompile with MaxKeys increased.
//
// keys is an array of strings, each of which can be
//   (1) a single character to be matched, such as "+";
//   (2) a range, such as "0-9", to be matched by a single character; or
//   (3) a text string of arbitrary length, such as "// MO".
//   keys for the start of a numeric data field might be specified by
//     Char_t *keys[] = { "//", "\n", "0-9", "+", "-", "." };
//   this will stop the search at '//', treated as a comment indicator,
//   or at end of line character '\n', unless a numerical match is found
//   first.  If the array to be searched is simply a string, the 
//   keys could include the entry "" to stop at the end of the string.
// nKeys is the number of selector strings in keys to examine; only
//   the first MaxKeys strings will be used.  nKeys can be calculated 
//   conviently by the caller as sizeof(keys) / sizeof (Char_t *).
// chars is a reference to a pointer to the character array.
//   On return, chars is UPDATED to point to the start of the field at 
//   which the search stopped.
// charLim points to the character past the last character to be used.
//
// Example:
//  Char_t *buf = "Test values: 12.345, -6, 789.0e-04 // Label\n";
//  Char_t *&bufr = buf;
//  Char_t *bufLim = buf + strlen(buf) - 1; // Omit \n
//  Char_t *keyNum[] = { "0-9", "+", "-" };
//  Char_t *keyEnd[] = { ",", " ", "\t", "//", "}" };
//  Int_t iKey = GetField(keyNum, sizeof(keyNum)/sizeof(Char_t *), bufr,
//         bufLim);
//  if (iKey >= 0)
//  { Char_t *start = buf++;
//    iKey = GetField(sizeof(keyEnd)/sizeof(Char_t *), keyEnd, bufr, bufLim);
//    if (iKey >= -1) // allow numeric field at end of line
//    { printf ("GetField found field '");
//      for (char *ll = start; ll < buf;) printf ("%c", *ll++); printf ("'\n");
//    }
//    buf++;
//  }
//
// 03-Apr-98  Creation (Robin Verdier)
//
// Notes:
// The syntax prevents a search for a 3-character string having '-' as
// its second character.  If this becomes a problem, other methods can
// be added to define the range-specifying character.  Wild card search
// characters like ? and * could be added but so far seem unnecessary.
//
// sizeof(keys) must be passed as an argument, since the information 
// cannot be calculated from the passed argument 'keys'.
//
// If the library function fgets is used to read a line, it returns both 
// an end of line character \n and a terminal '0'. 
//
// Attempts to declare the char array, rather than the pointer, as const 
// have been unsuccessful.
// The syntax 'char *&var' for 'reference to pointer to char' used here
// for chars was not accepted by the ROOT C interpreter, which required
// 'char (&)*var' at one time.  char *& now seems to work both for gcc
// and CInt.  A possible alternative is char **.

  Int_t   iKey;             // Index of matching key
  Char_t *lc;               // Current pointer in chars
  Bool_t  match = kFALSE;   // Temporary status
  const
  Int_t   MaxKeys = 20;     // Arbitrary storage limit
  Int_t   nKeysU;           // Number of keys to be used
  Char_t  RangeSpec = '-';  // Character specifying a range of characters

// Find types and lengths of keys and save them in local storage, to avoid 
// reanalyzing the keys for each character.
// A more elegant solution would pass as an argument a struct containing 
// the keys, their number, and workspace allocated by the caller.

  nKeysU = (nKeys <= MaxKeys)? nKeys : MaxKeys;
  Int_t   keyLens[MaxKeys];
  Int_t   keyTypes[MaxKeys];
  for (iKey = 0; iKey < nKeysU; iKey++)
// Set key type = 0 (single character), 1 (range of characters), 
// or 2 (search string of arbitrary length).
  { keyLens[iKey] = strlen(keys[iKey]);
    if (keyLens[iKey] == 0) keyLens[iKey] = 1; // Null string is a special case
    keyTypes[iKey] = (keyLens[iKey] == 1)? 0 : 2;
    if (keyLens[iKey] == 3 && keys[iKey][1] == RangeSpec) // e.g. "0-9", "A-Z"
      keyTypes[iKey] = 1;
  }
   
// Loop over characters in input array

  chars--;
  while (!match && ++chars < charLim)

// Loop on key strings until a match is found or all keys have
// been tested

  { iKey = -1;
    while (!match && ++iKey < nKeysU)
    { lc = chars;
      if (keyTypes[iKey] == 0) match = (*lc == keys[iKey][0]);
      else if (keyTypes[iKey] == 1)
        match = (*lc >= keys[iKey][0] && *lc <= keys[iKey][2]);
      else 
// Search string: examine characters until a match fails
      { match = kTRUE;
        Int_t ic = 0; // index of character in key
        while (match && (ic++ < keyLens[iKey]))
          match = (*lc++ == keys[iKey][ic-1]);
      }
    } // while (!match && ++iKey < nKeysU)
  } // while (!match && ++chars < charLim

  if (chars >= charLim) iKey = (nKeys <= MaxKeys)? -1 : -2;
  if (iKey == -2 && ReportLevel > 0 && iprint < Nprint)
  { printf ("TPhOpts::GetField: search failed, too many selectors" 
      " specified (%d); increase MaxKeys in GetField.\n", nKeys);
    iprint++;
  }
  return iKey;
}

/*
Int_t   TPhOpts::GetFloat (Int_t nStartDec, Char_t *startDec[],
                           Int_t nEndDec, Char_t *endDec[], Int_t nLines)
{
// Find next decimal number with nLines lines and return fStatus
  return 0;
}
*/
Int_t TPhOpts::GetLine()
// Read a new line, update pointers and fStatus, and return fStatus.
// Variables changed: fBufLims, fBufr, fIEnd, fLine
// GetLine could be changed to open a file if not already done.
{ Int_t ret = 0;
  if (!fIn)
  { if (ReportLevel >= 1)
      printf ("TPhOpts::GetLine: No input file is open.\n");
    ret = -2;
  }
  if (ret == 0) // 1)
  { Char_t *cres = fgets(fBufp, fMaxChars, fIn);
    if (cres) // 2)
    { fLine++;
      if (ReportLevel >= 4 && iprint < Nprint)
      { printf ("TPhOpts::GetLine read line %d with cres = %p:\n%s",
          fLine, cres, fBufp);
        iprint++;
      }
      ret = InitLine();
    } // 2)       
    else
    { // Assume fgets failure means end of file reached
      if (ReportLevel >= 3)
        printf ("TPhOpts:: GetLine reached end of input file after"
            " %d lines.\n", fLine);
      ret = -1;
    } // 2)
  } // 1)
  return ret;
}

Int_t TPhOpts::GetNum()
{
// Find a numeric field, return index >= 0 of key for start of field if 
// found, else values listed in GetField.
  fBufr = fBufp + fOffsets[eNumb];
  Int_t key1 = GetField (fDecKeys, fNDecKeys, fBufr, fBufLims[eNumb]);
  if (key1 >= 0) /*1*/
// If found, look for end
  { Char_t *start = fBufr;
    Int_t key2 = GetField (fDecEnd, fNEndKeys, fBufr, fBufLims[eNumb]);
// Allow field to terminate on end of buffer (Key2 -1)
    if (key2 >= -1) /*2*/
// Field found
    { fOffsets[eNumb] = start - fBufp;
      fLengths[eNumb] = fBufr - start;
// Find format of field, set fDecOffs = offset of "." and fExpOffs =
// offset of "e" or "E" (=> float if >0)
      fBufr = start;
      Char_t *dpKey[] = { "." };
      key2 = GetField (dpKey, 1, fBufr, fBufr + fLengths[eNumb]);
      fDecOffs = (key2 >= 0)? fBufr - start : 0;
      fBufr = start;
      Char_t *expKeys[] = { "e", "E" };
      key2 = GetField (expKeys, 2, fBufr, fBufr + fLengths[eNumb]);
      fExpOffs = (key2 >= 0)? fBufr - start : 0;
    } /*2*/
    else
// No field end found: should never happen
    { if (ReportLevel >= 1 && iprint < Nprint)
      { printf ("TPhOpts::GetNum: system error, no field end found.\n"); 
        iprint++;
      }
      key1 = -2;
      fLengths[eNumb] = 1; // just for diagnostic
    } /*2*/
  } /*1*/
  else
// No field found
  { // fStatus = fIEnd + 1; // 1 for EOL, 2 for //; see fEOLKeys
    fLengths[eNumb] = 1; // just for diagnostic
  } /*1*/
  fFields[eNumb] = fBufr;
  //  return fKeys[eNumb] = key1; FAILS?
  fKeys[eNumb] = key1;
  if (ReportLevel >= 6 && iprint < Nprint)
  { printf ("TPhOpts::GetNum returns %d for %d keys ", key1, fNDecKeys);
    for (int nn=0; nn<fNDecKeys; nn++) printf ("'%s' ", fDecKeys[nn]);
    printf ("in buf %p..%p\n", fBufc, fBufLims[eNumb]-1);
    iprint++;
  }
  return fKeys[eNumb];
}

Int_t TPhOpts::GetText(Char_t *texts[], Int_t nTexts)
{
// Get a text field and return index of found text selector,
// or -1 if no matching text field is found in buffer.
// Text selectors have the form
//   Char_t *texts[] = { "// MO", "// MP", "// MN", "// V" };
// THE TERMINAL ";" IS REQUIRED.
// The search will stop at whichever selector occurs first.
// On return fBufr points to the character on which the search ended and 
// fOffsets[eText] and fLengths[eText] are updated (the latter is set to
// 0 if a field is not found.

  fBufr = fBufp + fOffsets[eText];
  fBufc = fBufr;
  Int_t key = GetField (texts, nTexts, fBufr, fBufLims[eText]);
  if (ReportLevel >= 6 && iprint < Nprint)
  { printf ("TPhOpts::GetText returns %d for %d keys ", key, nTexts);
    for (int nn=0; nn<nTexts; nn++) printf ("'%s' ", texts[nn]);
    printf ("in buf %p..%p\n", fBufc, fBufLims[eText]-1);
    iprint++;
  }
  fOffsets[eText] = fBufr - fBufp;
  fLengths[eText] = (key >= 0)? sizeof(texts[key]) : 0;
  if (fLengths[eText] < 0)
  { if (ReportLevel >= 1 && iprint < Nprint)
    { printf ("TPhOpts::GetText gives key length = %d, reset to %d\n",
        fLengths[eText], 4);
      iprint++;
    }
    fLengths[eText] = 4;
  }
  fKeys[eText] = key;
  fFields[eText] = fBufr;
  return key;
}

/*
Int_t   TPhOpts::GetText (Int_t nTextKeys, Char_t *textKeys[], 
                          Int_t nLines)
{
// Find next matching text string within nLines lines and return fStatus
  return fStatus;
}
*/

Char_t *TPhOpts::InFileName() { return fInFile; }

Int_t TPhOpts::InitLine()
{
// Set pointers to start and end of current line.  Returns 0 if no error.
// Variables changed: fBufLims, fBufr, fIEnd.
  Int_t ret = 0;
// Set the delimiters for end of line to avoid extra character by 
// character comparisons.
  for (Int_t nn = 0; nn < 2; nn++)
  { fBufr = fBufp;
    if (nn == 0) // MAKE THE KEYS INDEXED ARRAYS
      fIEnd = GetField (fEOLTextKeys, fNEOLTextKeys, fBufr, fBufp + fMaxChars);
    else
      fIEnd = GetField (fEOLNumKeys, fNEOLNumKeys, fBufr, fBufp + fMaxChars);
    if (fIEnd >= 0)
      fBufLims[nn] = fBufr;
    else
    { fBufLims[nn] = fBufp + fMaxChars;
      if (ReportLevel >= 1 && iprint < Nprint)
      { printf ("TPhOpts::InitLine could not find EOL, fIEnd = %d.\n",
        fIEnd);
        iprint++;
      }
      ret = -2;
    } // if (fIEnd >= 0
  } // for (Int_t nn
  return ret;
}

Char_t *TPhOpts::OutFileName() { return fOutFile; }

void TPhOpts::Open(const Char_t *file, const Char_t *mode)
{ // Open options template (mode = "r") or copy (mode = "w") file
  Bool_t forceMode = kTRUE; // if true, force mode to "w" if not "r"
  Bool_t readMode = strcmp(mode, "r") == 0;
  if (strlen(file) > 0)
  { Char_t *muse = (Char_t *)mode;
    if (forceMode && !readMode) muse = "w";
    FILE *fp = fopen(file, muse);
// To handle errors here, we need to look in error.h and strerror(n)
// See Kernighan and Ritchie, Appendix B
    if (fp)
    { if (readMode)
      { fInFile = (Char_t *)file;
        fIn = fp;
// Reset all counters when a new input file is opened.
        for (int nn = 0; nn < 2; nn++)

        { fLineFields[nn] = 0;
          fFileFields[nn] = 0;
          fLengths[nn] = 0;
          fOffsets[nn] = 0;
        }
        fLine = 0;
// We need status for both input and output files.
        fStatus = 0;
      }
      else
      { fOutFile = (Char_t *)file;
        fOut = fp;
      }
      if (ReportLevel >= 3)
      { printf ("TPhOpts::Open opened file %s in mode %s at %p.\n", 
          file, muse, fp);
        iprint++;
      }
    }
    else // fp == 0:
    { if (ReportLevel >= 1)
      { printf ("TPhOpts::TPhOpts unable to open %s in mode %s.\n",
          file, muse);
        iprint++;
      }
    }
  }
  else // strlen(file) == 0:
    if (ReportLevel >= 1)
      printf ("TPhOpts::Open can't open, no filename specified.\n");
}

void TPhOpts::SetDecKeys (Int_t nDecKeys = fNDecKeys,
                          Char_t *decKeys[] = fDecKeys)
{ // Set current numeric start keys
  fNStartKeys = nDecKeys;
  fStartKeys = decKeys;
}

void TPhOpts::SetDecEnd (Int_t nEndKeys, Char_t *endKeys[])
{ // Set numeric field end keys
  fNEndKeys = nEndKeys;
  fEndKeys = endKeys;
}

void TPhOpts::SetEOLKeys (Int_t type = eNumb, Int_t nKeys = fNEOLNumKeysD, 
                          Char_t *keys[] = fEOLNumKeysD)
{ // Set current end-of-line keys
  if (type == eText)
  { fNEOLTextKeys = nKeys;
    fEOLTextKeys = keys;
  }
  else
  { fNEOLNumKeys = nKeys;
    fEOLNumKeys = keys;
  }
}

void TPhOpts::SetTextKeys (Int_t nTextKeys, Char_t *textKeys[])
{
// Set current text keys
  fNTextKeys = nTextKeys;
  fTextKeys = textKeys;
}

void TPhOpts::SetField(Float_t value)
{ if (fOut)
// Format value to match the input format.  This is not possible simply
// using sprintf since the ONLY format is x.<yyyy>ezzz, i.e., no 
// 120.0e-06, even if you specify %9.1e.  So this will have to be 
// done by creating the before-decimal, after-decimal and exponent 
// fields as leading-zero integers.  Until then, save two characters
// past length since sprintf often overflows the specified field length.
  { Char_t save1 = fBufp[fOffsets[eNumb] + fLengths[eNumb]];
    Char_t save2 = fBufp[fOffsets[eNumb] + fLengths[eNumb] + 1];
    if (fExpOffs < 0)
    { if (fDecOffs < 0)  // no exponent, no d.p., e.g., 125
        sprintf (fBufp + fOffsets[eNumb], "%*d", fLengths[eNumb], int(value));
      else // decimal point, no exponent, e.g. 125.0
        sprintf (fBufp + fOffsets[eNumb], "%*.*f", fLengths[eNumb], 
          fLengths[eNumb] - fDecOffs - 1, value);
    }
    else
    { if (fDecOffs < 0) // exponent, no decimal point, as 125e-0
      { sprintf (fBufp + fOffsets[eNumb], "%*.0e", fExpOffs + 4, value);
        if (fLengths[eNumb] < fExpOffs + 4) 
          fBufp[fOffsets[eNumb] + fExpOffs + 2] = 
            fBufp[fOffsets[eNumb] + fExpOffs + 3];
      }  
      else  // exponent and decipal point, as 1.25e2
      { sprintf (fBufp + fOffsets[eNumb], "%*.*e", fExpOffs + 4, 
          fExpOffs - 2, value);
        if (fLengths[eNumb] == fExpOffs + 2) 
          fBufp[fOffsets[eNumb] + fExpOffs + 1] =
            fBufp[fOffsets[eNumb] + fExpOffs + 3];
        if (fLengths[eNumb] == fExpOffs + 3) 
          fBufp[fOffsets[eNumb] + fExpOffs + 2] =
            fBufp[fOffsets[eNumb] + fExpOffs + 3];
      }  
    } // if (fExpOffs .. else
    fBufp[fOffsets[eNumb] + fLengths[eNumb]] = save1;
    fBufp[fOffsets[eNumb] + fLengths[eNumb] +1] = save2;
  } // if (fOut
  else
    if (ReportLevel >= 1 && iprint < Nprint)
      printf ("TPhOpts::SetField: no output file open "
        "(%d message-limit).\n", Nprint - iprint++);
}

Int_t   TPhOpts::SetFloat(Float_t value)
{ // Set current float field in outFile-- new version
  if (ReportLevel >= 1 && iprint < Nprint)
  { printf ("TPhOpts::SetFloat unimplemented, unable to set %f.\n", value);
    iprint++;
  }    
  return 0;
}

Int_t TPhOpts::SetLineRange (Int_t limit)
// Set search range relative to current line
{ fLinesPerCall = limit;
  return limit;
}
