#ifndef _TPhStop_INCLUDED_
#define _TPhStop_INCLUDED_
//#define DEBUG
//
///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhStop                                                        //
//                                                                   //
//    Phobos class for finding  stopping particles in an event       //
//    and identifying them.                                          //
//                                                                   //
///////////////////////////////////////////////////////////////////////

#include "TObject.h"
#include "TPolyLine.h"
#include "TPolyMarker.h"
#include "TMarker.h"

const Int_t   kNLayersStart= 4; 
const Int_t   kNLayersStop = 4; 
      //Float_t kDAngle_E[4]={0.95,1.60,1.25,2.90};
      //Float_t kDAngle_D[3]={0.95,1.60,2.90};
      //Float_t kDAngle_C[2]={0.95,2.90};
const Float_t kdE          = 0.0008; 
const Float_t kdang1       = 2.0;
const Float_t kdanglast    = 5.0;

const Float_t r2d          = 57.29578;

class TPhEvent;
class TPhObjectContainer;
class TPhTrack;
class TPhMCTrack;
class TPhMCHit;
class TPhVertex;
class TPhDetector;
class TPhSubDetector;
class TPhMomentumVector;
class TPhLayer;

class TPhStop : public TObject 
{ 
 public:
  TPhStop();
  TPhStop(Char_t *FileName);
  virtual Bool_t  ReadBandPara(Char_t *FileName);
  virtual ~TPhStop();
  virtual void              SetDetector     ( TPhDetector *det){ fDetector=det ;}
  virtual void              SetVerbose      ( Int_t n) { fVerbose = n; }

  virtual TPhObjectContainer* GetStoppingPtls( TPhEvent *Event,Char_t arm);
 
                                                            // master member functions
							    // which call the other functions
  virtual void   EnergyCut( TPhEvent *event, Char_t arm);   // hit energy cut
  virtual void   BreakCut();                                // break  cut
  virtual void   SelectCandidateTracks();                   // all combinatorial tracks
  virtual void   AngularCut();                              // theta phi cut
  virtual void   GetThetaPhi( TPhMCHit *hit, 
			      Float_t  *theta, 
			      Float_t  *phi       );        // calculate theta and phi
  virtual void  MassCut();                                  // mass parameter cut

  virtual Float_t  GetStopMass( TPhTrack *track   );
  virtual Float_t  GetEloss( TPhTrack *track   );
  virtual  void    Clear(TPhObjectContainer *StuckTracksBuffer);
  // data members
 private:
    TPhDetector        *fDetector;             //! Detector object for data access
    TPhSubDetector     *fSubDetector;          //! SubDetector object for data access
    TPhObjectContainer *fStuckTracksBuffer;    //! Container of tracks stuck in silicons
    TObjArray          *fHighElossHitList;     //! List of hits with E > kdE   
    Bool_t              fVerbose;              //! Control debugging output
    Int_t               fNLastLayer;           //! variable used to shorten a loop

    Float_t fMassLowEdgePi,fMassLowEdgeK,fMassLowEdgeP;
    Float_t fMassHighEdgePi,fMassHighEdgeK,fMassHighEdgeP;
    Float_t fMassBinWidthPi,fMassBinWidthK,fMassBinWidthP;

    Float_t fElossLowEdgePi,fElossLowEdgeK,fElossLowEdgeP;
    Float_t fElossHighEdgePi,fElossHighEdgeK,fElossHighEdgeP;
    Float_t fElossBinWidthPi,fElossBinWidthK,fElossBinWidthP;
    Int_t   fME_Pi[100][100];
    Int_t   fME_K[100][100];
    Int_t   fME_P[100][100];

 public:

    ClassDef(TPhStop,1)   // TPhStop class to find and identify stopping ptls 

};

#endif
