#include "TPhStop.h"
#include "TPhMCHit.h"
#include "TPhMCTrack.h"
#include "TPhDetector.h"
#include "TPhSubDetector.h"
#include "TPhLayer.h"
#include "TPhSensor.h"
#include "TPhEvent.h"

#include "TPhObjectContainer.h"

ClassImp(TPhStop)
///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhStop                                                        //
//                                                                   //
//    Phobos class for finding  stopping particles in an event       //
//    and identifying them.                                          //
//                                                                   //
// Author: Adam Trzupek                                              //
// $Id: TPhStop.cxx,v 1.8 1999/03/11 11:27:29 trzupek Exp $
//                                                                   //
///////////////////////////////////////////////////////////////////////

TPhStop::TPhStop()
{
}

TPhStop::TPhStop(Char_t *FileName)
{
  ReadBandPara(FileName);
  fStuckTracksBuffer   = new TPhObjectContainer("StuckTracks");
  fHighElossHitList    = new TObjArray();
}

Bool_t TPhStop::ReadBandPara(Char_t *FileName)
{
  //
  // The file with parameters of Mass vs Eloss bands is opened and read 
  // into arrays:

  Int_t NBinsXPi,NBinsYPi;
  Int_t NBinsXK,NBinsYK;
  Int_t NBinsXP,NBinsYP;

  fVerbose=1;
  Char_t str[100];

  FILE* Fp;
  Fp = fopen(FileName,"r");
  if(Fp == NULL)
    {
      printf(" TPhStop:Couldn't open %s\n",FileName);
      return kFALSE;
    }
  else
    {
      printf(" TPhStop:Opened file %s \n",FileName);
    }

  // read in data for pions

  fgets (str, 100, Fp);
  sscanf(str,"%f %f %d ",&fElossLowEdgePi,&fElossBinWidthPi,&NBinsXPi);
  fElossHighEdgePi=fElossLowEdgePi + fElossBinWidthPi * NBinsXPi;
  if(fVerbose==1)
    printf("%f %f %d  \n ",fElossLowEdgePi,fElossBinWidthPi,NBinsXPi);
  fgets (str, 100, Fp);
  sscanf(str,"%f %f %d ",&fMassLowEdgePi,&fMassBinWidthPi,&NBinsYPi);
  fMassHighEdgePi=fMassLowEdgePi + fMassBinWidthPi * NBinsXPi;
  if(fVerbose==1)printf("%f %f %d \n",fMassLowEdgePi,fMassBinWidthPi,NBinsYPi);
  if( NBinsXPi > 100 || NBinsYPi > 100 )
    {
      printf("TPhStop:increase string length \n");
      return kFALSE;
    }
  //Int_t ME_Pi[NBinsYPi][NBinsXPi];
  Int_t i;
  for(i = 0; i < NBinsYPi; i++)
    {
      fgets (str, 100, Fp);
      for(Int_t j=0;j<NBinsXPi;j++)
	{
	  fME_Pi[i][j]=0;
	  if(str[j]=='1')fME_Pi[i][j]=1;
	  if(fVerbose==1) printf("%d",fME_Pi[i][j]);
	}
      if(fVerbose==1) printf("\n");
    }
  // read in data for kaons

  fgets (str, 100, Fp);
  sscanf(str,"%f %f %d ",&fElossLowEdgeK,&fElossBinWidthK,&NBinsXK);
  fElossHighEdgeK=fElossLowEdgeK + fElossBinWidthK * NBinsXK;
  if(fVerbose==1)printf("%f %f %d  \n ",fElossLowEdgeK,fElossBinWidthK,NBinsXK);
  fgets (str, 100, Fp);
  sscanf(str,"%f %f %d ",&fMassLowEdgeK,&fMassBinWidthK,&NBinsYK);
  fMassHighEdgeK=fMassLowEdgeK + fMassBinWidthK * NBinsXK;
  if(fVerbose==1)printf("%f %f %d \n",fMassLowEdgeK,fMassBinWidthK,NBinsYK);
  if( NBinsXK > 100 || NBinsYK > 100 )
    {
      printf("TPhStop:increase string length \n");
      return kFALSE;
    }
  //Int_t ME_K[NBinsYK][NBinsXK];
  for(i = 0; i < NBinsYK; i++)
    {
      fgets (str, 100, Fp);
      for(Int_t j=0;j<NBinsXK;j++)
	{
	  fME_K[i][j]=0;
	  if(str[j]=='1')fME_K[i][j]=1;
	  if(fVerbose==1) printf("%d",fME_K[i][j]);
	}
      if(fVerbose==1) printf("\n");
    }
  // read in data for protons

  fgets (str, 100, Fp);
  sscanf(str,"%f %f %d ",&fElossLowEdgeP,&fElossBinWidthP,&NBinsXP);
  fElossHighEdgeP=fElossLowEdgeP + fElossBinWidthP * NBinsXP;
  if(fVerbose==1)printf("%f %f %d  \n ",fElossLowEdgeP,fElossBinWidthP,NBinsXP);
  fgets (str, 100, Fp);
  sscanf(str,"%f %f %d ",&fMassLowEdgeP,&fMassBinWidthP,&NBinsYP);
  fMassHighEdgeP=fMassLowEdgeP + fMassBinWidthP * NBinsXP;
  if(fVerbose==1)printf("%f %f %d \n",fMassLowEdgeP,fMassBinWidthP,NBinsYP);
  if( NBinsXP > 100 || NBinsYP > 100 )
    {
      printf("TPhStop:increase string length \n");
      return kFALSE;
    }
  //Int_t ME_P[NBinsYP][NBinsXP];
  for(i = 0; i < NBinsYP; i++)
    {
      fgets (str, 100, Fp);
      for(Int_t j=0;j<NBinsXP;j++)
	{
	  fME_P[i][j]=0;
	  if(str[j]=='1')fME_P[i][j]=1;
	  if(fVerbose==1)printf("%d",fME_P[i][j]);
	}
      if(fVerbose==1)printf("\n");
    }


  fVerbose=0;

 return kTRUE;
}


TPhStop::~TPhStop() 
{
  delete fStuckTracksBuffer;
  delete fHighElossHitList;
}



TPhObjectContainer *TPhStop::GetStoppingPtls(TPhEvent *event,Char_t arm)
{

  //  this function makes call to 
  //  functions    

  //  EnergyCut  
  //  BreakCut      
  //  SelectCandidateTracks
  //  MassCut
  //  AngularCut

  if(fVerbose==1)printf("GetStoppingPtls: \n");
  
  Int_t ncand=0,ncandA=0,ncandM=0;
  
  EnergyCut(event,arm);
  
  if(fHighElossHitList->GetEntries())
    {
      fNLastLayer=0;
      if(fVerbose==1)printf("GetStoppingPtls: No of hits after energy cut = %d \n", 
			    fHighElossHitList->GetEntries());
      BreakCut();
      if(fNLastLayer)
	{
	  if(fVerbose==1)printf("GetStoppingPtls: No of hits after break cut = %d \n",
				fHighElossHitList->GetEntries());
	  SelectCandidateTracks();
	  ncand = fStuckTracksBuffer->GetEntries();
	  if(fVerbose==1)printf("GetStoppingPtls: No of candidate tr. = %d\n",ncand);
	  if(ncand)
	    {
	      MassCut();
	      ncandM = fStuckTracksBuffer->GetEntries();
	      if(fVerbose==1)printf("GetStoppingPtls: No of candidate tr. after mass cut = %d\n",
				    ncandM);
	      if(ncandM)
		{
		  AngularCut();
		  ncandA = fStuckTracksBuffer->GetEntries();
		  if(fVerbose==1)printf("GetStoppingPtls: No of candidate tr. after theta phi cut = %d\n",
					ncandA);
		}
	    }
	} 
    }
  
  if(fVerbose==1)printf("GetStoppingPtls: leaving \n");

  // do some cleaning work

  fHighElossHitList->Clear();

  return fStuckTracksBuffer;
}

void TPhStop::EnergyCut(TPhEvent *event, Char_t arm)
{
  // all hits with energy > kdE are selected in layers form 0 to kNLayersStop+1

  if(fVerbose==1)printf("EnergyCut: 0 \n");
  Char_t *SubDetectorName;
  if(arm=='P')SubDetectorName="SpectP";
  if(arm=='N')SubDetectorName="SpectN";
  fSubDetector=fDetector->GetSubDetector(SubDetectorName);

  TPhMCHit  *hit;
  if(fVerbose==1)printf("EnergyCut: 1 \n");
  TPhObjectContainer *hits = (TPhObjectContainer *) event->GetObject("MCHits");
  if(fVerbose==1)printf("EnergyCut: 2 %d \n",hits->GetEntries());
  TObjArray *HitList; 
  if( hits ) HitList   = hits->GetObjectList();
  if(fVerbose==1)printf("EnergyCut: 3 %d \n",HitList->GetEntries());
  TIter NextHit(HitList);
  while((hit = (TPhMCHit *) NextHit()))
    {
      const Char_t *name=(fDetector->GetSensor(hit->GetSensorLabel()))->GetName();
      if(name[0]== arm)
	{
	  if(hit->GetDe() > kdE  &&  hit->GetLayer()<= kNLayersStop+1)
	    {
	      fHighElossHitList->Add(hit);
	      if(fVerbose==1)
		printf("EnergyCut: Found hit %s in layer %d with energy loss %g > %g\n",
		       (fDetector->GetSensor(hit->GetSensorLabel()))->GetName(),hit->GetLayer(),hit->GetDe(),kdE); 
	    }
	}
    }
}


void TPhStop::BreakCut()
{
  // the HitList( energy > kdE) is truncated at a layer 
  // where there is not any hit - the last layer with 
  // hits is remembered as data member fNLastLayer
  
  Int_t i,NHitsLayer;
  
  fDetector->ClearHits();
  
  TPhMCHit  *hit;
  
  if(fVerbose==1)printf("BreakCut:%d \n",fHighElossHitList->GetEntries());

  
  for(i=0;i<=kNLayersStop+1;i++)
    {
      NHitsLayer=0;
      TIter NextHit(fHighElossHitList);
      while((hit = (TPhMCHit *) NextHit()))
	{
	  if(i==hit->GetLayer())
	    {
	      NHitsLayer++;
	      if(fVerbose==1)printf("BreakCut: 2 %d \n",i);
	      fDetector->AddHit(hit);
	    }
	}
      if(NHitsLayer==0)break;
    }
  fNLastLayer=i-1;

  if(fVerbose==1)
    printf("BreakCut: last layer = %d, (< kNLayersSart=%d" \
	   "|| > kNLayersStop=%d then reset to 0)\n",
	   fNLastLayer,kNLayersStart,kNLayersStop);
  if(fNLastLayer < kNLayersStart || fNLastLayer > kNLayersStop)
    fNLastLayer=0;
}
  
void TPhStop::SelectCandidateTracks()
{

  TPhTrack   *track,*tr;
  TPhMCHit   *hit,*hitfromtrack;
  TPhLayer   *layer;
  TPhSensor  *sensor;

  if(fVerbose==1)printf("SelectCandidateTracks: \n");

  // start building event of candidate tracks,
  // add tracks to the event which contain only one hit in the layer 0

  if(fVerbose==1)printf("SelectCandidateTracks: \n");

  layer = fSubDetector->GetLayer(0);
  TIter NextSensor(layer->GetSensorList());
  while((sensor = (TPhSensor *) NextSensor()))
    {
      TIter NextHit(sensor->GetHitList());  // loop over hits in a layer
      while((hit = (TPhMCHit *) NextHit()))
	{    
	  if(layer->GetLabel()==0)
	    {
	      track = new TPhTrack();
	      track -> AddHit(hit);
	      fStuckTracksBuffer -> Add(track);	
	    }
	}
    }

  // continue building the event of candidate tracks,
  // add new tracks to the event which are old ones with
  // additional hit from the next layer
  if(fVerbose==1)printf("SelectCandidateTracks: \n");

  for(Int_t iLayer=1;iLayer<=fNLastLayer;iLayer++)
    {
      layer = fSubDetector->GetLayer(iLayer);
      TIter NextSensor(layer->GetSensorList());
      while((sensor = (TPhSensor *) NextSensor()))
	{
	  TIter NextHit(sensor->GetHitList());  // loop over hits in layer
	  while((hit = (TPhMCHit *) NextHit()))
	    {    
	      TIter NextTrack(fStuckTracksBuffer->GetObjectList());  
	      while((track = (TPhTrack *) NextTrack()) )
		{
		  Int_t Nhits=track->GetHitList()->GetEntries();
		  if (  Nhits >= iLayer+1)break; 
		  if (  Nhits == iLayer  ){ 
		    tr = new TPhTrack();
		    TIter NextHitdum(track->GetHitList());
		    while((hitfromtrack = (TPhMCHit *) NextHitdum()))tr->AddHit(hitfromtrack);
		    tr -> AddHit(hit);
		    fStuckTracksBuffer ->Add(tr);
		  }
		}
	    }
	}
    }

  if(fVerbose==1)printf("SelectCandidateTracks: 4 %d \n",
			fStuckTracksBuffer->GetObjectList()->GetEntries());
  // removes the candidates which have hits only in 
  // layer 0 or 0 and 1

  TIter NextTrack(fStuckTracksBuffer->GetObjectList());  
  while((track = (TPhTrack *) NextTrack()) )
    { 
      Int_t Nhits=track->GetHitList()->GetEntries();
      if ( Nhits == 1 || Nhits == 2 )
	{
	  fStuckTracksBuffer->GetObjectList()->Remove(track);
	  delete track;
	}
    } 

  if(fVerbose==1)printf("SelectCandidateTracks: 5 leaving  %d \n",
			fStuckTracksBuffer->GetObjectList()->GetEntries());
  return;
}

void TPhStop::MassCut()
{
  // the candidate tracks are evaluated by using the bands in 
  // the plots of the mass parameter vs. Eloss which are
  // kept in arrays  fME_Pi,  fME_K, fME_P

  TPhTrack      *track;

  Float_t        mass,  Eloss;
  Int_t         iMass, iEloss;
  Int_t         itrack=0, ok;

  if(fVerbose==1)printf("MassCut: \n");

  TIter NextTrack(fStuckTracksBuffer->GetObjectList());  
  while((track = (TPhTrack *) NextTrack()) )
    {
      ok=0;
      mass =1000*GetStopMass(track);			
      Eloss=1000*GetEloss(track);			

      if(mass  > fMassLowEdgePi  && mass  < fMassHighEdgePi && 
	 Eloss > fElossLowEdgePi && Eloss < fElossHighEdgePi )
	{
	  iMass  = (int)( (mass  - fMassLowEdgePi )/fMassBinWidthPi );
	  iEloss = (int)( (Eloss - fElossLowEdgePi)/fElossBinWidthPi);
	  if( *(*(fME_Pi+iMass)+iEloss) == 1)
	    {
	      ok=1;
	      track->SetGPid( 8);
	    }
	}
      if(mass  > fMassLowEdgeK  && mass  < fMassHighEdgeK && 
	 Eloss > fElossLowEdgeK && Eloss < fElossHighEdgeK )
	{
	  iMass  = (int)( (mass  - fMassLowEdgeK )/fMassBinWidthK );
	  iEloss = (int)( (Eloss - fElossLowEdgeK)/fElossBinWidthK);
	  if( *(*(fME_K+iMass)+iEloss) == 1)
	    {
	      ok=1;
	      track->SetGPid(11);	  
	    }
	}
      if(mass  > fMassLowEdgeP  && mass  < fMassHighEdgeP && 
	 Eloss > fElossLowEdgeP && Eloss < fElossHighEdgeP )
	{
	  iMass  = (int)( (mass  - fMassLowEdgeP )/fMassBinWidthP );
	  iEloss = (int)( (Eloss - fElossLowEdgeP)/fElossBinWidthP);
	  if( *(*(fME_P+iMass)+iEloss) == 1)
	    {
	      ok=1;
	      track->SetGPid(14);	  
	    }
	}
      if(fVerbose==1)printf("MassCut:: mass Eloss ok = %f %f %d \n",mass,Eloss,ok);
      if(ok == 1)
	{
	  track->SetMass(mass);
	  itrack++;
	}
      else
	{
	  fStuckTracksBuffer->GetObjectList()->Remove(track);
	  delete track; 	
	}
    }
  return;
}

Float_t TPhStop::GetStopMass(TPhTrack *track)
{

  // mass parameter is calculated

  TPhMCHit *hit;

  Float_t Mass=0.,Dei,Eloss_r,Dx2,Dy2,Dz2,ro_si=2.33;
  Int_t i,j;
  Int_t istop=track->GetNHits();
  
  if(fVerbose==1)printf("GetStopMass: %d \n",istop);

  for(i=0;i<istop-1;i++)	     
    {
      Eloss_r=0.;
      j=0;
      TIter NextHit(track->GetHitList());
      while((hit = (TPhMCHit *) NextHit()))
	{
	  if(fVerbose==1)printf("GetStopMass: %s %d \n",
              (fDetector->GetSensor(hit->GetSensorLabel()))->GetName(), i);
	  if(j==i)
	    {
	      Dei=hit->GetDe();
	      Dx2=hit->GetDx()*hit->GetDx();
	      Dy2=hit->GetDy()*hit->GetDy();
	      Dz2=hit->GetDz()*hit->GetDz();
	    }
	  if(j>=i)Eloss_r=Eloss_r+hit->GetDe();
	  j++;
	}  
      if(Dx2+Dy2+Dz2 > 0.0)
	Mass=Mass+Eloss_r*Dei/sqrt(Dx2+Dy2+Dz2);
      else
	return Mass=0.0;
    }
  Mass = Mass/ro_si/(istop-1); 
  return Mass;
}

Float_t TPhStop::GetEloss(TPhTrack *track)
{

  // Energy loss is calculated

  TPhMCHit    *hit;
  Float_t Eloss = 0.0;
  if(track->GetNHits()==0)return 0.0;
  TIter NextHit(track->GetHitList());
  while((hit = (TPhMCHit *) NextHit()))Eloss = Eloss + hit->GetDe();
  return Eloss/track->GetNHits();
}

void TPhStop::AngularCut()
{

  // candidate angles are evaluated using the 
  // precalculated values of theta and phi stored in 
  // DAngle_Pi_E_Theta, ..., DAngle_Pi_E_Phi ,... arrays

  TPhSensor      *sensor;
  TPhLayer       *layer;
  TPhMCHit       *hit;
  TPhTrack       *track;

  Int_t       i,NhitsAtTrack;
  Float_t     theta,phi,thetaprev,phiprev;

  Float_t     DAngle_Pi_E_Theta[4]={2.10,3.30,2.40,5.10};
  Float_t     DAngle_K_E_Theta[4] ={1.35,2.05,1.50,3.30};
  Float_t     DAngle_P_E_Theta[4] ={0.95,1.60,1.25,2.90};

  Float_t     DAngle_Pi_E_Phi[4]={2.70,3.55,3.250,5.60};
  Float_t     DAngle_K_E_Phi[4] ={1.90,2.65,2.15,3.90};
  Float_t     DAngle_P_E_Phi[4] ={1.60,1.60,1.85,3.60};

  Float_t    *DAngle_Theta,*DAngle_Phi;
  Int_t ok;

  if(fVerbose==1)printf("AngularCut: entering\n");

  Int_t itra=0;
  TIter NextTrack(fStuckTracksBuffer->GetObjectList());  
  while((track = (TPhTrack *) NextTrack()) )
    {
      if(fVerbose==1)printf("AngularCut: track no %d  \n",itra);
      if(fVerbose==1)track->Print("10");
      if(track->GetGPid()== 8)DAngle_Theta=DAngle_Pi_E_Theta;
      if(track->GetGPid()==11)DAngle_Theta=DAngle_K_E_Theta;
      if(track->GetGPid()==14)DAngle_Theta=DAngle_P_E_Theta;

      if(track->GetGPid()== 8)DAngle_Phi=DAngle_Pi_E_Phi;
      if(track->GetGPid()==11)DAngle_Phi=DAngle_K_E_Phi;
      if(track->GetGPid()==14)DAngle_Phi=DAngle_P_E_Phi;
      i=0;
      ok=1;
      itra++;
      TIter NextHit(track->GetHitList());
      while((hit = (TPhMCHit *) NextHit()))
	{
	  GetThetaPhi(hit,&theta,&phi);
	  if(i==1)
	    {
	      if(fabs(thetaprev - theta)>DAngle_Theta[0])ok=0;
	      if(fabs(phiprev   - phi  )>DAngle_Phi[0])ok=0;
	      NhitsAtTrack=i;
	    }
	  if(i>1)
	    {
	      if(fabs(thetaprev - theta)>DAngle_Theta[i-1])ok=0;
	      if(fabs(phiprev   - phi  )>DAngle_Phi[i-1])ok=0;
	      NhitsAtTrack=i;
	    }
	  thetaprev=theta;
	  phiprev=phi;
	  i++;
 	}

      if(fVerbose==1)printf("AngularCut:ok  %d \n",ok);

      if(ok==1)
	{
	  layer = fSubDetector->GetLayer(track->GetNHits());
	  TIter NextSensor(layer->GetSensorList());
	  while((sensor = (TPhSensor *) NextSensor()))
	    {
	      TIter NextHit1(sensor->GetHitList());
	      while((hit = (TPhMCHit *) NextHit1()))
		{
		  GetThetaPhi(hit,&theta,&phi);
		  if(fVerbose==1)printf("AngularCut:  thetaprev phiprev %f %f  %f %f  \n",
					 thetaprev,phiprev,theta,phi);
		  if(fabs(thetaprev - theta) < DAngle_Theta[i-1] && 
		     fabs(phiprev   - phi  ) < DAngle_Phi[i-1])ok=0;
		}
	    }
	  if(ok==0)
	    {
	      fStuckTracksBuffer->GetObjectList()->Remove(track); 	
	      delete track; 	
	    }
	}
      else
	{
	  fStuckTracksBuffer->GetObjectList()->Remove(track);
	  delete track; 	
	}
      if(fVerbose==1)printf("AngularCut: ok 2 %d \n",ok);
    }
  if(fVerbose==1)printf("AngularCut: leaving\n");
  return;
}

void  TPhStop::GetThetaPhi(TPhMCHit* hit, Float_t *theta, Float_t *phi)
{
  Float_t x,y,z,Phi,Theta;

  if(fVerbose==1)printf("GetThetaPhi: \n");
    
  x=hit->GetX();
  y=hit->GetY();
  z=hit->GetZ();
  Phi=90-r2d*atan2(x,y);
  if(hit->GetX()<0)Phi=Phi-180;
  Theta=r2d*atan(sqrt(x*x+y*y)/z);
  *phi=Phi;
  *theta=Theta;
  return;
}

void TPhStop::Clear(TPhObjectContainer *StuckTracksBuffer)
{
  TPhTrack *track;
  if(StuckTracksBuffer)
    {
      
      if(0)printf("\n Clear 1 : \n");
      TIter NextTrack(StuckTracksBuffer->GetObjectList());  
      while((track  =  (TPhTrack *) NextTrack()) )
	{ 
	  StuckTracksBuffer->GetObjectList()->Remove(track);
	  delete track;
	} 
    }
  StuckTracksBuffer->Clear();
}
