       BLOCK DATA PMCBDAT
*Documentation_Block
************************************************************************
*
*  Description:
*  Block data for the Phobos MC.  Not executable.
*
*  Arguments:
*  none
*
*  Referenced by:
*  UGINIT, to force loading
*
*  Calls:
*  none
*
*  Implicit inputs, outputs, side effects:
*  ANSI-compliant method for setting default values of common variables
*
*  Filename: PMC_SRC:BLOCKDATA.FOR
*
*  History:
*  Date       Name       Description
*  ----       ----       -----------
*  25-Jul-96  R. Verdier Creation
************************************************************************
*End_Documentation_Block

*KEEP,MUPEP.
c MUPEP.CMN: PHOBOS MC multi-pass event processing variables
c 16-Mar-95 K. W.     NeSkip added
c  6-Jun-93 R. V.     Creation
c Multi-pass event processing breaks up each event into groups of
c tracks, in order to avoid overflowing Zebra track storage.
c Common for access to control variables:
      common /MUPEP/ EPState, NtConc, NtpE, NtPrim, NeSkip, NtSkip,
     1  MPind, NtPrev, NtTot, NtMax, NvPrev
c Parameters and data values are assigned in MUPEP.CDA.
c Event processing state: EPState = 0 for first pass,
c 1 for continuing pass, 2 for final pass
      integer EPState
c Current pass index:
      integer MPind
c Maximum number of tracks to process concurrently in a group:
      integer NtConc
c Maximum track number for this group:
      integer NtMax
c Maximum number of tracks per event:
      integer NtpE
c Number of primary and secondary tracks generated before this pass:
      integer NtPrev
c Number of primary tracks and vertices generated:
      integer NtPrim, NvPrev
c Number of input file events to skip initially:
      integer NeSkip
c Number of tracks to skip initially:
      integer NtSkip
c Total number of primary and secondary tracks generated:
      integer NtTot
*KEEP,MUPEPD.
c MUPEPD.CMN: Data for PHOBOS common MUPEP
c 16-Mar-95 K. W.     NeSkip added
c  9-May-93 R. V.     Creation
      data EPState /0/, NtConc /1000/, Neskip /0/, NtSkip /0/
      data NtpE /20000/
*KEEP,NTU.
C Common block for Ntuple image and other analysis info.  File NTU.INC
C There may be one Truth Ntuple and/or one Hits Ntuple for each track.
C These must be correlated into events using the event number and other
C event information.
C 10-Jul-95  K. W.      Increased number of planes for spectrometer to 17
C 10-Mar-95  K. W.      Increased number of planes for spectrometer
C 19-Jul-94  R. V.      Update tagnames: Cxx -> Dxx, VOLTERM -> VOLUME
C 10-May-94  R. V.      Modify Ntuples to contain new hits information;
C                       add NTTAGS common for tags; move data to NTUD.INC
C 29-Jun-93  M. Baker   Modify Ntuples
C 10-Jan-93  R.Verdier  Separate Hits and Truth ntuples; rename to NTU.CMN
C    Mar-92  D. Mcleod  Creation as COMMON.NTU
C Number:  Name:    --------------- Description:----------------------------
C>>>>>>>GENERAL INFORMATION<<<<<<<<<<<<<<<<<<
*   1      RUN      run number
*   2      EVENT    event number
*   3      TRACK    track number
*   4      GEN      event generator type; 1 = fixed, 2 = random, etc.
*   5      ANAL     event analysis type; 1 = Ntuple only, 2 = resolution, etc.
*   6      SCAT     ISCAT = IMULS + 3*ILOSS + 9*IPIXL; 1st 2 are in /GCPHYS/
*   7      BIMP     Impact Parameter   ** NOT YET IMPLEMENTED **
C>>>>>>>THE TRUTH<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
*   8      XPROD    track origin
*   9      YPROD       "    "
*   10     ZPROD       "    "
*   11     IDPART   particle type (GEANT code)
*   12     CHRG     particle charge
*   13     TP       particle momentum (at production)
*   14     TK          "     kinetic energy    "
*   15     TPHI        "     azimuth angle
*   16     TTHETA      "     polar angle
C>>>>>>>ANALYSIS RESULTS..FOR THE WHOLE TRACK<<<<<<<<<<<<<<<<<<
*   17     ACCEPT   Acceptance Flag (+/-1=acc. in +/- wing, 2=both, 0=neither)
*   18     NHITSL   number of hits on track, set to a maximum of NHTM
*   19     USR1     spare, for later use
*   20     USR2              "
C>>>>>>>TRACK HITS<<<<<<<<
* The Truth ntuple stops here; the Hits ntuple continues with
* A series of particle history words
* NHITSL groups of 12 words each, at index = 32 + 12*(Ihit-1):
*
*   21     KCASE    creation mechanism
*   22     PARENT   track number
*   23     VOLUME   creation volume name
*   24     XTERM
*   25     YTERM
*   26     ZTERM
*   27     PXTERM
*   28     PYTERM
*   29     PZTERM
*   30     ETERM
*   31     STOP     stop mechanism

*   32+    PL       plane number hit
*   33+    X        Track center in plane in master coords (maybe pixeled)
*   34+    Y          "     "     "   "    "  "      "       "       "
*   35+    Z          "     "     "   "    "  "      "       "       "
*   36+    DE       Energy deposited in the plane
*   37+    DX       Exit minus entry position in master coord system
*   38+    DY        "    "    "    "      "        "    "
*   39+    DZ        "    "    "    "      "        "    "
*   40+    EFIN     track energy at exit from plane
*   41+    TOF      time registered by a TOF detector, in seconds
*   42+    ITRK     track number for this hit
*   43+    IS       Combined information:  IS = IVOL + 1000 * ISEQ
*                   ISEQ = sequential (first = 1) hit number on this track
*                   IVOL = system volume number of the plane
C The maximal number of hits is now NHTM = 17
C There are thus 31 + 12*NHTM or 235 entries in a Hits Ntuple, with zeroes
C after the last hit

      INTEGER NVMCTUP,NVTUP,NTOFF,NHTM
      PARAMETER (NTOFF=31, NVMCTUP=20, NHTM=17)
      PARAMETER (NVTUP=31+12*NHTM)

      REAL    RUN, EVENT, TRACK, GEN, ANAL, SCAT, BIMP
      REAL    XPROD, YPROD, ZPROD, IDPART, CHRG, TP, TK, TPHI, TTHETA
      REAL    ACCEPT, NHITSL, USR1, USR2
      REAL    STOP, PARENT, VOLTERM, XTERM, YTERM, ZTERM, PXTERM, PYTERM
      REAL    PZTERM, ETERM, UNUSED
      REAL    TRK(12,NHTM), TUP(NVTUP)

      COMMON/RHTUP/ RUN, EVENT, TRACK, GEN, ANAL, SCAT, BIMP,
     1  XPROD, YPROD, ZPROD, IDPART, CHRG, TP, TK, TPHI, TTHETA,
     2  ACCEPT, NHITSL, USR1, USR2, STOP, PARENT, VOLTERM, XTERM,
     3  YTERM, ZTERM, PXTERM, PYTERM, PZTERM, ETERM, UNUSED,
     4  TRK

      EQUIVALENCE (TUP(1),RUN)

      COMMON /NTTAGS/ TAGS(NVTUP)
      CHARACTER*8 TAGS
*KEEP,NTUD.
C Data for Ntuple element tag names.                     File NTUD.INC
C Variables are in common NTTAGS, defined in NTU.INC.
C 20-APR-95  R. V.      Change last tagnames to ... TFn, ITn, SVn
C 16-MAR-95  K. W.      Increase the number of planes to 15
C 19-Jul-94  R. V.      Update tagnames: Cxx -> Dxx, VOLTERM -> VOLUME
C 10-May-94  R.Verdier  Extracted from NTU.INC with new HITS info
      DATA TAGS/'RUN', 'EVENT', 'TRACK', 'GEN', 'ANAL', 'SCAT', 'BIMP',
     1 'XPROD', 'YPROD', 'ZPROD', 'IDPART', 'CHRG', 'TP', 'TK',
     2 'TPHI', 'TTHETA', 'ACCEPT', 'NHITSL', 'USR1', 'USR2', 'STOP',
     3 'PARENT', 'VOLUME', 'XTERM', 'YTERM', 'ZTERM', 'PXTERM',
     4 'PYTERM', 'PZTERM', 'ETERM', 'UNUSED',
     5 'PL1 ','X1  ','Y1  ','Z1  ','DE1 ','DX1 ','DY1 ','DZ1 ','EF1 ',
     6 'TF1 ','IT1 ','SV1 ',
     7 'PL2 ','X2  ','Y2  ','Z2  ','DE2 ','DX2 ','DY2 ','DZ2 ','EF2 ',
     8 'TF2 ','IT2 ','SV2 ',
     9 'PL3 ','X3  ','Y3  ','Z3  ','DE3 ','DX3 ','DY3 ','DZ3 ','EF3 ',
     * 'TF3 ','IT3 ','SV3 ',
     1 'PL4 ','X4  ','Y4  ','Z4  ','DE4 ','DX4 ','DY4 ','DZ4 ','EF4 ',
     2 'TF4 ','IT4 ','SV4 ',
     3 'PL5 ','X5  ','Y5  ','Z5  ','DE5 ','DX5 ','DY5 ','DZ5 ','EF5 ',
     4 'TF5 ','IT5 ','SV5 ',
     5 'PL6 ','X6  ','Y6  ','Z6  ','DE6 ','DX6 ','DY6 ','DZ6 ','EF6 ',
     6 'TF6 ','IT6 ','SV6 ',
     7 'PL7 ','X7  ','Y7  ','Z7  ','DE7 ','DX7 ','DY7 ','DZ7 ','EF7 ',
     8 'TF7 ','IT7 ','SV7 ',
     9 'PL8 ','X8  ','Y8  ','Z8  ','DE8 ','DX8 ','DY8 ','DZ8 ','EF8 ',
     * 'TF8 ','IT8 ','SV8 ',
     1 'PL9 ','X9  ','Y9  ','Z9  ','DE9 ','DX9 ','DY9 ','DZ9 ','EF9 ',
     2 'TF9 ','IT9 ','SV9 ',
     3 'PL10','X10 ','Y10 ','Z10 ','DE10','DX10','DY10','DZ10','EF10',
     4 'TF10','IT10','SV10',
     5 'PL11','X11 ','Y11 ','Z11 ','DE11','DX11','DY11','DZ11','EF11',
     6 'TF11','IT11','SV11',
     7 'PL12','X12 ','Y12 ','Z12 ','DE12','DX12','DY12','DZ12','EF12',
     8 'TF12','IT12','SV12' ,
     9 'PL13','X13 ','Y13 ','Z13 ','DE13','DX13','DY13','DZ13','EF13',
     * 'TF13','IT13','SV13',
     1 'PL14','X14 ','Y14 ','Z14 ','DE14','DX14','DY14','DZ14','EF14',
     2 'TF14','IT14','SV14',
     3 'PL15','X15 ','Y15 ','Z15 ','DE15','DX15','DY15','DZ15','EF15',
     4 'TF15','IT15','SV15',
     5 'PL16','X16 ','Y16 ','Z16 ','DE16','DX16','DY16','DZ16','EF16',
     6 'TF16','IT16','SV16',
     7 'PL17','X17 ','Y17 ','Z17 ','DE17','DX17','DY17','DZ17','EF17',
     8 'TF17','IT17','SV17' /
*KEEP,NTUEX.
c NTUEX.INC: PHOBOS MC Ntuple option extensions
c 10-May-94 R. V.     Correct filenames in documentation
c 6-Jun-93 R. V.     Creation
c This include file defines options for the size and format of the
c Phobos ntuples.  Data values are assigned in NTUEXD.INC.
c Define sizes for allocating ntuples, in 4-byte words.  The NHtrks
c and NMCtrks values limit the total size of the created disk file.
c The default values assigned to NWHits and NWMC here are actually
c overridden by Nvtup and Nvmctup, respectively, in NTUD.INC.
      common /NTUEX/ LreclW, NWHits, NWMC, NWover, NWPRIM,
     1               NHtrks, NMCtrks
      integer
     1  LreclW,  ! Disk file logical record length, words
     2  NWHits,  ! # Hits ntuple variables [columns]
     3  NWMC,    ! # MC Truth ntuple variables [columns]
     4  NWover,  ! Approx. Zebra system overhead, words
     5  NWprim,  ! Primary storage buffer, words
     6  NHtrks,  ! Maximum # of tracks in Hits ntuple
     7  NMCtrks  ! Maximum # of tracks in MC ntuple
*KEEP,NTUEXD.
c NTUEXD.INC: PHOBOS MC Ntuple option extensions data
c 10-May-94 R. V.      NWHits and NWMC changed for new HITS information
c 6-Jun-93 R. V.     Creation
c This include file specifies data values for common NTUEX, defined
c in NTUEX.INC.
c Define sizes for allocating ntuples, in 4-byte words.  The NHtrks
c and NMCtrks values limit the total size of the created disk file.
c The default values assigned to NWHits and NWMC here are actually
c overridden by Nvtup and Nvmctup, respectively, in NTUD.INC.
      data
     1  LreclW /  1024 /, ! Disk file logical record length, words
     2  NWHits /   175 /, ! # Hits ntuple variables [columns]
     3  NWMC   /    20 /, ! # MC Truth ntuple variables [columns]
     4  NWover /  3500 /, ! Approx. Zebra system overhead, words
     5  NWprim /  8192 /, ! Primary storage buffer, words
     6  NHtrks / 60000 /, ! Maximum # of tracks in Hits ntuple
     7  NMCtrks/ 60000 /  ! Maximum # of tracks in MC ntuple
*KEEP,RHSIL.
C RHSIL.CMN: PHOBOS detector plane dimensions, positions, and
c granularity; UGC parameters
c The actual and maximum numbers of detector planes and detector hit
c parameters.  Their default values are defined in RHSILD.INC.
c 12-Dec-97 K. W.       PADSOLD and PADSNEW from UGCSET moved to a common
c  4-Nov-97 K. W.       Increased number of PADS parameters to 10
c                       Increased NSLPM to 800
c  1-Jul-97 K. W.       Added data for GEANT pixelization (GSDVN)
c  1-May-97 R. V.       Increased NSLPM to 500
c  3-MAR-97 K. W.       Removed SPL, added PADS data
C  3-Jan-97 K. W.       Increased NSLPM to 180
c  3-Jul-96 K. W.       Geometry version number added
c  5-Apr-96 D. McL.     Moved data value of UgcpMax to RHSILD
c  ...
c    Mar-92 D.McLeod    Creation
c Storage limit for number of silicon planes:
      INTEGER NSLPM
      PARAMETER (NSLPM=800)
c Storage limit for number of parameters for each hit:
      integer NSipMaxp
      parameter (NSipMaxp = 12)
c Safety limit for number of parameter values that may be entered in
c a UGC text input record:
      integer UgcpMaxp
      parameter (UgcpMaxp = 10)
c Number of silicon planes in use:
      integer NSLP
c Additional information on detectors - pad sizes:
C  DETNAME  - name of the detector volume
C  DIVXNAME - name for division in X direction
C  DIVYNAME - name for division in Y direction
C  DIVZNAME - name for division in Z direction
C  NDIVX    - number of divisions in X direction
C  NDIVY    - number of divisions in Y direction
C  NDIVZ    - number of divisions in Z direction
C  IPADFL   - pad type flag
C  PADn   - four pad parameters
C             (pad size in X, Y, Z direction for IPADFL=1 for example)
      CHARACTER*4 DETNAME(NSLPM), DIVXNAME(NSLPM),
     +             DIVYNAME(NSLPM), DIVZNAME(NSLPM)
      INTEGER NDIVX(NSLPM), NDIVY(NSLPM), NDIVZ(NSLPM)
      REAL PAD1(NSLPM),PAD2(NSLPM),PAD3(NSLPM),PAD4(NSLPM),PAD5(NSLPM)
      REAL PAD6(NSLPM),PAD7(NSLPM),PAD8(NSLPM),PAD9(NSLPM),PAD10(NSLPM)
      INTEGER NDETS, IPADFL(NSLPM)
      LOGICAL PADSOLD, PADSNEW
      COMMON /RHPADS/ NDETS, PADSOLD, PADSNEW,
     +       DETNAME, DIVXNAME, DIVYNAME, DIVZNAME,
     +       NDIVX, NDIVY, NDIVZ,
     +       IPADFL, PAD1, PAD2, PAD3, PAD4, PAD5,
     +               PAD6, PAD7, PAD8, PAD9, PAD10
c Number of silicon planes - compared with final value of NDETS
      COMMON /RHSIL/ NSLP
c UGC parameter information:
      integer UgcpMax
      real UGCpars(UgcpMaxp)
c Declare silicon plane parameters; their default values are specified
c in RHSILD.INC
      integer NSIP
c Names of parameters:
      CHARACTER*4 SINAMH(NSipMaxp)
c Mumber of bits stored for each parameter:
      INTEGER SIBITH(NSipMaxp)
c Multiplicative scale factors and origins, defined so that
c   Stored value = (Parameter value + SIORIG) * SIFACT:
      REAL SIFACT(NSipMaxp), SIORIG(NSipMaxp)
c Common for access to UGC parameters and hit parameter definitions:
      common /UGCPAR/ UgcpMax, UGCpars,
     1                NSIP, SINAMH, SIBITH, SIORIG, SIFACT
      real GEOV  !  geometry version number
      common /ugGEOV/ GEOV
*KEEP,RHSILD.
C RHSILD.INC: Data for PHOBOS detector hit parameters
c  7-Jul-98 K. W.   NSIP - number of hits parameters is now 12
c 12-Dec-97 K. W.   Added PADSNEW and PADSOLD initialization
c 26-Sep-95 D. McL. Moved data value for UgcpMax from RHSIL; UgcpMaxp is
c                   defined there
c 10-May-94 R. V.  Extracted from RHSIL.INC, added Don McLeod's mods
c Number of hit parameters:
      data NSIP /12/
c Names of parameters:
      DATA SINAMH / 'X   ', 'Y   ', 'Z   ', 'DE  ', 'COSX', 'COSY',
     1              'COSZ', 'EFIN', 'IVOL', 'ITRK', 'ISEQ', 'TLNG'/
c Mumber of bits stored for each parameter:
      DATA SIBITH / 12*32 /
c Multiplicative scale factors and origins, defined so that
c   Stored value = (Parameter value + SIORIG) * SIFACT:
      DATA SIORIG / 1.e3, 1.e3, 1.e3, 0.,  1.e3, 1.e3, 1.e3,
     +                                      0.,   0.,  0.,  0., 0. /
      DATA SIFACT / 1.e5, 1.e5, 1.e5, 1.e7, 1.e5, 1.e5, 1.e5,
     +                                    1.e4, 1.e11, 1.,  1., 1.e5 /
c Number of UGC parameters
      DATA UgcpMax /UgcpMaxp/
c Pads type definition unknown at first
      DATA PADSNEW, PADSOLD / .FALSE., .FALSE. /
*KEEP,IO.
C Common block for GPH logical units etc.: file IO.CMN
c 13-Aug-96  R. V.       Added USRBATCHK for KUIP commands for UGCONS
c 21-Jul-96  R. V.       Remove unused record; note CARDDEF now obsolete
c 22-Feb-93  R. V.       Added USRMCNTU and IMCNTU for MC Ntuple
c 14-Nov-92  R. Verdier  Added UGCIN, IUGCIN, CARDDEF and ICARDDEF
c                        for UGC and EUCARD (obsolete) input units
C    Apr-92  C. Parsons  Replacement for original COMMON.CDT
      INTEGER USRGCIN, CARDDEF, USRIN, USROUT, USRMAP, USRNTU, USRZEB,
     *  USRMCNTU, USRBATKUMAC
      PARAMETER (USRIN = 4, USROUT = 16, USRMAP = 18)
      PARAMETER (USRNTU = 15, USRZEB = 17, USRGCIN = 19, CARDDEF = 20)
      PARAMETER (USRMCNTU = 21, USRBATKUMAC = 22)
      INTEGER ICARDDEF, IMCNTU, IUSRGCIN, IUSRIN, IUSROUT, IUSRMAP,
     *  IUSRNTU, IUSRZEB, IUSRAOU, IUSRBATKUMAC
      COMMON /IO/ IUSRIN, IUSROUT, IUSRMAP, IUSRNTU, IUSRZEB, IUSRGCIN,
     *  ICARDDEF, IMCNTU, IUSRAOU, IUSRBATKUMAC
*KEEP,FILES.
C Common block for UFILE routine
c 22-Aug-97  K. W.       Creation

      integer MaxFiles
      parameter (MaxFiles = 12)

c Preassigned files:
c RHDATA  is for FFREAD and STPARS control input
c GCDEFS  is for UGC system geometry and control input
c CARDS$DEF is for interactive input of FFREAD and STPARS data
c RHASC is for standard debug output
c BATKUMAC is for a KUIP batch command file
c FLDASC is for ascii-format magnetic field data
c RHZEB is for Zebra output
c RHNTU is for Ntuple output for tracks containing hits only
c MCNTU is for Ntuple output for all tracks
c GENEVS is for the ascii track input file
c The corresponding standard logical unit numbers usrin, ... are
c defined in IO.CMN now, but eventually should be defined directly by
c the definition for Luns below.
c Additional numbers 101, ... are pre-set for formatted ascii output
c files; for other file types, the user should call with mode = 1 and
c do his own OPEN.

      integer NFiles, NFilesPre
      character*32 LogNames(MaxFiles)    !  logical names of files
      character*63 FileNames(MaxFiles)   !  actual names of files
      integer Luns(MaxFiles)             !  FORTRAN logical units
      character*2 LogIndex(MaxFiles)     !  actual index of LogName
c                                        !  (excluding first file)

      integer Iform(MaxFiles)            !  FORMATTED/UNFORMATTED switch
      integer Irw(MaxFiles)              !  READONLY switch
      integer Istat(MaxFiles)            !  NEW/OLD/UNKNOWN switch

      common /cfiles/ NFiles, NfilesPre, LogNames, FileNames, Luns,
     + LogIndex, Iform, Irw,  Istat
*KEEP,FILESD.
C Common block for UFILE routine - DATA STATEMENTS
c 23-Feb-98  K. W.       Status for RHASC is always UNKNOWN
c 22-Aug-97  K. W.       Creation

      DATA NFiles /10/
      DATA NFilesPre /10/
      DATA LogNames
     * /'RHDATA',  'GCDEFS',  'CARDS$DEF', 'RHASC', 'BATKUMAC',
     *  'FLDASC',  'RHZEB',   'RHNTU',     'MCNTU', 'GENEVS',    2*' '/

      DATA  Luns
     * / usrin,   usrgcin,   carddef,     usrout,     usrbatkumac,
     *   usrmap,  usrzeb,    usrntu,      usrmcntu,   91,      92, 93 /

      DATA  LogIndex /  MaxFiles*'?' /

c Iform = 1 for FORMATTED, 2 for UNFORMATTED, 3 for unspecified
      DATA  Iform
     * /   1,       1,         1,            1,           1,
     *     1,       2,         1,            1,           1,    2*1/

c Irw = 1 for READONLY, 2 for normal READ/WRITE
      DATA Irw
     * /   1,       1,         1,            2,           1,
     *     1,       2,         2,            2,           1,    2*2/

c Istat = 1 for OLD, 2 for NEW, 3 for UNKNOWN
c Created file status: UNKNOWN for OSF1, NEW for other o.s.'s
c*****DATA Istat / 1, 1, 1, 3, 1,   1, 3, 3, 3, 1,   2*3/
      DATA Istat / 1, 1, 1, 3, 1,   1, 2, 2, 2, 1,   2*2/
*KEND.


      end
