      SUBROUTINE GETPMCPAR(NAME, NVALUES, VALUES)
*Documentation_Block
************************************************************************
*
*  Description:
*  Get from PMC parameters of the simulations
*  NOTE: the supported parameters list should be identical with
*        those in  SETPMCPAR  and  GETPMCPNAME
*
*  Arguments:
*  NAME    - parameter keyword
*  NVALUES - number of values for this parameter
*            (ERROR signalled by 0)
*  VALUES  - table of parameter values
*
*  Called by:
*  Phat
*
*  Calls:
*
*  Implicit inputs, outputs, side effects:
*  Reads information from standard input - or from a common /CGETKINE/
*
*  History:
*  Date       Name       Description
*  ----       ----       -----------
*   17-Dec-98  K.W.    Added almost all control file parameters
*    7-Jul-98  K.W.    Added TRIG
*   27-Apr-98  K.W.    Creation
************************************************************************
*End_Documentation_Block

      IMPLICIT NONE

      CHARACTER*4 NAME
      INTEGER  NVALUES
      REAL VALUES(10)

*KEEP,GCCUTS.
      COMMON/GCCUTS/CUTGAM,CUTELE,CUTNEU,CUTHAD,CUTMUO,BCUTE,BCUTM
     +             ,DCUTE ,DCUTM ,PPCUTM,TOFMAX,GCUTS(5)
C
      REAL          CUTGAM,CUTELE,CUTNEU,CUTHAD,CUTMUO,BCUTE,BCUTM
     +             ,DCUTE ,DCUTM ,PPCUTM,TOFMAX,GCUTS
C
*KEEP,GCPHYS.
      COMMON/GCPHYS/IPAIR,SPAIR,SLPAIR,ZINTPA,STEPPA
     +             ,ICOMP,SCOMP,SLCOMP,ZINTCO,STEPCO
     +             ,IPHOT,SPHOT,SLPHOT,ZINTPH,STEPPH
     +             ,IPFIS,SPFIS,SLPFIS,ZINTPF,STEPPF
     +             ,IDRAY,SDRAY,SLDRAY,ZINTDR,STEPDR
     +             ,IANNI,SANNI,SLANNI,ZINTAN,STEPAN
     +             ,IBREM,SBREM,SLBREM,ZINTBR,STEPBR
     +             ,IHADR,SHADR,SLHADR,ZINTHA,STEPHA
     +             ,IMUNU,SMUNU,SLMUNU,ZINTMU,STEPMU
     +             ,IDCAY,SDCAY,SLIFE ,SUMLIF,DPHYS1
     +             ,ILOSS,SLOSS,SOLOSS,STLOSS,DPHYS2
     +             ,IMULS,SMULS,SOMULS,STMULS,DPHYS3
     +             ,IRAYL,SRAYL,SLRAYL,ZINTRA,STEPRA
      COMMON/GCPHLT/ILABS,SLABS,SLLABS,ZINTLA,STEPLA
     +             ,ISYNC
     +             ,ISTRA
*
      INTEGER IPAIR,ICOMP,IPHOT,IPFIS,IDRAY,IANNI,IBREM,IHADR,IMUNU
     +       ,IDCAY,ILOSS,IMULS,IRAYL,ILABS,ISYNC,ISTRA
      REAL    SPAIR,SLPAIR,ZINTPA,STEPPA,SCOMP,SLCOMP,ZINTCO,STEPCO
     +       ,SPHOT,SLPHOT,ZINTPH,STEPPH,SPFIS,SLPFIS,ZINTPF,STEPPF
     +       ,SDRAY,SLDRAY,ZINTDR,STEPDR,SANNI,SLANNI,ZINTAN,STEPAN
     +       ,SBREM,SLBREM,ZINTBR,STEPBR,SHADR,SLHADR,ZINTHA,STEPHA
     +       ,SMUNU,SLMUNU,ZINTMU,STEPMU,SDCAY,SLIFE ,SUMLIF,DPHYS1
     +       ,SLOSS,SOLOSS,STLOSS,DPHYS2,SMULS,SOMULS,STMULS,DPHYS3
     +       ,SRAYL,SLRAYL,ZINTRA,STEPRA,SLABS,SLLABS,ZINTLA,STEPLA
C
*KEEP,GCFLAG.
      COMMON/GCFLAG/IDEBUG,IDEMIN,IDEMAX,ITEST,IDRUN,IDEVT,IEORUN
     +        ,IEOTRI,IEVENT,ISWIT(10),IFINIT(20),NEVENT,NRNDM(2)
      COMMON/GCFLAX/BATCH, NOLOG
      LOGICAL BATCH, NOLOG
C
      INTEGER       IDEBUG,IDEMIN,IDEMAX,ITEST,IDRUN,IDEVT,IEORUN
     +        ,IEOTRI,IEVENT,ISWIT,IFINIT,NEVENT,NRNDM
C
*KEEP,BANGC.
C Event generator parameters                 File: BANGC.CMN
C 20-Sep-96  K. Wozniak Added new parameters to control BANG mode = 2
C                       RSPX, RSPZ, RSPR (removed RMAG)
C 10-Mar-95  K. Wozniak Added new parameters to control track generation
c                       and selection (ETAMIN,ETAMAX, IDMIN,IDMAX ..)
c 22-Jun-94  R. V.      Remove NSEQN storage (see PH_HIT)
c 10-May-94  D. McLeod  Increased MXMUL to 13000 for HIJET events
c 17-Dec-92  R. Verdier Changed name from BANG to BANGC for CMS
c 22-Nov-92  D. Mcleod  Added EXF and NSEQN
      INTEGER MXMUL
      PARAMETER (MXMUL=13000)  ! maximum multiplicity for BANG etc.
      INTEGER IRMUL         !  multiplicity in random particle generation
      INTEGER IRARM         !  particles going to 1 or 2 arms selected
      INTEGER IRPID(2)      !  particle id range: min, max
      REAL RMOM(3)          !  momentum of particles:  min, max and step
                            !  (first step if 1/p is changed)
      REAL REXP             !  mean transv. mom. - for momentum generation
      REAL RETA(2)          !  ETA range: min, max
      REAL RANG(3)          !  starting theta angle and step
      REAL RPHI(2)          !  range of emission phi angle: min, max
      REAL RROT(2)          !  phi initial and additional rotation angle (GEN=0)
      REAL RBXX,RBYY,RBZZ   !  central position of the vertex
      REAL RBXY             !  width of the beam intersection in x-y
      REAL RBDX,RBDY,RBDZ   !  the range of positions of the vertex
C                           !
      REAL RSPX,RSPZ,RSPR   !  position and radius of the circle describing
C                           !  area covered by spectrometer
      REAL PHIR,CZ,ZC,RS    !  working variables for BANG
      COMMON/RHBNG/IRMUL, IRARM, IRPID, RMOM, REXP, RETA, RANG, RPHI,
     1             RROT,
     2             RBXX,RBYY,RBZZ, RBDX, RBDY, RBDZ, RBXY,
     3             RSPX,RSPZ,RSPR, PHIR,CZ,ZC,RS
*KEEP,FLAG.
C  PHOBOS flags, run control, operating modes, misc.   File FLAG.CMN
C   7-Jul-98  K. W.      Added  ITRIG flag (initial numer in TRIG command)
C  27-Nov-97  K. W.      Added IAOUF flag
C  31-Mar-95  R.V.       Change STRA to SECT to avoid GEANT 3.21 conflict
C  22-Nov-92  D. McLeod
C   These are set in input cards
      INTEGER ISASC,ISHBO,ISHDO,ISNTU, ISMEM,ISAOU,IAOUF, ISECT,IDRAW,
     1        IGEN,IANAL,ISCAT,IPIXL,ISEQ,NTRK,IZEB, ITRIG
      REAL WPIXL,WLOSS
      COMMON/RHFLG/ISASC,ISHBO,ISHDO,ISNTU,
     1  ISMEM,ISAOU,IAOUF, ISECT,IDRAW,
     2  IGEN,IANAL,ISCAT,IPIXL,ISEQ,NTRK,IZEB,WPIXL,WLOSS, ITRIG
*KEEP,RHSYS.
C  Common blocks for GPH PHOBOS simulation:  FILE RHSYS.CMN
C  General geometric layout of PHOBOS detector exclusive of detectors
C  These values are on input cards or data statements, except *MAX are
C  calculated to encompass all elements of the system.
C  20-Sep-96  K.W       Removed obsolete BMTBRAD, BERTHIK
C                       Removed FANGL - now local variable in BANG
C  12-Jul-96  K.W       Position of the magnet pole center MAGX, MAGZ
C                       (previously XOFF,ZOFF) separated from the magnet
C                       offset (now MOFF - 3-D vector)
C  14 Dec-92  R.V.      TMAG redefined and changed to integer
C  22-Nov-92 D. McLeod  Renamed from COMMON.SYS; TMAG, ZOFF, XOFF
C                       are added from the original GPH version
      REAL   BFLD         ! Magnetic field strength
     &      ,MAGX,MAGZ    ! Coordinates of field center
     &      ,MOFF(3)      ! offset of the field
     &      ,RFLD         ! Radius of field (perfect dipole only)
     1   ,XMAX,YMAX,ZMAX,ZMIN,XMMAX,ZMMAX
      INTEGER MAGT, TMAG  ! Field type: 0 Zebra bank, 1 = dipole,
                          ! 2 = ASCII map
      COMMON/RHSYS/ BFLD,MAGT,MAGX,MAGZ,MOFF
     1  ,RFLD,XMAX,YMAX,ZMAX,ZMIN,XMMAX,ZMMAX
      EQUIVALENCE (TMAG, MAGT)
*KEEP,MUPEP.
c MUPEP.CMN: PHOBOS MC multi-pass event processing variables
c 16-Mar-95 K. W.     NeSkip added
c  6-Jun-93 R. V.     Creation
c Multi-pass event processing breaks up each event into groups of
c tracks, in order to avoid overflowing Zebra track storage.
c Common for access to control variables:
      common /MUPEP/ EPState, NtConc, NtpE, NtPrim, NeSkip, NtSkip,
     1  MPind, NtPrev, NtTot, NtMax, NvPrev
c Parameters and data values are assigned in MUPEP.CDA.
c Event processing state: EPState = 0 for first pass,
c 1 for continuing pass, 2 for final pass
      integer EPState
c Current pass index:
      integer MPind
c Maximum number of tracks to process concurrently in a group:
      integer NtConc
c Maximum track number for this group:
      integer NtMax
c Maximum number of tracks per event:
      integer NtpE
c Number of primary and secondary tracks generated before this pass:
      integer NtPrev
c Number of primary tracks and vertices generated:
      integer NtPrim, NvPrev
c Number of input file events to skip initially:
      integer NeSkip
c Number of tracks to skip initially:
      integer NtSkip
c Total number of primary and secondary tracks generated:
      integer NtTot
*KEND.

      INTEGER I

C
C     Executable Statements
C     =====================
C
      NVALUES = 0

      IF(NAME .EQ. 'CUTS')  THEN
         NVALUES = 5
         VALUES(1) = CUTGAM
         VALUES(2) = CUTELE
         VALUES(3) = CUTNEU
         VALUES(4) = CUTHAD
         VALUES(5) = CUTMUO
      ELSE IF(NAME .EQ. 'MULS')  THEN
         NVALUES = 1
         VALUES(1) = IMULS
      ELSE IF(NAME .EQ. 'DCAY')  THEN
         NVALUES = 1
         VALUES(1) = IDCAY
      ELSE IF(NAME .EQ. 'HADR')  THEN
         NVALUES = 1
         VALUES(1) = IHADR
      ELSE IF(NAME .EQ. 'LOSS')  THEN
         NVALUES = 1
         VALUES(1) = ILOSS
      ELSE IF(NAME .EQ. 'PAIR')  THEN
         NVALUES = 1
         VALUES(1) = IPAIR
      ELSE IF(NAME .EQ. 'BREM')  THEN
         NVALUES = 1
         VALUES(1) = IBREM
      ELSE IF(NAME .EQ. 'COMP')  THEN
         NVALUES = 1
         VALUES(1) = ICOMP
      ELSE IF(NAME .EQ. 'PHOT')  THEN
         NVALUES = 1
         VALUES(1) = IPHOT
      ELSE IF(NAME .EQ. 'DRAY')  THEN
         NVALUES = 1
         VALUES(1) = IDRAY
      ELSE IF(NAME .EQ. 'ANNI')  THEN
         NVALUES = 1
         VALUES(1) = IANNI
      ELSE IF(NAME .EQ. 'GEN ')  THEN
         NVALUES = 1
         VALUES(1) = IGEN
      ELSE IF(NAME .EQ. 'SMEM')  THEN
         NVALUES = 1
         VALUES(1) = ISMEM
      ELSE IF(NAME .EQ. 'SAOU')  THEN
         NVALUES = 1
         VALUES(1) = ISAOU
      ELSE IF(NAME .EQ. 'MAGT')  THEN
         NVALUES = 1
         VALUES(1) = MAGT
      ELSE IF(NAME .EQ. 'TRIG')  THEN
         NVALUES = 1
         VALUES(1) = ITRIG
      ELSE IF(NAME .EQ. 'PIXL')  THEN
         NVALUES = 1
         VALUES(1) = IPIXL
      ELSE IF(NAME .EQ. 'SECT')  THEN
         NVALUES = 1
         VALUES(1) = ISECT
      ELSE IF(NAME .EQ. 'RBXX')  THEN
         NVALUES = 1
         VALUES(1) = RBXX
      ELSE IF(NAME .EQ. 'RBYY')  THEN
         NVALUES = 1
         VALUES(1) = RBYY
      ELSE IF(NAME .EQ. 'RBZZ')  THEN
         NVALUES = 1
         VALUES(1) = RBZZ
      ELSE IF(NAME .EQ. 'RMOM')  THEN
         NVALUES = 3
         VALUES(1) = RMOM(1)
         VALUES(2) = RMOM(2)
         VALUES(3) = RMOM(3)
      ELSE IF(NAME .EQ. 'RPID')  THEN
         NVALUES = 2
         VALUES(1) = IRPID(1)
         VALUES(2) = IRPID(2)
      ELSE IF(NAME .EQ. 'RETA')  THEN
         NVALUES = 2
         VALUES(1) = RETA(1)
         VALUES(2) = RETA(2)
      ELSE IF(NAME .EQ. 'RPHI')  THEN
         NVALUES = 2
         VALUES(1) = RPHI(1)
         VALUES(2) = RPHI(2)
      ELSE IF(NAME .EQ. 'RARM')  THEN
         NVALUES = 1
         VALUES(1) = IRARM
      ELSE IF(NAME .EQ. 'RMUL')  THEN
         NVALUES = 1
         VALUES(1) = IRMUL
      ELSE IF(NAME .EQ. 'RROT')  THEN
         NVALUES = 2
         VALUES(1) = RROT(1)
         VALUES(2) = RROT(2)
      ELSE IF(NAME .EQ. 'REXP')  THEN
         NVALUES = 1
         VALUES(1) = REXP
      ELSE IF(NAME .EQ. 'RANG')  THEN
         NVALUES = 3
         VALUES(1) = RANG(1)
         VALUES(2) = RANG(2)
         VALUES(3) = RANG(3)
      ELSE IF(NAME .EQ. 'RSPX')  THEN
         NVALUES = 1
         VALUES(1) = RSPX
      ELSE IF(NAME .EQ. 'RSPZ')  THEN
         NVALUES = 1
         VALUES(1) = RSPZ
      ELSE IF(NAME .EQ. 'RSPR')  THEN
         NVALUES = 1
         VALUES(1) = RSPR
      ELSE IF(NAME .EQ. 'RBXY')  THEN
         NVALUES = 1
         VALUES(1) = RBXY
      ELSE IF(NAME .EQ. 'RBDX')  THEN
         NVALUES = 1
         VALUES(1) = RBDX
      ELSE IF(NAME .EQ. 'RBDY')  THEN
         NVALUES = 1
         VALUES(1) = RBDY
      ELSE IF(NAME .EQ. 'RBDZ')  THEN
         NVALUES = 1
         VALUES(1) = RBDZ
      ELSE IF(NAME .EQ. 'RNDM')  THEN
         NVALUES = 2
         VALUES(1) = NRNDM(1)
         VALUES(2) = NRNDM(2)
      ELSE IF(NAME .EQ. 'RUNG')  THEN
         NVALUES = 1
         VALUES(1) = IDRUN
      ELSE IF(NAME .EQ. 'ANAL')  THEN
         NVALUES = 1
         VALUES(1) = IANAL
      ELSE IF(NAME .EQ. 'NESK')  THEN
         NVALUES = 1
         VALUES(1) = NESKIP
      ELSE IF(NAME .EQ. 'NTSK')  THEN
         NVALUES = 1
         VALUES(1) = NTSKIP
      ELSE IF(NAME .EQ. 'NTPE')  THEN
         NVALUES = 1
         VALUES(1) = NTPE
      ELSE IF(NAME .EQ. 'NTCO')  THEN
         NVALUES = 1
         VALUES(1) = NTCONC
      ELSE IF(NAME .EQ. 'SWIT')  THEN
         NVALUES = 10
         DO  I=1,10
            VALUES(I) = ISWIT(I)
         ENDDO
      ELSE IF(NAME .EQ. 'DEBU')  THEN
         NVALUES = 3
         VALUES(1) = IDEMIN
         VALUES(2) = IDEMAX
         VALUES(3) = ITEST
      ENDIF

      RETURN
      END
