       Subroutine GUKINE
*Documentation_Block
************************************************************************
*
*  Description:
*  User version of GUKINE loads the KINE structure with tracks.
*
*  Arguments:
*  none
*
*  Called by:
*  GTRIG
*
*  Calls:
*  BANG, GPKINE, GPVERT, GSKINE, GSVERT, LENOCC, RANNOR, UFILE
*  ADDPMCRINFO, ADDPMCEINFO, ADDPMCEOF
*
*  Implicit inputs, outputs, side effects:
*  Loads KINE structure; uses user KINE buffer.  Variables in MUPEP
*  control # tracks processed concurrently.
*
*  Filename: PMC_SRC:GUKINE.FOR
*
*  History:
*  Date       Name       Description
*  ----       ----       -----------
*  23-Nov-98  K. W.      Added handling of information lines in the ADAT file
*  14-Jul-98  K. W.      Corrected error in BANG mode when IRMUL > NTCO
*                        by calculation of the number of tracks for generation
*                        in BANG (given by a parameter)
*  25-Mar-98  K. W.      RROT denoting initial and additional rotation
*                        and not only additional rotation as previously
*  11-Feb-98  K. W.      Modifications necessary to enable interactive
*                        definition of simulated particles
*   5-Nov-97  K. W.      variables for server call in SELF,IF=-NOSERV.
*   7-Feb-97  K. W.      corrected error in phi selection criterion
*  21-Nov-96  K. W.      corrected infinite loop in the case of non existing
*                        input file
*  25-Jul-96  R. V.      Remove MUPEPD include for ANSI compliance
*  21-Jul-96  R. V.      Remove unused plb, ier, pp, pt
*  05-Jul-96  K.Wozniak  Information from the file copied to Ascii OUtput file
*  04-Apr-95  D. McLeod  UFILE replaced by SERVER as part of UNIX port
*  07-Aug-95  R. Verdier RBZZ sets VERT(3), not VERT(2)
*  10-Jul-95  K. Wozniak Changes in the meaning of the GEN parameter:
*                        GEN = 0  - tracks from an input file
*                        GEN = 1-10 - tracks from BANG generator
*                        GEN = 11-20 - previously GEN=4 mode and extentions
*                        New step mode added (step  1/p)
*  11-Apr-95  K. Wozniak Corrected calculations in mode=4 -
*                        removed dependence of momentum on emmision angle
*  17-Mar-95  K. Wozniak Skipping of events and tracks at the begining
*                        of input data file added
*  10-Mar-95  K. Wozniak Changed the meaning of the GEN parameter:
*                        GEN = 1 - random flat distributions of p,eta,phi,vertex
*                        GEN = 2 - tracks covering the spectrometer acceptance
*                        GEN = 3 - random tracks with more "physical" distr.
*                        GEN = 4 - tracks with a parameter changing continously
*                        GEN = 5 - tracks from an input file
*                        The selection of momentum, phi etc. is performed
*                        for each GEN mode
*  18-Jul-94  K. Wozniak Initialize NtPrev
*  09-Nov-93  R. V.      MUPEP.CDA -> MUPEPD.CMN, BANG.CMN -> BANGC.CMN
*                          for code management
*  05-Jun-93  R. V.      Multipass mods: read or generate only NtConc
*                        tracks per pass, Ntpe tracks per event;
*                        see MUPEP.CMN, MUPEP.CDA for definitions.
*                        Use UFILE for track input on logical GENEVS,
*                        replacing FOR037.
*  27-Jun-92  R. V.      Add user buffer UJKBUF to JKINE
*  27-Mar-92  C. Parsons Separate from GPH; add track read-in
*     Mar-92  D. Mcleod  Creation in GPH package
************************************************************************
*End_Documentation_Block

      IMPLICIT NONE

*KEEP,FLAG.
C  PHOBOS flags, run control, operating modes, misc.   File FLAG.CMN
C   7-Jul-98  K. W.      Added  ITRIG flag (initial numer in TRIG command)
C  27-Nov-97  K. W.      Added IAOUF flag
C  31-Mar-95  R.V.       Change STRA to SECT to avoid GEANT 3.21 conflict
C  22-Nov-92  D. McLeod
C   These are set in input cards
      INTEGER ISASC,ISHBO,ISHDO,ISNTU, ISMEM,ISAOU,IAOUF, ISECT,IDRAW,
     1        IGEN,IANAL,ISCAT,IPIXL,ISEQ,NTRK,IZEB, ITRIG
      REAL WPIXL,WLOSS
      COMMON/RHFLG/ISASC,ISHBO,ISHDO,ISNTU,
     1  ISMEM,ISAOU,IAOUF, ISECT,IDRAW,
     2  IGEN,IANAL,ISCAT,IPIXL,ISEQ,NTRK,IZEB,WPIXL,WLOSS, ITRIG
*KEEP,FLAG2.
C  PHOBOS flags, run control, operating modes, misc.   File FLAG2.INC
C  14-Nov-95  D. McLeod
*     Glossary:  IPSIG = identity of kill signal sent by user as interrupt
*                EXSERV = flag for existence of external server file
*                SERNAME = name part only of server file, also run name
*                ROOTPATH = program environment pathname (no trailing /)
*                ROOTVERS = version subdirectory e.g. /prod, /test, /dev
*                SIMVER  = version number for the PHOBOS Monte Carlo
      INTEGER IPSIG
      REAL SIMVER
      LOGICAL EXSERV
      CHARACTER SERNAME*20, ROOTPATH*60, ROOTVERS*10
      COMMON/RHFLG2/IPSIG, EXSERV, SIMVER, SERNAME, ROOTPATH, ROOTVERS
*KEEP,IO.
C Common block for GPH logical units etc.: file IO.CMN
c 13-Aug-96  R. V.       Added USRBATCHK for KUIP commands for UGCONS
c 21-Jul-96  R. V.       Remove unused record; note CARDDEF now obsolete
c 22-Feb-93  R. V.       Added USRMCNTU and IMCNTU for MC Ntuple
c 14-Nov-92  R. Verdier  Added UGCIN, IUGCIN, CARDDEF and ICARDDEF
c                        for UGC and EUCARD (obsolete) input units
C    Apr-92  C. Parsons  Replacement for original COMMON.CDT
      INTEGER USRGCIN, CARDDEF, USRIN, USROUT, USRMAP, USRNTU, USRZEB,
     *  USRMCNTU, USRBATKUMAC
      PARAMETER (USRIN = 4, USROUT = 16, USRMAP = 18)
      PARAMETER (USRNTU = 15, USRZEB = 17, USRGCIN = 19, CARDDEF = 20)
      PARAMETER (USRMCNTU = 21, USRBATKUMAC = 22)
      INTEGER ICARDDEF, IMCNTU, IUSRGCIN, IUSRIN, IUSROUT, IUSRMAP,
     *  IUSRNTU, IUSRZEB, IUSRAOU, IUSRBATKUMAC
      COMMON /IO/ IUSRIN, IUSROUT, IUSRMAP, IUSRNTU, IUSRZEB, IUSRGCIN,
     *  ICARDDEF, IMCNTU, IUSRAOU, IUSRBATKUMAC
*KEEP,BANGC.
C Event generator parameters                 File: BANGC.CMN
C 20-Sep-96  K. Wozniak Added new parameters to control BANG mode = 2
C                       RSPX, RSPZ, RSPR (removed RMAG)
C 10-Mar-95  K. Wozniak Added new parameters to control track generation
c                       and selection (ETAMIN,ETAMAX, IDMIN,IDMAX ..)
c 22-Jun-94  R. V.      Remove NSEQN storage (see PH_HIT)
c 10-May-94  D. McLeod  Increased MXMUL to 13000 for HIJET events
c 17-Dec-92  R. Verdier Changed name from BANG to BANGC for CMS
c 22-Nov-92  D. Mcleod  Added EXF and NSEQN
      INTEGER MXMUL
      PARAMETER (MXMUL=13000)  ! maximum multiplicity for BANG etc.
      INTEGER IRMUL         !  multiplicity in random particle generation
      INTEGER IRARM         !  particles going to 1 or 2 arms selected
      INTEGER IRPID(2)      !  particle id range: min, max
      REAL RMOM(3)          !  momentum of particles:  min, max and step
                            !  (first step if 1/p is changed)
      REAL REXP             !  mean transv. mom. - for momentum generation
      REAL RETA(2)          !  ETA range: min, max
      REAL RANG(3)          !  starting theta angle and step
      REAL RPHI(2)          !  range of emission phi angle: min, max
      REAL RROT(2)          !  phi initial and additional rotation angle (GEN=0)
      REAL RBXX,RBYY,RBZZ   !  central position of the vertex
      REAL RBXY             !  width of the beam intersection in x-y
      REAL RBDX,RBDY,RBDZ   !  the range of positions of the vertex
C                           !
      REAL RSPX,RSPZ,RSPR   !  position and radius of the circle describing
C                           !  area covered by spectrometer
      REAL PHIR,CZ,ZC,RS    !  working variables for BANG
      COMMON/RHBNG/IRMUL, IRARM, IRPID, RMOM, REXP, RETA, RANG, RPHI,
     1             RROT,
     2             RBXX,RBYY,RBZZ, RBDX, RBDY, RBDZ, RBXY,
     3             RSPX,RSPZ,RSPR, PHIR,CZ,ZC,RS
*KEEP,GCFLAG.
      COMMON/GCFLAG/IDEBUG,IDEMIN,IDEMAX,ITEST,IDRUN,IDEVT,IEORUN
     +        ,IEOTRI,IEVENT,ISWIT(10),IFINIT(20),NEVENT,NRNDM(2)
      COMMON/GCFLAX/BATCH, NOLOG
      LOGICAL BATCH, NOLOG
C
      INTEGER       IDEBUG,IDEMIN,IDEMAX,ITEST,IDRUN,IDEVT,IEORUN
     +        ,IEOTRI,IEVENT,ISWIT,IFINIT,NEVENT,NRNDM
C
*KEEP,GCPHYS.
      COMMON/GCPHYS/IPAIR,SPAIR,SLPAIR,ZINTPA,STEPPA
     +             ,ICOMP,SCOMP,SLCOMP,ZINTCO,STEPCO
     +             ,IPHOT,SPHOT,SLPHOT,ZINTPH,STEPPH
     +             ,IPFIS,SPFIS,SLPFIS,ZINTPF,STEPPF
     +             ,IDRAY,SDRAY,SLDRAY,ZINTDR,STEPDR
     +             ,IANNI,SANNI,SLANNI,ZINTAN,STEPAN
     +             ,IBREM,SBREM,SLBREM,ZINTBR,STEPBR
     +             ,IHADR,SHADR,SLHADR,ZINTHA,STEPHA
     +             ,IMUNU,SMUNU,SLMUNU,ZINTMU,STEPMU
     +             ,IDCAY,SDCAY,SLIFE ,SUMLIF,DPHYS1
     +             ,ILOSS,SLOSS,SOLOSS,STLOSS,DPHYS2
     +             ,IMULS,SMULS,SOMULS,STMULS,DPHYS3
     +             ,IRAYL,SRAYL,SLRAYL,ZINTRA,STEPRA
      COMMON/GCPHLT/ILABS,SLABS,SLLABS,ZINTLA,STEPLA
     +             ,ISYNC
     +             ,ISTRA
*
      INTEGER IPAIR,ICOMP,IPHOT,IPFIS,IDRAY,IANNI,IBREM,IHADR,IMUNU
     +       ,IDCAY,ILOSS,IMULS,IRAYL,ILABS,ISYNC,ISTRA
      REAL    SPAIR,SLPAIR,ZINTPA,STEPPA,SCOMP,SLCOMP,ZINTCO,STEPCO
     +       ,SPHOT,SLPHOT,ZINTPH,STEPPH,SPFIS,SLPFIS,ZINTPF,STEPPF
     +       ,SDRAY,SLDRAY,ZINTDR,STEPDR,SANNI,SLANNI,ZINTAN,STEPAN
     +       ,SBREM,SLBREM,ZINTBR,STEPBR,SHADR,SLHADR,ZINTHA,STEPHA
     +       ,SMUNU,SLMUNU,ZINTMU,STEPMU,SDCAY,SLIFE ,SUMLIF,DPHYS1
     +       ,SLOSS,SOLOSS,STLOSS,DPHYS2,SMULS,SOMULS,STMULS,DPHYS3
     +       ,SRAYL,SLRAYL,ZINTRA,STEPRA,SLABS,SLLABS,ZINTLA,STEPLA
C
*KEEP,GCNUM.
      COMMON/GCNUM/NMATE ,NVOLUM,NROTM,NTMED,NTMULT,NTRACK,NPART
     +            ,NSTMAX,NVERTX,NHEAD,NBIT
      COMMON /GCNUMX/ NALIVE,NTMSTO
C
      INTEGER      NMATE ,NVOLUM,NROTM,NTMED,NTMULT,NTRACK,NPART
     +            ,NSTMAX,NVERTX,NHEAD,NBIT ,NALIVE,NTMSTO
C
*KEEP,MUPEP.
c MUPEP.CMN: PHOBOS MC multi-pass event processing variables
c 16-Mar-95 K. W.     NeSkip added
c  6-Jun-93 R. V.     Creation
c Multi-pass event processing breaks up each event into groups of
c tracks, in order to avoid overflowing Zebra track storage.
c Common for access to control variables:
      common /MUPEP/ EPState, NtConc, NtpE, NtPrim, NeSkip, NtSkip,
     1  MPind, NtPrev, NtTot, NtMax, NvPrev
c Parameters and data values are assigned in MUPEP.CDA.
c Event processing state: EPState = 0 for first pass,
c 1 for continuing pass, 2 for final pass
      integer EPState
c Current pass index:
      integer MPind
c Maximum number of tracks to process concurrently in a group:
      integer NtConc
c Maximum track number for this group:
      integer NtMax
c Maximum number of tracks per event:
      integer NtpE
c Number of primary and secondary tracks generated before this pass:
      integer NtPrev
c Number of primary tracks and vertices generated:
      integer NtPrim, NvPrev
c Number of input file events to skip initially:
      integer NeSkip
c Number of tracks to skip initially:
      integer NtSkip
c Total number of primary and secondary tracks generated:
      integer NtTot
*KEEP,RHSYS.
C  Common blocks for GPH PHOBOS simulation:  FILE RHSYS.CMN
C  General geometric layout of PHOBOS detector exclusive of detectors
C  These values are on input cards or data statements, except *MAX are
C  calculated to encompass all elements of the system.
C  20-Sep-96  K.W       Removed obsolete BMTBRAD, BERTHIK
C                       Removed FANGL - now local variable in BANG
C  12-Jul-96  K.W       Position of the magnet pole center MAGX, MAGZ
C                       (previously XOFF,ZOFF) separated from the magnet
C                       offset (now MOFF - 3-D vector)
C  14 Dec-92  R.V.      TMAG redefined and changed to integer
C  22-Nov-92 D. McLeod  Renamed from COMMON.SYS; TMAG, ZOFF, XOFF
C                       are added from the original GPH version
      REAL   BFLD         ! Magnetic field strength
     &      ,MAGX,MAGZ    ! Coordinates of field center
     &      ,MOFF(3)      ! offset of the field
     &      ,RFLD         ! Radius of field (perfect dipole only)
     1   ,XMAX,YMAX,ZMAX,ZMIN,XMMAX,ZMMAX
      INTEGER MAGT, TMAG  ! Field type: 0 Zebra bank, 1 = dipole,
                          ! 2 = ASCII map
      COMMON/RHSYS/ BFLD,MAGT,MAGX,MAGZ,MOFF
     1  ,RFLD,XMAX,YMAX,ZMAX,ZMIN,XMMAX,ZMMAX
      EQUIVALENCE (TMAG, MAGT)
*KEEP,UJKBUF.
C  PHOBOS JKINE user buffer definitions.          File: UJKBUF.INC
c 18-Jul-94  R. V.  Added NHitsAct
	integer Nwubuf, Nwubmax, NHitsAct
	parameter (Nwubmax = 11)
	common /UJKB/ Nwubuf, Ubuf(Nwubmax), NHitsAct
	real Ubuf
	integer IUBUF(Nwubmax)
	equivalence (IUBUF, UBUF)
c NHitsAct:   The number of hits on the track being propagated
c The user words at present are:
c UBUF(1):    Process in which particle was created (DCAY, BREM etc.)
c     (2):    Parent track number, negative is parent not stored
c             (possible in SECT=1 mode)
c     (3):    Volume number at origin, (4 characters in one integer word)
c             not definded for primaries and undefined for all tracks
c             in SECT=1 mode (integer value 0)
c     (4-6):  X, Y, Z at termination
c     (7-10): Px, Py, Pz, E at termination
c     (11):   Istop at track termination, positive when particle stopped
c             in a sensitive volume, negative otherwize
*KEND.

C External function returning length of a string
      integer LENOCC
      real RANF
      external RANF

C  Local:
      Integer LUN,  ! unit to open event file on
     &        ID    ! Geant ID from event file
      LOGICAL RUNFLAG
      INTEGER ios, Nusef, NVTX, NT, NTRGEN
      INTEGER ISTEPA, ISTEPP, ISTEPV, IR
      REAL VERT(3), PLAB(3)
      REAL  R, PS, PX, PY, PZ, DPP, V1, V2, V
      CHARACTER*80 FILE_NAME
      CHARACTER*80 LINESTR
      CHARACTER*120 LINE
      INTEGER HDATE(2), HTIME(2)
      real pmag,theta,phi,phi180,eta  ! particle parameters
      real phid,phid180
c Limits on accepted theta to avoid infinite pseudorapidity:
      real ThetaMin / 1.0e-7 /, ThetaMax / 3.1415926 /

      data Nusef /0/
      save ID, LUN, VERT
      SAVE RUNFLAG
      DATA RUNFLAG / .TRUE. /
      SAVE ISTEPA, ISTEPP, ISTEPV, IR
      DATA ISTEPA, ISTEPP, ISTEPV / 0, 0, 0 /
      DATA IR  /12347/


c End of declarations
C
C Error message should be  handled more generally
C
      integer lout
      lout=6
C

      IF(IGEN.EQ.0) THEN

c *********************************************************
c Text file event input
c Input is read on logical unit GENEVS.
c All lines start with a single KEY letter:
C   letter "C"  "H"  "R"  "I"  Comment line - copied to Ascii OUtput file
C                                        and transfered to Phat
C               lines starting with  "C" and "R" are treated as run info
C               up to the moment when any other line is found
C   letter "V"  Vertex (x,y,z) where particles start
C   letter "P"  Particle ID, P, THETA, PHI of particle
C   letter "E"  End of event
c   anything else stops the job with an error message, as
c   does a read or format conversion error.
c Processing is controlled by variables Ntconc and Ntpe in common
c MUPEP.  A maximum of Ntconc tracks are processed in one pass.
c Tracks are processed until Ntpe primary tracks have been processed,
c or an "E" line is read.  If no "E" line is present, the file is
c rewound and reread with a phi-rotation added.
C
C Sample event input file
C
C    C Define an event with three particles , pi+, pi-, p
C    C
C    V 0.0 0.0 0.0
C    P 8 1.0 0.5 0.0
C    P 9 2.0 0.5 0.1
C    P 14 3.0 0.75 0.0
C    E
c *********************************************************

************************************************************
*   Opening of the input file
************************************************************

        if (Epstate .eq. 0) then
          call UFILE (0, 'GENEVS', Lun, File_Name)
          if (Lun .le. 0) then
            write (lout, '(a)') ' GUKINE: unable to open track file:' //
     +                           File_Name
            Epstate = 3
          else
            WRITE (lout,'(2a)') ' GUKINE: reading tracks from ',
     1      File_name( 1:LENOCC(File_name) )
            Nusef = 0

            IF(NESKIP .EQ. 0)  THEN
               R = (RROT(1)+FLOAT(NUSEF)*RROT(2)) / 57.2957795
               WRITE(LINE, '('' R ROTATION '', F10.5)')   R
               IF(ISAOU .GT. 0)  WRITE(IUSRAOU, '(A)') LINE(1:80)
               IF(ISMEM .GT. 0)  CALL ADDPMCRINFO(LINE)
               WRITE(LINE, '(A)')
     +              ' R DEFR ROTATION applied rotation in phi (rad)'
               IF(ISAOU .GT. 0)  WRITE(IUSRAOU, '(A)') LINE(1:80)
               IF(ISMEM .GT. 0)  CALL ADDPMCRINFO(LINE)
            ENDIF

************************************************************
*  Skipping of NeSkip events and then NtSkip tracks
*  at the beginning of input text file
************************************************************
            do while  (NeSkip .gt. 0  .or.  NtSkip .gt. 0)
               Read (LUN, '(a)', iostat=ios) linestr

               if (ios .eq. -1) then
c On end of file, rewind and reread file with increased phi rotation
                 if(abs(RROT(2)) .gt. 0.001)  then
                   rewind (LUN)
                   Nusef = Nusef + 1
                   write(lout, '(a,i5,a)')
     +                     ' GUKINE: Input file rewound ',
     +                                  nusef, '  time'
                 else     ! file is not rewound if no rotation requested
                   Epstate = 3
                   NtSkip = 0
                   NeSkip = 0     !  abort run
                   write(lout, '(a,2f12.2)')
     +              ' GUKINE: Input file can not be rewound RROT =',
     +                     RROT
                 endif

               elseif (ios .gt. 0) then
c On read error, abort run
                 write (lout, '(2a/1x,a)')
     +               ' GUKINE: Quit while skipping',
     1               ' input read error, line:', linestr
                  Epstate = 3
                  NeSkip = 0
                  NtSkip = 0

               else

                 If (NeSkip .gt. 0)  then

                   If (linestr(1:1).eq.'E' .or.
     +                           linestr(1:1).eq.'e') Then
c On "E" line: count down the number of events to skip
                     NeSkip = NeSkip - 1
                     Epstate = 2
                     IF(NESKIP .EQ. 0)  THEN
                        R = (RROT(1)+FLOAT(NUSEF)*RROT(2))
     +                                            / 57.2957795
                        WRITE(LINE, '('' R ROTATION '', F10.5)')   R
                        IF(ISAOU .GT. 0)
     +                           WRITE(IUSRAOU, '(A)') LINE(1:80)
                        IF(ISMEM .GT. 0)  CALL ADDPMCRINFO(LINE)
                        WRITE(LINE, '(A)')
     +               ' R DEFR ROTATION applied rotation in phi (rad)'
                        IF(ISAOU .GT. 0)
     +                           WRITE(IUSRAOU, '(A)') LINE(1:80)
                        IF(ISMEM .GT. 0)  CALL ADDPMCRINFO(LINE)
                     ENDIF
                   endif
                   IF((EPSTATE .EQ. 0  .OR.  EPSTATE .EQ. 2)  .AND.
     +                   (LINESTR(1:1) .EQ. 'R' .OR.
     +                                LINESTR(1:1) .EQ. 'r'))   THEN
                      WRITE(LINE, '(1X, A)') LINESTR
                      IF(ISAOU .GT. 0)
     +                      WRITE(IUSRAOU, '(A)') LINE(1:81)
                      IF(ISMEM .GT. 0)  CALL ADDPMCRINFO(LINE)
                   ENDIF

                 else    !  only NtSkip is greater than  0

                   If (linestr(1:1).eq.'E' .or.
     +                           linestr(1:1).eq.'e') Then
c On "E" line: More tracks to skip than in the event - it is an error
                      write (lout, '(2a/1x,a)')
     +                  ' GUKINE: Quit while skipping, not enough ',
     1                  ' tracks to skip in the event'
                     NtSkip = 0
                     Epstate = 3
                   endif
                   IF((EPSTATE .EQ. 0  .OR.  EPSTATE .EQ. 2)  .AND.
     +                   (LINESTR(1:1) .EQ. 'R'  .OR.
     +                                LINESTR(1:1) .EQ. 'r'))   THEN
                      WRITE(LINE, '(1X, A)') LINESTR
                      IF(ISAOU .GT. 0)
     +                            WRITE(IUSRAOU, '(A)') LINE(1:81)
                      IF(ISMEM .GT. 0)  CALL ADDPMCRINFO(LINE)
                   ENDIF
                   IF((EPSTATE .EQ. 0  .OR.  EPSTATE .EQ. 2)  .AND.
     +                                 LINESTR(1:1) .EQ. 'H')   THEN
                      WRITE(LINE, '(1X, A)') LINESTR
                      IF(ISAOU .GT. 0)
     +                            WRITE(IUSRAOU, '(A)') LINE(1:81)
                      IF(ISMEM .GT. 0)  CALL ADDPMCEINFO(LINE)
                   ENDIF
                   If (linestr(1:1).eq.'P' .or.
     +                           linestr(1:1).eq.'p') Then
                      NtSkip = NtSkip - 1
                   endif

                 endif  !  NeSkip .gt. 0

               endif  ! ios .gt. 0

            enddo   !   while (NeSkip.GT.0)
          endif   !  Lun .le. 0
        endif   !  Epstate .eq. 0

        if (Epstate .eq. 0 .or. Epstate .eq. 2) then

************************************************************
*   Initialization at the begining of event
************************************************************

          MPind = 0
          NtPrim = 0
          NtTot = 0
          NvPrev = 0

**********************************************************************
* Define a default vertex position in case not specified in dataset
**********************************************************************
          IF(RBXY .LT. 0.0001)  THEN   ! flat distribution of vertex position
            IF(RBDX .GT. 0.0001)  THEN
               VERT(1) = RBXX + RBDX * (2.0*RANF(IR)-1.0)
            ELSE
               VERT(1) = RBXX
            ENDIF
            IF(RBDY .GT. 0.0001)  THEN
               VERT(2) = RBYY + RBDY * (2.0*RANF(IR)-1.0)
            ELSE
               VERT(2) = RBYY
            ENDIF
            IF(RBDZ .GT. 0.0001)  THEN
               VERT(3) = RBZZ + RBDZ * (2.0*RANF(IR)-1.0)
            ELSE
               VERT(3) = RBZZ
            ENDIF

          ELSE    !  double Gaussian (x-y) and trianglular (z) distributions

            V = 100.0
            DO WHILE(V .GT. 1.0)
              V1 = 2.0*RANF(IR) -1.0   !generate double Gaussian..
              V2 = 2.0*RANF(IR) -1.0   !see particle data booklet
              V = V1*V1 + V2*V2
            ENDDO
            VERT(1) = RBXX + RBXY*V1*SQRT(-2.0*LOG(V)/V)
            VERT(2) = RBYY + RBXY*V2*SQRT(-2.0*LOG(V)/V)

C     Generate a triangular probability distribution for intersection z
            V1 = RANF(IR)
            V2 = RANF(IR)
            IF(V1.GT.V2) THEN
              VERT(3) = RBZZ + 0.5*RBDZ*(V1-1.0)
            ELSE
              VERT(3) = RBZZ + 0.5*RBDZ*V1
            ENDIF

          ENDIF   !   if(RBXY .GT. ...

        elseif (Epstate .eq. 1) then

*****************************************************************
*   In the middle of the event (when splitted into small parts)
*****************************************************************

c Save cumulative vertex count in case we eventually write it in Ntuples
          NvPrev = NVTX
        endif   !   if (Epstate .eq. 0 .or. Epstate .eq. 2)


        NtPrev = NtTot

c Set flag to force a primary vertex to be defined
        NVTX = -1

c Set the number of tracks and read the next multi-pass group

        mPind = MPind + 1
        NtMax = min(NtPrim + NtConc, NtpE)
        if (Epstate .gt. 2) NtMax = 0   !  after an error - no tracks

************************************************************
*   Loop over tracks in the input text file
************************************************************

        do while (NtPrim .lt. NtMax)

          Read (LUN, '(a)', iostat=ios) linestr

          if (ios .eq. -1) then
c On end of file, rewind and reread file with increased phi rotation
            if(NtPrim .gt. 0)  then   !  some tracks before EOF
               Epstate = 3
               write(lout, '(A)')
     +           'GUKINE: end of input file before end of event'
            endif
            if(abs(RROT(2)) .gt. 0.001)  then
              rewind (LUN)
              Nusef = Nusef + 1
              write(lout, '(a,i5,a)')  ' GUKINE: Input file rewound ',
     +                                  nusef, '  time'
              R = (RROT(1)+FLOAT(NUSEF)*RROT(2)) / 57.2957795
              WRITE(LINE, '('' R ROTATION '', F10.5)')   R
              IF(ISAOU .GT. 0)  WRITE(IUSRAOU, '(A)') LINE(1:80)
              IF(ISMEM .GT. 0)  CALL ADDPMCRINFO(LINE)
              WRITE(LINE, '(A)')
     +              ' R DEFR ROTATION applied rotation in phi (rad)'
              IF(ISAOU .GT. 0)  WRITE(IUSRAOU, '(A)') LINE(1:80)
              IF(ISMEM .GT. 0)  CALL ADDPMCRINFO(LINE)
            else     ! file is not rewound if no rotation requested
              Epstate = 3
              NtMax = 0
              write(lout, '(a,2f12.2)')
     +           ' GUKINE: Input file can not be rewound RROT =', RROT
            endif   !  if(abs(RROT(2)) .gt. 0.001)

          elseif (ios .gt. 0) then
c On read error, abort run
            write (lout, '(2a/1x,a)') ' GUKINE: Quit on track',
     1   ' input read error, line:', linestr
            Epstate = 3
            NtMax = 0

          else   !   (ios .eq. 0)

***********************************************************************
*   Analysis of the line from the input file
***********************************************************************

c "R" or "H" line: allowed only on the beginning of the event
            IF(LINESTR(1:1).EQ.'R'  .OR.  LINESTR(1:1).EQ.'H' .OR.
     +           LINESTR(1:1).EQ.'r'  .OR.  LINESTR(1:1).EQ.'h')  THEN
               IF(EPSTATE .EQ. 0  .OR.  EPSTATE .EQ. 2)  THEN
                  WRITE(LINE, '(1X, A)') LINESTR
                  IF(ISAOU .GT. 0)
     +                          WRITE(IUSRAOU, '(A)') LINE(1:81)
                  IF(ISMEM .GT. 0)  THEN
                     IF(LINESTR(1:1).EQ.'R' .OR. LINESTR(1:1).EQ.'r')
     +                                          CALL ADDPMCRINFO(LINE)
                     IF(LINESTR(1:1).EQ.'H' .OR. LINESTR(1:1).EQ.'h')
     +                                          CALL ADDPMCEINFO(LINE)
                  ENDIF
               ELSE
                  EPSTATE = 3
                  WRITE(LOUT, '(A / 1X, A)')
     +              'GUKINE: run or event header line inside an event',
     +              LINESTR
               ENDIF

            ElseIf ((linestr(1:1).eq.'V').or.(linestr(1:1).eq.'v')) Then
c On "V" line: save vertex position
              read (linestr(2:),*) vert(1), vert(2), vert(3)
              CALL GSVERT(VERT, 0, 0, 0, 0, NVTX)
              Epstate = 1

            ElseIf ((linestr(1:1).eq.'P').or.(linestr(1:1).eq.'p')) Then
              Epstate = 1
c "P" line: save track kinematics including phi-rotation
              read (linestr(2:),*,iostat=ios) id, pmag, theta, phi
              if (ios .eq. 0) then
                phi = phi +                         !   rotation of phi
     +                 (RROT(1) +  float(Nusef)*RROT(2)) / 57.2957795

                do  while (phi .gt. 3.14159265359)  !   normalization of phi
                    phi = phi - 6.28318530718
                enddo
                do  while (phi .lt. -3.14159265359)
                    phi = phi + 6.28318530718
                enddo
                phi180 = phi + 3.14159265359  !
                if(phi180 .gt. 3.14159265359)
     +                    phi180 = phi180 - 6.28318530718

                phid = phi * 57.2957795
                phid180 = phi180 * 57.2957795
                theta = min (max (theta, ThetaMin), ThetaMax)
                plab(1) = pmag * sin(theta) * cos(phi)
                plab(2) = pmag * sin(theta) * sin(phi)
                plab(3) = pmag * cos(theta)
                eta = -alog( tan(theta/2.0) )

*************************************************************************
*  Select only the tracks that are in the requested intervals of momentum,
*  eta,  phi etc.
*************************************************************************
                if(rmom(1) .le. pmag   .and.   pmag .le. rmom(2)  .and.
     +            reta(1) .le. eta  .and.  eta .le. reta(2)  .and.
     +            irpid(1) .le. id  .and.  id .le. irpid(2)  .and.
     +            ((rphi(1) .le. phid .and. phid .le. rphi(2))  .or.
     +             (irarm .eq. 2 .and.
     +               rphi(1) .le. phid180 .and. phid180 .le. rphi(2)))
     +                               )   then

C Load default vertex if there has been no vertex card.
                  If (nvtx .lt. 0) CALL GSVERT (VERT, 0, 0, 0, 0, NVTX)
                  CALL UZERO(UBUF, 1, NWUBMAX)
                  CALL GSKINE (PLAB, ID, NVTX, UBUF, NWUBMAX, NT)
                  NtPrim = NtPrim + 1
                  NtTot = NtTot + 1
c Set end-of-event flag after Ntpe tracks
                  if (NtPrim .ge. Ntpe) Epstate = 2
                endif    !  selection of tracks

              else     !  ios .ne. 0
c On illegal input, abort
                write (lout, '(2a/1x,a)') ' GUKINE: Quit on track',
     1          ' input conversion error in line:', linestr
                Epstate = 3
                NtMax = 0
              endif

*************************************************************************
*  Other lines
*************************************************************************
            ElseIf ((linestr(1:1).eq.'E').or.(linestr(1:1).eq.'e')) Then
c On "E" line: set end-of-event flag and force end of loop
              NtMax = NtPrim
              Epstate = 2

            ElseIf ((linestr(1:1).eq.'C').or.(linestr(1:1).eq.'c')) Then
c "C" line: ignore comment

            ElseIf ((linestr(1:1).eq.'I').or.(linestr(1:1).eq.'i')) Then
c "I" line: obsolete, but still treated as run comment files
               IF(EPSTATE .EQ. 0  .OR.  EPSTATE .EQ. 2)  THEN
                  WRITE(LINE, '('' R C '', A)') LINESTR
                  IF(ISAOU .GT. 0)
     +                         WRITE(IUSRAOU, '(A)') LINE(2:87)
                  IF(ISMEM .GT. 0)  CALL ADDPMCRINFO(LINE)
               ELSE
                  EPSTATE = 3
                  WRITE(LOUT, '(A / 1X, A)')
     +              'GUKINE: run header line inside an event',
     +              LINESTR
               ENDIF

            Else
c Unrecognized line: abort; probably wrong file
              write (lout, '(2a/1x,a)') ' GUKINE: Quit on illegal ',
     1          ' track input line:', linestr
              EpState = 3
              NtMax = 0
            EndIf

          endif ! ios

        end do ! while (Ntprim .lt. NtMax)
C              ! end of event found or the number of tracks read
C              ! was equal NtCo

      ENDIF   !   (IGEN .EQ. 0)
C---------------------------------------  end of IGEN = 0 section


      IF(IGEN .NE. 0)  THEN
        IF(EPSTATE .EQ. 0)   THEN
           WRITE(LINE, '(A, F3.1)') ' R GENERATOR   PMC', SIMVER
           IF(IGEN .EQ. 10)
     +           WRITE(LINE, '(A)') ' R GENERATOR   PMC+PHAT'
           IF(ISAOU .GT. 0)  WRITE(IUSRAOU, '(A)') LINE(1:80)
           IF(ISMEM .GT. 0)  CALL ADDPMCRINFO(LINE)
           WRITE(LINE, '(A, I5)')   ' R VERSION     ', IGEN
           IF(ISAOU .GT. 0)  WRITE(IUSRAOU, '(A)') LINE(1:80)
           IF(ISMEM .GT. 0)  CALL ADDPMCRINFO(LINE)
           CALL DATIMH(HDATE, HTIME)
           WRITE(LINE, '(A,2A4)')   ' R DATE        ', HDATE
           IF(ISAOU .GT. 0)  WRITE(IUSRAOU, '(A)') LINE(1:80)
           IF(ISMEM .GT. 0)  CALL ADDPMCRINFO(LINE)
           WRITE(LINE, '(A, I8)')   ' R RUN      ', IDRUN
           IF(ISAOU .GT. 0)  WRITE(IUSRAOU, '(A)') LINE(1:80)
           IF(ISMEM .GT. 0)  CALL ADDPMCRINFO(LINE)

           WRITE(LINE, '(A)')  ' R DEFR GENERATOR  generator name'
           IF(ISAOU .GT. 0)  WRITE(IUSRAOU, '(A)') LINE(1:80)
           IF(ISMEM .GT. 0)  CALL ADDPMCRINFO(LINE)
           WRITE(LINE, '(A)')  ' R DEFR VERSION PMC generation mode'
           IF(ISAOU .GT. 0)  WRITE(IUSRAOU, '(A)') LINE(1:80)
           IF(ISMEM .GT. 0)  CALL ADDPMCRINFO(LINE)
           WRITE(LINE, '(A)')  ' R DEFR DATE date of the simulation'
           IF(ISAOU .GT. 0)  WRITE(IUSRAOU, '(A)') LINE(1:80)
           IF(ISMEM .GT. 0)  CALL ADDPMCRINFO(LINE)
           WRITE(LINE, '(A)')  ' R DEFR RUN run number'
           IF(ISAOU .GT. 0)  WRITE(IUSRAOU, '(A)') LINE(1:80)
           IF(ISMEM .GT. 0)  CALL ADDPMCRINFO(LINE)
           IF(IGEN .NE. 10)   THEN
             WRITE (lout,'(a)')
     +         ' GUKINE: using tracks generated by BANG.'
             if(IRMUL .GT. NtpE)  then
                WRITE(lout, *) ' GUKINE:  RMUL truncated to NTPE = ',
     +                                         NtpE
                IRMUL = NtpE
             endif
             WRITE(LINE, '(A)')  ' R DEFE EVNO    event number'
             IF(ISAOU .GT. 0)  WRITE(IUSRAOU, '(A)') LINE(1:80)
             IF(ISMEM .GT. 0)  CALL ADDPMCRINFO(LINE)
             WRITE(LINE, '(A)')
     +              ' R DEFE NSTAB   number of produced particles'
             IF(ISAOU .GT. 0)  WRITE(IUSRAOU, '(A)') LINE(1:80)
             IF(ISMEM .GT. 0)  CALL ADDPMCRINFO(LINE)
           ENDIF
        ENDIF   !   IF(EPSTATE .EQ. 0)
        IF((EPSTATE .EQ. 0  .OR.  EPSTATE .EQ. 2)  .AND.
     +                                      IGEN .NE. 10)   THEN
           WRITE(LINE, '(A, I8)')  ' H EVNO    ', IEVENT
           IF(ISAOU .GT. 0)  WRITE(IUSRAOU, '(A)') LINE(1:80)
           IF(ISMEM .GT. 0)  CALL ADDPMCEINFO(LINE)
           WRITE(LINE, '(A, I8)')  ' H NSTAB   ', IRMUL
           IF(IGEN .GT. 10)  WRITE(LINE,'(A)') ' H NSTAB          1'
           IF(ISAOU .GT. 0)  WRITE(IUSRAOU, '(A)') LINE(1:80)
           IF(ISMEM .GT. 0)  CALL ADDPMCEINFO(LINE)
        ENDIF   !   IF((EPSTATE .EQ. 0  .OR.  EPSTATE .EQ. 2) ...
      ENDIF   !   IF(IGEN .NE. 0)

      IF(IGEN .GE. 1  .AND.  IGEN .LE. 9)   THEN

**********************************************************************
*  Multiple random tracks from random beam interactions:  from BANG  *
**********************************************************************

c Set up for the next multi-pass group

        if (Epstate .eq. 0 .or. Epstate .eq. 2) then
          Epstate = 1
          MPind = 0
          NtPrim = 0
          NtTot = 0
          NvPrev = 0
        elseif (Epstate .eq. 1) then
c Save the cumulative vertex count in case we eventually write it
c into ntuples
          NvPrev = NVTX
        endif
        NTRGEN = min(IRMUL - NtPrim, NtConc)
c Generate the next group, set flag if final group
        NtPrev = NtTot
        NVTX = -1
        if (NTRGEN .GT. 0) call BANG(IGEN, NTRGEN, NVTX)
        MPind = MPind + 1
        NtPrim = NtPrim + NTRGEN
        NtTot = NtTot + NTRGEN
        if (NtPrim .ge. IRMUL) EpState = 2

      ENDIF   !   (IGEN .GE. 1  .AND.  IGEN .LE. 9)
C-------------------------------------------------   end of BANG  mode


*******************************************************************
*  PARTICLE DATA AND VERTEX ARE GIVEN INTERACTIVELY               *
*******************************************************************

      IF(IGEN .EQ. 10)  THEN

         CALL GET_KINE(ID, PLAB, VERT)

         Epstate = 2
         MPind = 0
         NtPrim = 1
         NtTot = 1
         NvPrev = 0

         CALL GSVERT(VERT,0,0,0,0,NVTX)
         CALL UZERO(UBUF, 1, NWUBMAX)
         CALL GSKINE(PLAB,ID,NVTX,UBUF,NWUBMAX,NT)

      ENDIF   !   (IGEN .EQ. 10)


*******************************************************************
* STEP THE VALUES OF MOMENTUM OR 1/MOMENTUM FIRST,  THEN          *
* THE EMISSION ANGLE THETA  AND FINALLY OF THE VERTEX POSITION    *
*******************************************************************

      IF(IGEN .GE. 11  .AND. IGEN .LE. 12)  THEN

         IF(IGEN .EQ. 11)  THEN
            PS = RMOM(1)+FLOAT(ISTEPP)*RMOM(3)
         ELSE IF (IGEN .EQ. 12)  THEN
            IF(RMOM(1) .LT. 0.001)  RMOM(1) = 0.001
            IF(RMOM(2) .LT. 0.001)  RMOM(2) = 0.001
            DPP = 1.0/RMOM(1) - 1.0/(RMOM(1)+RMOM(3))
            PS = 1.0/RMOM(1) - FLOAT(ISTEPP)*DPP
            PS = 1.0/PS
         ENDIF
         PX = PS * SIN((RANG(1)+FLOAT(ISTEPA)*RANG(3))/57.2957795) *
     +          COS(RPHI(1)/57.2957795)
         PY = PS * SIN((RANG(1)+FLOAT(ISTEPA)*RANG(3))/57.2957795) *
     +          SIN(RPHI(1)/57.2957795)
         PZ = PS * COS((RANG(1)+FLOAT(ISTEPA)*RANG(3))/57.2957795)
         PLAB(1) = PX
         PLAB(2) = PY
         PLAB(3) = PZ
         VERT(3) = RBZZ + FLOAT(ISTEPV)*RBDZ
         ISTEPP = ISTEPP + 1
         IF(IGEN .EQ. 11)  THEN
            PS = RMOM(1)+FLOAT(ISTEPP)*RMOM(3)
         ELSE IF (IGEN .EQ. 12)  THEN
            PS = 1.0/RMOM(1) - FLOAT(ISTEPP)*DPP
            IF(PS .LT. 0.00001)   PS = 0.00001
            PS = 1.0/PS
         ENDIF
         IF(PS .GT. RMOM(2))  THEN
           ISTEPP = 0
           ISTEPA = ISTEPA + 1
           IF(FLOAT(ISTEPA)*RANG(3) .GT. RANG(2)-RANG(1))  THEN
             ISTEPA = 0
             ISTEPV = ISTEPV + 1
           ENDIF
         ENDIF

         Epstate = 2
         MPind = 0
         NtPrim = 1
         NtTot = 1
         NvPrev = 0

         CALL GSVERT(VERT,0,0,0,0,NVTX)
         CALL UZERO(UBUF, 1, NWUBMAX)
         CALL GSKINE(PLAB,IRPID(1),NVTX,UBUF,NWUBMAX,NT)

      ENDIF   !   (IGEN .GE. 11  .AND. IGEN .LE. 12)


      IF(IGEN .GE. 13  .AND. IGEN .LE. 14)  THEN

*******************************************************************
* STEP THE  VALUES OF THE EMISSION ANGLE THETA FIRST              *
* THEN  OF THE VALUES OF MOMENTUM OR 1/MOMENTUM, AND              *
* FINALLY OF THE VERTEX POSITION                                  *
*******************************************************************

         IF(IGEN .EQ. 13)  THEN
            PS = RMOM(1)+FLOAT(ISTEPP)*RMOM(3)
         ELSE IF (IGEN .EQ. 14)  THEN
            IF(RMOM(1) .LT. 0.001)  RMOM(1) = 0.001
            IF(RMOM(2) .LT. 0.001)  RMOM(2) = 0.001
            DPP = 1.0/RMOM(1) - 1.0/(RMOM(1)+RMOM(3))
            PS = 1.0/RMOM(1) - FLOAT(ISTEPP)*DPP
            PS = 1.0/PS
         ENDIF
         PX = PS * SIN((RANG(1)+FLOAT(ISTEPA)*RANG(3))/57.2957795) *
     +          COS(RPHI(1)/57.2957795)
         PY = PS * SIN((RANG(1)+FLOAT(ISTEPA)*RANG(3))/57.2957795) *
     +          SIN(RPHI(1)/57.2957795)
         PZ = PS * COS((RANG(1)+FLOAT(ISTEPA)*RANG(3))/57.2957795)
         PLAB(1) = PX
         PLAB(2) = PY
         PLAB(3) = PZ
         VERT(3) = RBZZ + FLOAT(ISTEPV)*RBDZ
         ISTEPA = ISTEPA + 1
         IF(FLOAT(ISTEPA)*RANG(3) .GT. RANG(2)-RANG(1))  THEN
           ISTEPA = 0
           ISTEPP = ISTEPP + 1
           IF(IGEN .EQ. 13)  THEN
              PS = RMOM(1)+FLOAT(ISTEPP)*RMOM(3)
           ELSE IF (IGEN .EQ. 14)  THEN
              PS = 1.0/RMOM(1) - FLOAT(ISTEPP)*DPP
              IF(PS .LT. 0.00001)   PS = 0.00001
              PS = 1.0/PS
           ENDIF
           IF(PS .GT. RMOM(2))  THEN
              ISTEPP = 0
              ISTEPV = ISTEPV + 1
           ENDIF
         ENDIF

         Epstate = 2
         MPind = 0
         NtPrim = 1
         NtTot = 1
         NvPrev = 0

         CALL GSVERT(VERT,0,0,0,0,NVTX)
         CALL UZERO(UBUF, 1, NWUBMAX)
         CALL GSKINE(PLAB,IRPID(1),NVTX,UBUF,NWUBMAX,NT)

      ENDIF


**********************************
*  Unrecognized GEN type:
**********************************

      IF(IGEN .LT. 0  .AND. IGEN .GT. 14)  THEN

         WRITE(lout,'('' GEN= '',I2,'' NOT IMPLEMENTED'')') IGEN
         Epstate = 2

      ENDIF

*************************************************
*  Kinematic debug (controlled by ISWIT(1))
*************************************************

      IF(IDEBUG .EQ. 1  .AND.  ISWIT(1) .GE. 1) THEN
         CALL GPRINT('VERT',0)
         CALL GPRINT('KINE',0)
      ENDIF

      RETURN
      END
