      SUBROUTINE PH_GEOM
*************************************************************************
*   Stores in the AOU file information on the geometry of active
*   detector elements
*   Also transfers geometry information to Phat
*
* History:
*  23-Nov-98   K.W.   Add call to ADDPMCRINFO
*   7-Jul-98   K.W.   Protection against error when the ACOS argument is
*                     greater than 1.0
*  15-May-98   K.W.   Add call to MAKEDETECTOR
*                     Geometry version written to AOU here not in UGINIT
*  27-Apr-98   K.W.   Add calls to ADDPMCDEVO and ADDPMCROT, if ISMEM>0
*                     Writing only if ISAOU>0
*   4-Nov-97   K.W.   Increased nuber of PADS paramaters to 10
*                     check of PADS parameters for IPADFL = 4  (TUBS)
*  17-Sep-97   K.W.   effective rotation matrix for each detector stored
*                     in place of all original rotation matrixes
*                     available from GEANT
*   2-Jul-97   K.W.   added consistency tests of pixelization parameters
*                     and number of division
*  23-Apr-97   K.W.   correction to avoid error when no rotation matrices
*                     are defined in the geometry file
*  27-Feb-97   K.W.   Creation
*************************************************************************

      IMPLICIT NONE

*KEEP,FLAG.
C  PHOBOS flags, run control, operating modes, misc.   File FLAG.CMN
C   7-Jul-98  K. W.      Added  ITRIG flag (initial numer in TRIG command)
C  27-Nov-97  K. W.      Added IAOUF flag
C  31-Mar-95  R.V.       Change STRA to SECT to avoid GEANT 3.21 conflict
C  22-Nov-92  D. McLeod
C   These are set in input cards
      INTEGER ISASC,ISHBO,ISHDO,ISNTU, ISMEM,ISAOU,IAOUF, ISECT,IDRAW,
     1        IGEN,IANAL,ISCAT,IPIXL,ISEQ,NTRK,IZEB, ITRIG
      REAL WPIXL,WLOSS
      COMMON/RHFLG/ISASC,ISHBO,ISHDO,ISNTU,
     1  ISMEM,ISAOU,IAOUF, ISECT,IDRAW,
     2  IGEN,IANAL,ISCAT,IPIXL,ISEQ,NTRK,IZEB,WPIXL,WLOSS, ITRIG
*KEEP,GCBANK.
      INTEGER IQ,LQ,NZEBRA,IXSTOR,IXDIV,IXCONS,LMAIN,LR1,JCG
      INTEGER KWBANK,KWWORK,IWS
      REAL GVERSN,ZVERSN,FENDQ,WS,Q
C
      PARAMETER (KWBANK=69000,KWWORK=5200)
      COMMON/GCBANK/NZEBRA,GVERSN,ZVERSN,IXSTOR,IXDIV,IXCONS,FENDQ(16)
     +             ,LMAIN,LR1,WS(KWBANK)
      DIMENSION IQ(2),Q(2),LQ(8000),IWS(2)
      EQUIVALENCE (Q(1),IQ(1),LQ(9)),(LQ(1),LMAIN),(IWS(1),WS(1))
      EQUIVALENCE (JCG,JGSTAT)
      INTEGER       JDIGI ,JDRAW ,JHEAD ,JHITS ,JKINE ,JMATE ,JPART
     +      ,JROTM ,JRUNG ,JSET  ,JSTAK ,JGSTAT,JTMED ,JTRACK,JVERTX
     +      ,JVOLUM,JXYZ  ,JGPAR ,JGPAR2,JSKLT
C
      COMMON/GCLINK/JDIGI ,JDRAW ,JHEAD ,JHITS ,JKINE ,JMATE ,JPART
     +      ,JROTM ,JRUNG ,JSET  ,JSTAK ,JGSTAT,JTMED ,JTRACK,JVERTX
     +      ,JVOLUM,JXYZ  ,JGPAR ,JGPAR2,JSKLT
C
*KEEP,GCNUM.
      COMMON/GCNUM/NMATE ,NVOLUM,NROTM,NTMED,NTMULT,NTRACK,NPART
     +            ,NSTMAX,NVERTX,NHEAD,NBIT
      COMMON /GCNUMX/ NALIVE,NTMSTO
C
      INTEGER      NMATE ,NVOLUM,NROTM,NTMED,NTMULT,NTRACK,NPART
     +            ,NSTMAX,NVERTX,NHEAD,NBIT ,NALIVE,NTMSTO
C
*KEEP,IO.
C Common block for GPH logical units etc.: file IO.CMN
c 13-Aug-96  R. V.       Added USRBATCHK for KUIP commands for UGCONS
c 21-Jul-96  R. V.       Remove unused record; note CARDDEF now obsolete
c 22-Feb-93  R. V.       Added USRMCNTU and IMCNTU for MC Ntuple
c 14-Nov-92  R. Verdier  Added UGCIN, IUGCIN, CARDDEF and ICARDDEF
c                        for UGC and EUCARD (obsolete) input units
C    Apr-92  C. Parsons  Replacement for original COMMON.CDT
      INTEGER USRGCIN, CARDDEF, USRIN, USROUT, USRMAP, USRNTU, USRZEB,
     *  USRMCNTU, USRBATKUMAC
      PARAMETER (USRIN = 4, USROUT = 16, USRMAP = 18)
      PARAMETER (USRNTU = 15, USRZEB = 17, USRGCIN = 19, CARDDEF = 20)
      PARAMETER (USRMCNTU = 21, USRBATKUMAC = 22)
      INTEGER ICARDDEF, IMCNTU, IUSRGCIN, IUSRIN, IUSROUT, IUSRMAP,
     *  IUSRNTU, IUSRZEB, IUSRAOU, IUSRBATKUMAC
      COMMON /IO/ IUSRIN, IUSROUT, IUSRMAP, IUSRNTU, IUSRZEB, IUSRGCIN,
     *  ICARDDEF, IMCNTU, IUSRAOU, IUSRBATKUMAC
*KEEP,RHSIL.
C RHSIL.CMN: PHOBOS detector plane dimensions, positions, and
c granularity; UGC parameters
c The actual and maximum numbers of detector planes and detector hit
c parameters.  Their default values are defined in RHSILD.INC.
c 12-Dec-97 K. W.       PADSOLD and PADSNEW from UGCSET moved to a common
c  4-Nov-97 K. W.       Increased number of PADS parameters to 10
c                       Increased NSLPM to 800
c  1-Jul-97 K. W.       Added data for GEANT pixelization (GSDVN)
c  1-May-97 R. V.       Increased NSLPM to 500
c  3-MAR-97 K. W.       Removed SPL, added PADS data
C  3-Jan-97 K. W.       Increased NSLPM to 180
c  3-Jul-96 K. W.       Geometry version number added
c  5-Apr-96 D. McL.     Moved data value of UgcpMax to RHSILD
c  ...
c    Mar-92 D.McLeod    Creation
c Storage limit for number of silicon planes:
      INTEGER NSLPM
      PARAMETER (NSLPM=800)
c Storage limit for number of parameters for each hit:
      integer NSipMaxp
      parameter (NSipMaxp = 12)
c Safety limit for number of parameter values that may be entered in
c a UGC text input record:
      integer UgcpMaxp
      parameter (UgcpMaxp = 10)
c Number of silicon planes in use:
      integer NSLP
c Additional information on detectors - pad sizes:
C  DETNAME  - name of the detector volume
C  DIVXNAME - name for division in X direction
C  DIVYNAME - name for division in Y direction
C  DIVZNAME - name for division in Z direction
C  NDIVX    - number of divisions in X direction
C  NDIVY    - number of divisions in Y direction
C  NDIVZ    - number of divisions in Z direction
C  IPADFL   - pad type flag
C  PADn   - four pad parameters
C             (pad size in X, Y, Z direction for IPADFL=1 for example)
      CHARACTER*4 DETNAME(NSLPM), DIVXNAME(NSLPM),
     +             DIVYNAME(NSLPM), DIVZNAME(NSLPM)
      INTEGER NDIVX(NSLPM), NDIVY(NSLPM), NDIVZ(NSLPM)
      REAL PAD1(NSLPM),PAD2(NSLPM),PAD3(NSLPM),PAD4(NSLPM),PAD5(NSLPM)
      REAL PAD6(NSLPM),PAD7(NSLPM),PAD8(NSLPM),PAD9(NSLPM),PAD10(NSLPM)
      INTEGER NDETS, IPADFL(NSLPM)
      LOGICAL PADSOLD, PADSNEW
      COMMON /RHPADS/ NDETS, PADSOLD, PADSNEW,
     +       DETNAME, DIVXNAME, DIVYNAME, DIVZNAME,
     +       NDIVX, NDIVY, NDIVZ,
     +       IPADFL, PAD1, PAD2, PAD3, PAD4, PAD5,
     +               PAD6, PAD7, PAD8, PAD9, PAD10
c Number of silicon planes - compared with final value of NDETS
      COMMON /RHSIL/ NSLP
c UGC parameter information:
      integer UgcpMax
      real UGCpars(UgcpMaxp)
c Declare silicon plane parameters; their default values are specified
c in RHSILD.INC
      integer NSIP
c Names of parameters:
      CHARACTER*4 SINAMH(NSipMaxp)
c Mumber of bits stored for each parameter:
      INTEGER SIBITH(NSipMaxp)
c Multiplicative scale factors and origins, defined so that
c   Stored value = (Parameter value + SIORIG) * SIFACT:
      REAL SIFACT(NSipMaxp), SIORIG(NSipMaxp)
c Common for access to UGC parameters and hit parameter definitions:
      common /UGCPAR/ UgcpMax, UGCpars,
     1                NSIP, SINAMH, SIBITH, SIORIG, SIFACT
      real GEOV  !  geometry version number
      common /ugGEOV/ GEOV
*KEND.

      CHARACTER*120 LINE
      CHARACTER*4 CNAME
      INTEGER NR,JR,I, N,J,JJ, IV,JD,JDV,JDVV,KV
      INTEGER ISLVOL,ISLSHP,ISLJ,ISLNPAR
      INTEGER JDMAX
      PARAMETER (JDMAX=30)
      INTEGER JDLINK(JDMAX),JDNIN(JDMAX),JDIN(JDMAX), JDROT(JDMAX)
      REAL TRXJD(JDMAX), TRYJD(JDMAX), TRZJD(JDMAX)
      INTEGER NPARMAX
      PARAMETER (NPARMAX=40)
      REAL SLPAR(NPARMAX), STEP
      real*8 TRX, TRY, TRZ, TRXS, TRYS, TRZS
      REAL*8 DROTM(9), DROTA(6)
      REAL*8 R1, R2, R3, R4, R5, R6, R7, R8, R9
      REAL  RROTM(9), RROTA(6)
      REAL  CENTER(3), PADPAR(10)

      REAL*8 XX, TH,THD, STH, PH, PHD, DEGRAD

      DEGRAD = 0.0174532925199432958


      IF(ISAOU .GT. 0)  THEN
         WRITE(IUSRAOU, '('' * '')' )
         WRITE(IUSRAOU, '('' * Geometry version'')' )
         WRITE(IUSRAOU, '('' G GEOV '', F10.3)' )  GEOV
         WRITE(IUSRAOU, '('' * '')' )
      ENDIF
      IF(ISMEM .GT. 0)  THEN
         CALL MAKEDETECTOR(GEOV)
         WRITE(LINE, '('' G GEOV '', F10.3)' )  GEOV
         CALL ADDPMCRINFO(LINE)
      ENDIF

C  Loop over the active volumes
      DO  200 N=1,NSLP
        ISLVOL = 0
        DO  IV=1,NVOLUM
           KV = IQ(JVOLUM+IV)
           CALL UHTOC(KV, 4, CNAME, 4)
           IF(CNAME .EQ. DETNAME(N))   THEN
              ISLVOL = IV
              GO TO 110
           ENDIF
        ENDDO
  110   CONTINUE
        IF(ISLVOL .EQ. 0)   THEN
           WRITE(*,*) ' *** ERROR IN PH_GEOM - DETECTOR:  ',
     +                DETNAME(N), '   NOT DEFINED IN THE GEOMETRY'
           GO TO 200
        ENDIF

C      find parameters of the volume
        ISLJ = LQ(JVOLUM-ISLVOL)
        ISLSHP = Q(ISLJ+2)
        ISLNPAR = Q(ISLJ+5)
        IF(ISLNPAR .GT. NPARMAX)  THEN
           WRITE(*,*) ' *** WARNING - PH_GEOM - TOO MANY ',
     +       'SHAPE PARAMETERS ', ISLNPAR, '    IN VOLUME ',
     +         DETNAME(N), '   TRUNCATED TO ', NPARMAX
           ISLNPAR = NPARMAX
        ENDIF
        DO  I=1,ISLNPAR
           SLPAR(I) = Q(ISLJ+6+I)
        ENDDO

C      find the position of the volume in the volumes tree
C      calculate effective translation and
C      find the numbers of rotation matrices for this volume
        JD = 1    ! depth of the volume in the tree
        JDLINK(1) = LQ(JVOLUM-1)
        JDNIN(1) = Q(JDLINK(1)+3)
        JDIN(1) = 1
  120   CONTINUE
          JDV = LQ(JDLINK(JD)-JDIN(JD))   ! link to the lower level volume
          IV = Q(JDV+2)
          JDROT(JD) = Q(JDV+4)
          TRXJD(JD) = Q(JDV+5)
          TRYJD(JD) = Q(JDV+6)
          TRZJD(JD) = Q(JDV+7)
          IF(IV .EQ. ISLVOL)  GO TO 150  ! volume found

          JDVV = LQ(JVOLUM-IV)
          IF(Q(JDVV+3) .GT. 0)   THEN   ! there are volumes deeper
             JD = JD + 1
             JDLINK(JD) = JDVV
             JDNIN(JD) = Q(JDVV+3)
             JDIN(JD) = 1
             GO TO 120
          ELSE    ! try next volume on the same level or upper
  125        CONTINUE
             IF(JDIN(JD) .LT. JDNIN(JD))   THEN
                JDIN(JD) = JDIN(JD) + 1    ! next volume on this level
                GO TO 120
             ELSE
                JD = JD -1   ! go one level upper
                IF(JD .LT. 0)  THEN
                   WRITE(*,*) ' *** ERROR IN PH_GEOM - DETECTOR  ',
     +                DETNAME(N), '   NOT POSITIONED IN THE TREE'
                   GO TO 200
                ENDIF
                GO TO 125
             ENDIF
          ENDIF

  150   CONTINUE   !  detector found in the tree
        TRX = 0.0
        TRY = 0.0
        TRZ = 0.0
        DROTM(1) = 1.0   !  initial rotation matrix (no rotation)
        DROTM(2) = 0.0
        DROTM(3) = 0.0
        DROTM(4) = 0.0
        DROTM(5) = 1.0
        DROTM(6) = 0.0
        DROTM(7) = 0.0
        DROTM(8) = 0.0
        DROTM(9) = 1.0
        DO  J=1,JD   !  calculate effective translation
           JJ = JD - J + 1
           NR = JDROT(JJ)
           IF(NR .GT. 0)  THEN   !  rotate translation vector
             JR = LQ(JROTM-NR)
             IF(JR .GT. 0)  THEN   !  safety check only
                TRXS = DBLE(Q(JR+1)) * TRX +
     +                 DBLE(Q(JR+4)) * TRY +
     +                 DBLE(Q(JR+7)) * TRZ
                TRYS = DBLE(Q(JR+2)) * TRX +
     +                 DBLE(Q(JR+5)) * TRY +
     +                 DBLE(Q(JR+8)) * TRZ
                TRZS = DBLE(Q(JR+3)) * TRX +
     +                 DBLE(Q(JR+6)) * TRY +
     +                 DBLE(Q(JR+9)) * TRZ
                TRX = TRXS + TRXJD(JJ)   !  add actual and
                TRY = TRYS + TRYJD(JJ)   !  rotated translation vector
                TRZ = TRZS + TRZJD(JJ)
             ENDIF    !  (JR .GT. 0)
          ELSE    !   .NOT. (NR .GT. 0)   !  add actual and previous
             TRX = TRX + TRXJD(JJ)        !  translation vector
             TRY = TRY + TRYJD(JJ)
             TRZ = TRZ + TRZJD(JJ)
          ENDIF   !   (NR .GT. 0)

          NR = JDROT(J)
          IF(NR .GT. 0)  THEN   !  volume rotated - multiply previous
            JR = LQ(JROTM-NR)   !  rotation matrix with a new one
            IF(JR .GT. 0)  THEN  ! safety - check if rotation matrix exists
               R1 = DBLE(Q(JR+1)) * DROTM(1) +
     +              DBLE(Q(JR+2)) * DROTM(4) +
     +              DBLE(Q(JR+3)) * DROTM(7)
               R2 = DBLE(Q(JR+1)) * DROTM(2) +
     +              DBLE(Q(JR+2)) * DROTM(5) +
     +              DBLE(Q(JR+3)) * DROTM(8)
               R3 = DBLE(Q(JR+1)) * DROTM(3) +
     +              DBLE(Q(JR+2)) * DROTM(6) +
     +              DBLE(Q(JR+3)) * DROTM(9)
               R4 = DBLE(Q(JR+4)) * DROTM(1) +
     +              DBLE(Q(JR+5)) * DROTM(4) +
     +              DBLE(Q(JR+6)) * DROTM(7)
               R5 = DBLE(Q(JR+4)) * DROTM(2) +
     +              DBLE(Q(JR+5)) * DROTM(5) +
     +              DBLE(Q(JR+6)) * DROTM(8)
               R6 = DBLE(Q(JR+4)) * DROTM(3) +
     +              DBLE(Q(JR+5)) * DROTM(6) +
     +              DBLE(Q(JR+6)) * DROTM(9)
               R7 = DBLE(Q(JR+7)) * DROTM(1) +
     +              DBLE(Q(JR+8)) * DROTM(4) +
     +              DBLE(Q(JR+9)) * DROTM(7)
               R8 = DBLE(Q(JR+7)) * DROTM(2) +
     +              DBLE(Q(JR+8)) * DROTM(5) +
     +              DBLE(Q(JR+9)) * DROTM(8)
               R9 = DBLE(Q(JR+7)) * DROTM(3) +
     +              DBLE(Q(JR+8)) * DROTM(6) +
     +              DBLE(Q(JR+9)) * DROTM(9)
               DROTM(1) = R1
               DROTM(2) = R2
               DROTM(3) = R3
               DROTM(4) = R4
               DROTM(5) = R5
               DROTM(6) = R6
               DROTM(7) = R7
               DROTM(8) = R8
               DROTM(9) = R9
            ELSE    !   missing matrix
               WRITE(*,*) ' *** ERROR IN PH_GEOM - MISSING ',
     +                  'ROTATION MATRIX: ', NR,
     +                  '   FOR DETECTOR:  ', DETNAME(N)
            ENDIF  !   (JR .GT. 0)
          ENDIF  !   (NR .GT. 0)
        ENDDO
c  rotation matrix in the format of nine parameters
        IF(ISAOU .GT. 0)
     +     WRITE(IUSRAOU, '('' G ROTM '', I4, 9F10.6)' ) N, DROTM
        IF(ISMEM .GT. 0)   THEN
          DO  I=1,9
             RROTM(I) = DROTM(I)
          ENDDO
        ENDIF
c
c  calculate six rotation angles from 9 parameters rotation matrix
c
        IF(DROTM(3) .GT.  1.0)   DROTM(3) =  1.0
        IF(DROTM(3) .LT. -1.0)   DROTM(3) = -1.0
        TH = DACOS(DROTM(3))
        THD = TH / DEGRAD
        DROTA(1) = THD
        STH = DSIN(TH)
        IF(STH .GE. ABS(DROTM(1)) .AND. ABS(STH) .GT. 0.1E-35)  THEN
          XX = DROTM(1) / STH
          IF(ABS(XX) .LT. 0.7)  THEN
             PH = DACOS( DROTM(1) / STH )
             PHD = PH / DEGRAD
             IF(DROTM(2) .LT. 0.0)  PHD = 360.0 - PHD
          ELSE
             PH = DASIN( DROTM(2) / STH )
             PHD = PH / DEGRAD
             IF(DROTM(1) .LT. 0.0)  PHD = 180.0 - PHD
             IF(PHD .LT. 0.0)  PHD = 360.0 + PHD
          ENDIF
        ELSE
          PHD = 0.0
          IF(DROTM(1) .LT. 0.0)   PHD = 180.0
        ENDIF
        DROTA(2) = PHD

        IF(DROTM(6) .GT.  1.0)   DROTM(6) =  1.0
        IF(DROTM(6) .LT. -1.0)   DROTM(6) = -1.0
        TH = DACOS(DROTM(6))
        THD = TH / DEGRAD
        DROTA(3) = THD
        STH = DSIN(TH)
        IF(STH .GE. ABS(DROTM(4)) .AND. ABS(STH) .GT. 0.1E-35)  THEN
          XX = DROTM(4) / STH
          IF(ABS(XX) .LT. 0.7)  THEN
             PH = DACOS( DROTM(4) / STH )
             PHD = PH / DEGRAD
             IF(DROTM(5) .LT. 0.0)  PHD = 360.0 - PHD
          ELSE
             PH = DASIN( DROTM(5) / STH )
             PHD = PH / DEGRAD
             IF(DROTM(4) .LT. 0.0)  PHD = 180.0 - PHD
             IF(PHD .LT. 0.0)  PHD = 360.0 + PHD
          ENDIF
        ELSE
          PHD = 0.0
          IF(DROTM(4) .LT. 0.0)   PHD = 180.0
        ENDIF
        DROTA(4) = PHD

        IF(DROTM(9) .GT.  1.0)   DROTM(9) =  1.0
        IF(DROTM(9) .LT. -1.0)   DROTM(9) = -1.0
        TH = DACOS( DBLE(DROTM(9)) )
        THD = TH / DEGRAD
        DROTA(5) = THD
        STH = DSIN(TH)
        IF(STH .GE. ABS(DROTM(7)) .AND. ABS(STH) .GT. 0.1E-35)  THEN
          XX = DROTM(7) / STH
          IF(ABS(XX) .LT. 0.7)  THEN
             PH = DACOS( DROTM(7) / STH )
             PHD = PH / DEGRAD
             IF(DROTM(8) .LT. 0.0)  PHD = 360.0 - PHD
          ELSE
             PH = DASIN( DROTM(8) / STH )
             PHD = PH / DEGRAD
             IF(DROTM(7) .LT. 0.0)  PHD = 180.0 - PHD
             IF(PHD .LT. 0.0)  PHD = 360.0 + PHD
          ENDIF
        ELSE
          PHD = 0.0
          IF(DROTM(8) .LT. 0.0)   PHD = 180.0
        ENDIF
        DROTA(6) = PHD
        IF(ISAOU .GT. 0)
     +     WRITE(IUSRAOU, '('' G ROTA '', I4, 9F12.6)' ) N, DROTA
        IF(ISMEM .GT. 0)   THEN
           DO  I=1,6
              RROTA(I) = DROTA(I)
           ENDDO
           CALL ADDPMCROT(N, RROTM, RROTA)
        ENDIF

C
c  parameters of the volume
c
        JD = 1
        JDROT(1) = N

        IF(ISAOU .GT. 0)   THEN
          WRITE(IUSRAOU, '('' G DEVO '', A4, I4, 3I3, 3F11.5)' )
     +      DETNAME(N), N, ISLSHP, ISLNPAR, JD, TRX, TRY, TRZ
          WRITE(IUSRAOU, '(1X, I3, 10F8.4)' ) IPADFL(N),
     +      PAD1(N), PAD2(N), PAD3(N), PAD4(N), PAD5(N),
     +      PAD6(N), PAD7(N), PAD8(N), PAD9(N), PAD10(N)
          WRITE(IUSRAOU, '(1X, 10F11.5)' ) (SLPAR(I),I=1,ISLNPAR)
          WRITE(IUSRAOU, '(1X, 20I5)' ) (JDROT(J), J=1,JD)
        ENDIF
        IF(ISMEM .GT. 0)   THEN
          CENTER(1) = TRX
          CENTER(2) = TRY
          CENTER(3) = TRZ
          PADPAR(1) = PAD1(N)
          PADPAR(2) = PAD2(N)
          PADPAR(3) = PAD3(N)
          PADPAR(4) = PAD4(N)
          PADPAR(5) = PAD5(N)
          PADPAR(6) = PAD6(N)
          PADPAR(7) = PAD7(N)
          PADPAR(8) = PAD8(N)
          PADPAR(9) = PAD9(N)
          PADPAR(10) = PAD10(N)
          CALL ADDPMCDEVO(N, DETNAME(N), ISLSHP, CENTER,
     +         ISLNPAR, SLPAR,  JD, JDROT,
     +         IPADFL(N), PADPAR)
        ENDIF

c  tests of consistency of volume dimensions and pixelization data
c
       IF(IPADFL(N) .EQ. 1)   THEN   !   box volumes
        IF(NDIVX(N) .NE. 0)  THEN
          STEP = 2.0 * SLPAR(1) / FLOAT(NDIVX(N))
          IF(ABS(STEP-PAD1(N)) .GT. 0.00001)   THEN
             WRITE(*,*) ' *** ERROR IN PH_GEOM - DETECTOR  ',
     +          DETNAME(N), '   X PAD SIZE OR X DIVISION WRONG'
             WRITE(*,*) '                       X PAD = ', PAD1(N),
     +         '   NDIVX = ', NDIVX(N), '  0.5*XSIZE = ', SLPAR(1)
          ENDIF
        ENDIF
        IF(NDIVY(N) .NE. 0)  THEN
          STEP = 2.0 * SLPAR(2) / FLOAT(NDIVY(N))
          IF(ABS(STEP-PAD2(N)) .GT. 0.00001)   THEN
             WRITE(*,*) ' *** ERROR IN PH_GEOM - DETECTOR  ',
     +          DETNAME(N), '   Y PAD SIZE OR Y DIVISION WRONG'
             WRITE(*,*) '                       Y PAD = ', PAD2(N),
     +         '   NDIVY = ', NDIVY(N), '  0.5*YSIZE = ', SLPAR(2)
          ENDIF
        ENDIF
        IF(NDIVZ(N) .NE. 0)  THEN
          STEP = 2.0 * SLPAR(3) / FLOAT(NDIVZ(N))
          IF(ABS(STEP-PAD3(N)) .GT. 0.00001)   THEN
             WRITE(*,*) ' *** ERROR IN PH_GEOM - DETECTOR  ',
     +          DETNAME(N), '   Z PAD SIZE OR Z DIVISION WRONG'
             WRITE(*,*) '                       Z PAD = ', PAD3(N),
     +         '   NDIVZ = ', NDIVZ(N), '  0.5*ZSIZE = ', SLPAR(3)
          ENDIF
        ENDIF
       ENDIF       !   (IPADFL(N) .EQ. 1)     box volumes

       IF(IPADFL(N) .EQ. 6)   THEN   !   TUBS  volumes
         IF(NDIVX(N) .NE. 0)   THEN
           STEP = (SLPAR(2) - SLPAR(1)) / FLOAT(NDIVX(N))
           IF(ABS(STEP-PAD2(N)) .GT. 0.001)   THEN
              WRITE(*,*) ' *** ERROR IN PH_GEOM - DETECTOR  ',
     +           DETNAME(N), '   R PAD SIZE OR R DIVISION WRONG'
              WRITE(*,*) '      R PAD = ', PAD2(N),
     +           '   NDIVR = ', NDIVX(N),
     +           '   R = ', SLPAR(1), '   TO ', SLPAR(2)
           ENDIF
         ELSE   !   NDIVX(N) = 0
           IF(ABS(SLPAR(1)-PAD2(N)) .GT. 0.00001  .OR.
     +            ABS(SLPAR(2)-PAD10(N)) .GT. 0.00001 )  THEN
             WRITE(*,*) ' *** ERROR IN PH_GEOM - DETECTOR  ',
     +          DETNAME(N)
             WRITE(*,*) '     MINIMAL AND MAXIMAL DETECTOR RADIUS =',
     +          SLPAR(1), SLPAR(2)
             WRITE(*,*) '     DIFFERENT THAN RADIUSES   FOR  PADS =',
     +          PAD2(N), PAD10(N)
           ENDIF
           IF(PAD2(N) .GT. PAD3(N)  .OR.
     +       PAD3(N) .GT. PAD4(N)  .OR.  PAD4(N) .GT. PAD5(N)  .OR.
     +       PAD5(N) .GT. PAD6(N)  .OR.  PAD6(N) .GT. PAD7(N)  .OR.
     +       PAD7(N) .GT. PAD8(N)  .OR.  PAD8(N) .GT. PAD9(N)  .OR.
     +       PAD9(N) .GT. PAD10(N) )  THEN
               WRITE(*,*) ' *** ERROR IN PH_GEOM - DETECTOR  ',
     +            DETNAME(N)
               WRITE(*,*) '     RADIUSES IN GS SET PADS COMMAND',
     +                    ' NOT INCREASING'
           ENDIF
         ENDIF
         IF(NDIVY(N) .NE. 0)   THEN
          STEP = ABS(SLPAR(5) - SLPAR(4)) / FLOAT(NDIVY(N))
          IF(ABS(STEP-PAD1(N)) .GT. 0.001)   THEN
            WRITE(*,*) ' *** ERROR IN PH_GEOM - DETECTOR  ',
     +          DETNAME(N), '   PHI PAD SIZE OR PHI DIVISION WRONG'
            WRITE(*,*) '      PHI PAD = ', PAD1(N),
     +          '   NDIVPHI = ', NDIVY(N),
     +          '   PHI = ', SLPAR(4), '   TO ', SLPAR(3)
          ENDIF
         ENDIF
       ENDIF       !   (IPADFL(N) .EQ. 6)     TUBS  volumes

  200 CONTINUE

      RETURN
      END
