      subroutine UGCGET (line, lline, itoken, loct, lent, lnext)
*Documentation_Block
************************************************************************
*
*  Description:
*  UGCGET executes geometry and control information definitions for
*  Phobos.
*  UGCGET fetches the next token from a UGC input line, skipping
*  in-line comments (initially delimited by curly brackets "{ }") and
*  information following the line terminator (initially "!").
*  Tokens are separated by white space, initially consisting of
*  sequential blanks and tabs.
*  Strings are composed of any legal characters; if they contain
*  embedded white space, they must be surrounded by apostrophes (').
*  Also numbers that should be processesd further as text and not a number
*  have to be enclosed in apostrophes.
*  Continuation lines, quotes within strings such as 'isn''t', and
*  logical nesting are not yet implemented.
*
*  Arguments:
*  *line:   storage for text lines and stored definitions
*  *lline:  length of the line to be parsed
*  itoken*: index of the next token = 1, 2, ...; < 0 if error
*  loct*:   location in line of the start of the found token
*  lent*:   length of the found token
*  lnext*:  location in line of the whitespace following the found
*           token
*
*  Called by:
*  UGCIN, while processing a line of UGC text input.
*
*  Calls:
*  none
*
*  Implicit inputs, outputs, side effects:
*  none
*
*  History:
*  Date       Name       Description
*  ----       ----       -----------
*  12-Dec-97  K. W.      token with enclosing apostrophes is picked out
*                        as whole, the aphostrophes have to be omitted
*                        later
*  02-Nov-92  R. Verdier Creation
************************************************************************
*End_Documentation_Block

      implicit none

c Argument declarations:

      character*(*) line
      integer itoken, loct, lent, lline, lnext

c Local declarations:

      character*1 cch, blank /' '/, tab /' '/, apos /''''/,
     *  opencomment /'{'/, closecomment /'}'/

      integer idebug /1/, mode, n1, n2

c End of declarations

c Check both loct and lline for validity

      if (loct .le. 0) then
        if (idebug .gt. 0) print '(a,i8,a)',
     *    'UGCGET called with loct = ', loct, ', taken as 1.'
        loct = 1
      endif
      if (lline .le. 0) then
        if (idebug .gt. 0) print '(a,i8,a)',
     *    'UGCGET called with lline = ', lline, ', taken as 1.'
c The following must be fixed:
        lline = 1
      endif
c Set n1 and n2 to the indices of the first and last-plus-1 non-
c whitespace [blank or tab] characters in the string, and mode to
c 0 (normal search)
      n1 = 0
      n2 = 0
      mode = 0
      do while (n1 .eq. 0 .or. n2 .eq. 0)
        cch = line(loct:loct)
        if (cch .eq. opencomment .and. mode .eq. 0) then
          mode = 2
        else if (cch .eq. closecomment .and. mode .eq. 2) then
          mode = 0
        else if (mode .eq. 2) then
c                !  omit  the comment
        else if (cch .ne. blank .and. cch .ne. tab) then
          if (n1 .eq. 0) then
             if (cch .eq. apos .and. mode .eq. 0)  mode = 1
             n1 = loct  !  include starting apostrophe
          else if (cch .eq. apos .and. mode .eq. 1) then
             if (line(loct-1:loct-1) .ne. apos) mode = 0
c The treatment of 'xxx''xxx' here must be changed so that the
c two internal apostrophes are compressed to one; this apparently
c requires all strings to be moved to separate storage, to avoid
c modifying input.
          endif
        else
          if (n1 .ne. 0 .and. n2 .eq. 0 .and. mode .eq. 0) then
            n2 = loct
          endif
        endif
        if (loct .ge. lline) n2 = lline
        if (idebug .gt. 2) print '(a,i3,3a,i3,a,i3)',
     *    ' UGCGET: loct=', loct, ', cch=', cch, ', n1=', n1,
     *    ', n2=', n2
        loct = loct + 1
      end do
      itoken = itoken + 1
      loct = n1
      lnext = n2
      lent = n2 - n1

      return
      end
