      SUBROUTINE UGINIT
*Documentation_Block
************************************************************************
*
* Description:
*  Initializes GEANT program, reads control dataset(s), and sets up
*  geometry description for Phobos.
*
* Arguments: none
*
* Called by:
*  GEPHOB, the main program
*
* Calls:
*  FFKEY, FFSET, GDRAW, GFFGO, GINIT, GOPEN, GPART, GPHYSI,
*  GPRINT, GZINIT, STPARS, STTOIN, STTORL, UFILE, UGCIN, UGEOM,
*  USCOM, USINIT, VZERO, PH_OPT
*  ADDPMCRINFO, PMCEOH, GUFLD
*
* Implicit inputs, outputs, side effects:
*  Via COMMONs listed below
*
* File: pmc_src:UGINIT.FOR
*
*  History:
*  Date       Name       Description
*  ----       ----       -----------
*  23-Nov-98  K. W.      Added calls to ADDPHCRINFO and PMCEOH
*                        First call to GUFLD moved here from PH_OUT
*                        PMC version set to 4.2
*  13-Nov-98  K. W.      New PMC version set to  4.1
*  16-Jul-98  K. W.      New PMC version set to  4.0
*   7-Jul-98  K. W.      store the number of events declared by TRIG
*  27-Apr-98  K. W.      test of ISAOU flag (and ISMEM) moved to PH_GEOM
*                        thus PH_GEOM is always called
*                        Geometry version written to AOU file in PH_GEOM
*  25-Mar-98  K. W.      Change the sequence of information in AOU file
*                        RROT denotes initial and additional rotation in phi
*                        of particles read from the input file
*  12-Feb-98  K. W.      Add call to PH_RESTART to intialize all starting
*                        variables (IEVENT, FIRSTCalls, etc.)
*                        NSLP set to actual number of sensors (with warning)
*  10-Feb-98  K. W.      Modifications necessary for incorporation of PMC
*                        in Phat; added call to PH_OPT to allow interactive
*                        modifications of simulation options
*  31-Dec-97  K. W.      PMC version 3.0 created - with fully impemented
*                        pixelization
*  27-Nov-97  K. W.      AOUF option added - to select fixed or compact
*                        format of AOU file
*  25-Aug-97  K. W.      Opening FLDASC moved to GUFLD
*                        (to enable handling multiple field files)
*  23-Aug-97  K. W.      PIXL option reimplemented, used now to put pixel
*                        numbers into the AOU file
*  23-Apr-97  K. W.      time limit switch - to ignore time limit
*   4-Mar-97  K. W.      Removed unused commands SILD and SILP
*   7-Feb-97  K. W.      SIMVER and CMZ PHOSIM version coordinated at 2.02
*  23-SEP-96  K. W.      SIMVER initialization added (moved from GEPHOB)
*  20-SEP-96  K. W.      Added new parameters for BANG mode=2
*                        More information in the AScii Output file
*                        Removed obsolete keywords
*  25-Jul-96  R. V.      Remove +SEQ,*D. and redundant "," in write,
*                          for ANSI compliance
*  21-Jul-96  R. V.      Removed unused cc, j, n
*  12-Jul-96  K.W.       magnetic field center (MAGX,MAGZ) separated from
*                        the field offset (MOFF(3))
*  09-Jul-96  K.W.       added commands to define magnetic field from
*                        control file
*  04-Jul-96  K.W.       added Ascii OUtput file code (initialzation and
*                        instructions to write the header of AOU file)
*  18-Jul-96  R.V.       Remove the EUCARD facility unless CMZ selects EUCARD
*  08-Sep 95  DMcL       DATE, TIME => Cernlib DATIMH & associated changes
*  23-Jul-95  DMcL       Changed UFILE to SERVER calls for UNIX port
*                        idebug => jdebug to avoid GCFLAG conflict
*  31 Mar-95  R.V.       Change STRA to SECT to avoid GEANT 3.21 conflict
*  20-MAR-95  K.W.       A call to USCOM was added
*  10-MAR-95  K.W.       Changed parameters required by GUKINE and BANG
*  10-May-94  R.V.       Add data SEQs NTUD & RHSILD, after D. McLeod
*   5-May-94  R.V.       Remove control card setting IZEB=DISP
*   9-Nov-93  R.V.       BANG.CMN -> BANGC.CMN to avoid MMS confusion;
*                        also NTUEX.CDA -> NTUEXD.CMN for code mgmt.
*  17-Jun-93  R.V.       Added NHTK, NTTK, NTUEX.CMN & .CDA for ntuples,
*                        NTPE and NTCO for multi-pass processing
*  20-Mar-93  K. Wozniak Added MAGT = 3 for cylindrical field map
*   7-Feb-93  R.V.       Added SAHO to control ascii hit output
*  12-Jan-93  R.V.       FileName added to UFILE;
*                        COMMON.NTU & COMMON.HST -> NTU.CMN & HST.CMN
*  11-Dec-92  R.V.       Changed to use UFILE instead of UFILES, move
*                        GPRINT call from UGEOM
*  23-Nov-92  R.V.       Added MAGT, ZOFF, and RFLD setting for
*                        C.P.'s perfect dipole field mods to GUFLD
*  14-Nov-92  R. Verdier Connected to UGC text file geometry input
*                        system, made EUCARD call conditional
*   1 Jul 92  J. Shea    Separated from GE_..., replaced hard-coded
*                        data by card input, added CARDS
*   1 Jun 92  C. Parsons Moved from GPH into GE_ALL-THE-REST.FOR,
*                        changed COMMON=>.CMN, UHINIT=>USINIT, ...
*  28-Mar-92  D. McLeod  Creation
*
************************************************************************
*End_Documentation_Block

      IMPLICIT NONE

C     Common blocks from GEANT which might be referred to:

      INTEGER LIN,LOUT,NUNITS,LUNITS
      COMMON/GCUNIT/LIN,LOUT,NUNITS,LUNITS(5)

      INTEGER NHSTA,NGET,NSAVE,NSETS,NPRIN,NGEOM,NVIEW,NPLOT
      INTEGER NSTAT,LHSTA,LGET,LSAVE,LSETS,LPRIN
      INTEGER LGEOM,LVIEW,LPLOT,LSTAT
      COMMON/GCLIST/NHSTA,NGET ,NSAVE,NSETS,NPRIN,NGEOM,NVIEW,NPLOT
     +       ,NSTAT,LHSTA(20),LGET (20),LSAVE(20),LSETS(20),LPRIN(20)
     +             ,LGEOM(20),LVIEW(20),LPLOT(20),LSTAT(20)

*KEEP,BANGC.
C Event generator parameters                 File: BANGC.CMN
C 20-Sep-96  K. Wozniak Added new parameters to control BANG mode = 2
C                       RSPX, RSPZ, RSPR (removed RMAG)
C 10-Mar-95  K. Wozniak Added new parameters to control track generation
c                       and selection (ETAMIN,ETAMAX, IDMIN,IDMAX ..)
c 22-Jun-94  R. V.      Remove NSEQN storage (see PH_HIT)
c 10-May-94  D. McLeod  Increased MXMUL to 13000 for HIJET events
c 17-Dec-92  R. Verdier Changed name from BANG to BANGC for CMS
c 22-Nov-92  D. Mcleod  Added EXF and NSEQN
      INTEGER MXMUL
      PARAMETER (MXMUL=13000)  ! maximum multiplicity for BANG etc.
      INTEGER IRMUL         !  multiplicity in random particle generation
      INTEGER IRARM         !  particles going to 1 or 2 arms selected
      INTEGER IRPID(2)      !  particle id range: min, max
      REAL RMOM(3)          !  momentum of particles:  min, max and step
                            !  (first step if 1/p is changed)
      REAL REXP             !  mean transv. mom. - for momentum generation
      REAL RETA(2)          !  ETA range: min, max
      REAL RANG(3)          !  starting theta angle and step
      REAL RPHI(2)          !  range of emission phi angle: min, max
      REAL RROT(2)          !  phi initial and additional rotation angle (GEN=0)
      REAL RBXX,RBYY,RBZZ   !  central position of the vertex
      REAL RBXY             !  width of the beam intersection in x-y
      REAL RBDX,RBDY,RBDZ   !  the range of positions of the vertex
C                           !
      REAL RSPX,RSPZ,RSPR   !  position and radius of the circle describing
C                           !  area covered by spectrometer
      REAL PHIR,CZ,ZC,RS    !  working variables for BANG
      COMMON/RHBNG/IRMUL, IRARM, IRPID, RMOM, REXP, RETA, RANG, RPHI,
     1             RROT,
     2             RBXX,RBYY,RBZZ, RBDX, RBDY, RBDZ, RBXY,
     3             RSPX,RSPZ,RSPR, PHIR,CZ,ZC,RS
*KEEP,CARDS.
C  Common blocks for GPH PHOBOS simulation:  FILE CARDS.CMN
C  General geometric layout of PHOBOS detector exclusive of detectors
C  These values are on input cards or data statements.
C
      REAL YGAP,YCRH,THCR
C     1   OUT1,OUT2,BMV1,BMV2,BMT1,BMT2,SPE1,SPE2,CRB1,CRB2,
C     1   CRS1,CRS2,FIL1,FIL2
C
      COMMON/CARDS/YGAP,YCRH,THCR
C         OUT1(6),OUT2(6),BMV1(6),BMV2(6),
C     1   BMT1(6),BMT2(6),SPE1(7),SPE2(7),CRB1(6),CRB2(6),
C     1   CRS1(6),CRS2(6),FIL1(6),FIL2(6)



*KEEP,FLAG.
C  PHOBOS flags, run control, operating modes, misc.   File FLAG.CMN
C   7-Jul-98  K. W.      Added  ITRIG flag (initial numer in TRIG command)
C  27-Nov-97  K. W.      Added IAOUF flag
C  31-Mar-95  R.V.       Change STRA to SECT to avoid GEANT 3.21 conflict
C  22-Nov-92  D. McLeod
C   These are set in input cards
      INTEGER ISASC,ISHBO,ISHDO,ISNTU, ISMEM,ISAOU,IAOUF, ISECT,IDRAW,
     1        IGEN,IANAL,ISCAT,IPIXL,ISEQ,NTRK,IZEB, ITRIG
      REAL WPIXL,WLOSS
      COMMON/RHFLG/ISASC,ISHBO,ISHDO,ISNTU,
     1  ISMEM,ISAOU,IAOUF, ISECT,IDRAW,
     2  IGEN,IANAL,ISCAT,IPIXL,ISEQ,NTRK,IZEB,WPIXL,WLOSS, ITRIG
*KEEP,FLAG2.
C  PHOBOS flags, run control, operating modes, misc.   File FLAG2.INC
C  14-Nov-95  D. McLeod
*     Glossary:  IPSIG = identity of kill signal sent by user as interrupt
*                EXSERV = flag for existence of external server file
*                SERNAME = name part only of server file, also run name
*                ROOTPATH = program environment pathname (no trailing /)
*                ROOTVERS = version subdirectory e.g. /prod, /test, /dev
*                SIMVER  = version number for the PHOBOS Monte Carlo
      INTEGER IPSIG
      REAL SIMVER
      LOGICAL EXSERV
      CHARACTER SERNAME*20, ROOTPATH*60, ROOTVERS*10
      COMMON/RHFLG2/IPSIG, EXSERV, SIMVER, SERNAME, ROOTPATH, ROOTVERS
*KEEP,IO.
C Common block for GPH logical units etc.: file IO.CMN
c 13-Aug-96  R. V.       Added USRBATCHK for KUIP commands for UGCONS
c 21-Jul-96  R. V.       Remove unused record; note CARDDEF now obsolete
c 22-Feb-93  R. V.       Added USRMCNTU and IMCNTU for MC Ntuple
c 14-Nov-92  R. Verdier  Added UGCIN, IUGCIN, CARDDEF and ICARDDEF
c                        for UGC and EUCARD (obsolete) input units
C    Apr-92  C. Parsons  Replacement for original COMMON.CDT
      INTEGER USRGCIN, CARDDEF, USRIN, USROUT, USRMAP, USRNTU, USRZEB,
     *  USRMCNTU, USRBATKUMAC
      PARAMETER (USRIN = 4, USROUT = 16, USRMAP = 18)
      PARAMETER (USRNTU = 15, USRZEB = 17, USRGCIN = 19, CARDDEF = 20)
      PARAMETER (USRMCNTU = 21, USRBATKUMAC = 22)
      INTEGER ICARDDEF, IMCNTU, IUSRGCIN, IUSRIN, IUSROUT, IUSRMAP,
     *  IUSRNTU, IUSRZEB, IUSRAOU, IUSRBATKUMAC
      COMMON /IO/ IUSRIN, IUSROUT, IUSRMAP, IUSRNTU, IUSRZEB, IUSRGCIN,
     *  ICARDDEF, IMCNTU, IUSRAOU, IUSRBATKUMAC
*KEEP,GCBANK.
      INTEGER IQ,LQ,NZEBRA,IXSTOR,IXDIV,IXCONS,LMAIN,LR1,JCG
      INTEGER KWBANK,KWWORK,IWS
      REAL GVERSN,ZVERSN,FENDQ,WS,Q
C
      PARAMETER (KWBANK=69000,KWWORK=5200)
      COMMON/GCBANK/NZEBRA,GVERSN,ZVERSN,IXSTOR,IXDIV,IXCONS,FENDQ(16)
     +             ,LMAIN,LR1,WS(KWBANK)
      DIMENSION IQ(2),Q(2),LQ(8000),IWS(2)
      EQUIVALENCE (Q(1),IQ(1),LQ(9)),(LQ(1),LMAIN),(IWS(1),WS(1))
      EQUIVALENCE (JCG,JGSTAT)
      INTEGER       JDIGI ,JDRAW ,JHEAD ,JHITS ,JKINE ,JMATE ,JPART
     +      ,JROTM ,JRUNG ,JSET  ,JSTAK ,JGSTAT,JTMED ,JTRACK,JVERTX
     +      ,JVOLUM,JXYZ  ,JGPAR ,JGPAR2,JSKLT
C
      COMMON/GCLINK/JDIGI ,JDRAW ,JHEAD ,JHITS ,JKINE ,JMATE ,JPART
     +      ,JROTM ,JRUNG ,JSET  ,JSTAK ,JGSTAT,JTMED ,JTRACK,JVERTX
     +      ,JVOLUM,JXYZ  ,JGPAR ,JGPAR2,JSKLT
C
*KEEP,GCCUTS.
      COMMON/GCCUTS/CUTGAM,CUTELE,CUTNEU,CUTHAD,CUTMUO,BCUTE,BCUTM
     +             ,DCUTE ,DCUTM ,PPCUTM,TOFMAX,GCUTS(5)
C
      REAL          CUTGAM,CUTELE,CUTNEU,CUTHAD,CUTMUO,BCUTE,BCUTM
     +             ,DCUTE ,DCUTM ,PPCUTM,TOFMAX,GCUTS
C
*KEEP,GCFLAG.
      COMMON/GCFLAG/IDEBUG,IDEMIN,IDEMAX,ITEST,IDRUN,IDEVT,IEORUN
     +        ,IEOTRI,IEVENT,ISWIT(10),IFINIT(20),NEVENT,NRNDM(2)
      COMMON/GCFLAX/BATCH, NOLOG
      LOGICAL BATCH, NOLOG
C
      INTEGER       IDEBUG,IDEMIN,IDEMAX,ITEST,IDRUN,IDEVT,IEORUN
     +        ,IEOTRI,IEVENT,ISWIT,IFINIT,NEVENT,NRNDM
C
*KEEP,GCKINE.
      COMMON/GCKINE/IKINE,PKINE(10),ITRA,ISTAK,IVERT,IPART,ITRTYP
     +      ,NAPART(5),AMASS,CHARGE,TLIFE,VERT(3),PVERT(4),IPAOLD
C
      INTEGER       IKINE,ITRA,ISTAK,IVERT,IPART,ITRTYP,NAPART,IPAOLD
      REAL          PKINE,AMASS,CHARGE,TLIFE,VERT,PVERT
C
*KEEP,GCPHYS.
      COMMON/GCPHYS/IPAIR,SPAIR,SLPAIR,ZINTPA,STEPPA
     +             ,ICOMP,SCOMP,SLCOMP,ZINTCO,STEPCO
     +             ,IPHOT,SPHOT,SLPHOT,ZINTPH,STEPPH
     +             ,IPFIS,SPFIS,SLPFIS,ZINTPF,STEPPF
     +             ,IDRAY,SDRAY,SLDRAY,ZINTDR,STEPDR
     +             ,IANNI,SANNI,SLANNI,ZINTAN,STEPAN
     +             ,IBREM,SBREM,SLBREM,ZINTBR,STEPBR
     +             ,IHADR,SHADR,SLHADR,ZINTHA,STEPHA
     +             ,IMUNU,SMUNU,SLMUNU,ZINTMU,STEPMU
     +             ,IDCAY,SDCAY,SLIFE ,SUMLIF,DPHYS1
     +             ,ILOSS,SLOSS,SOLOSS,STLOSS,DPHYS2
     +             ,IMULS,SMULS,SOMULS,STMULS,DPHYS3
     +             ,IRAYL,SRAYL,SLRAYL,ZINTRA,STEPRA
      COMMON/GCPHLT/ILABS,SLABS,SLLABS,ZINTLA,STEPLA
     +             ,ISYNC
     +             ,ISTRA
*
      INTEGER IPAIR,ICOMP,IPHOT,IPFIS,IDRAY,IANNI,IBREM,IHADR,IMUNU
     +       ,IDCAY,ILOSS,IMULS,IRAYL,ILABS,ISYNC,ISTRA
      REAL    SPAIR,SLPAIR,ZINTPA,STEPPA,SCOMP,SLCOMP,ZINTCO,STEPCO
     +       ,SPHOT,SLPHOT,ZINTPH,STEPPH,SPFIS,SLPFIS,ZINTPF,STEPPF
     +       ,SDRAY,SLDRAY,ZINTDR,STEPDR,SANNI,SLANNI,ZINTAN,STEPAN
     +       ,SBREM,SLBREM,ZINTBR,STEPBR,SHADR,SLHADR,ZINTHA,STEPHA
     +       ,SMUNU,SLMUNU,ZINTMU,STEPMU,SDCAY,SLIFE ,SUMLIF,DPHYS1
     +       ,SLOSS,SOLOSS,STLOSS,DPHYS2,SMULS,SOMULS,STMULS,DPHYS3
     +       ,SRAYL,SLRAYL,ZINTRA,STEPRA,SLABS,SLLABS,ZINTLA,STEPLA
C
*KEEP,HST.
C  File HST.CMN (originally COMMON.HST)
C  Common block for inline plot control.. to generate plots during
C  the GEANT run from the track Ntuples (rather than later via PAW)
C  Its contents are set by control cards in the same file as FFREAD:

C  HIST ID TAGNAME  (short form.. default bins and interval)
C    TAGNAME can be Ntuple tag name or e.g. #35 for 35th entry

C  HIST ID TAGNAME NCHANNELS XMIN XMAX is the long form.
C  HIST ID TAGX NX XMI XMA TAGY NY YMI YMA  scatterplot, args as HBOOK2
C  (The number of arguments distinguishes these three formats.)

C      HCONTROL(I,J), J = 1,NHMX contains the control information:
C  I = 1:  User supplied ID number of plot, floated integer
C  I = 2:  Ntuple entry number (floated), from TAGNAME or # input
C  I = 3:  number of channels (0 if short form card)
C  I = 4-5 or 4-9: arguments as in card formats above, all floating

      INTEGER NHMX,NHSTS
      PARAMETER(NHMX=50)
      REAL HCONTROL
      COMMON/RHHST/HCONTROL(9,NHMX),NHSTS

*KEEP,MUPEP.
c MUPEP.CMN: PHOBOS MC multi-pass event processing variables
c 16-Mar-95 K. W.     NeSkip added
c  6-Jun-93 R. V.     Creation
c Multi-pass event processing breaks up each event into groups of
c tracks, in order to avoid overflowing Zebra track storage.
c Common for access to control variables:
      common /MUPEP/ EPState, NtConc, NtpE, NtPrim, NeSkip, NtSkip,
     1  MPind, NtPrev, NtTot, NtMax, NvPrev
c Parameters and data values are assigned in MUPEP.CDA.
c Event processing state: EPState = 0 for first pass,
c 1 for continuing pass, 2 for final pass
      integer EPState
c Current pass index:
      integer MPind
c Maximum number of tracks to process concurrently in a group:
      integer NtConc
c Maximum track number for this group:
      integer NtMax
c Maximum number of tracks per event:
      integer NtpE
c Number of primary and secondary tracks generated before this pass:
      integer NtPrev
c Number of primary tracks and vertices generated:
      integer NtPrim, NvPrev
c Number of input file events to skip initially:
      integer NeSkip
c Number of tracks to skip initially:
      integer NtSkip
c Total number of primary and secondary tracks generated:
      integer NtTot
*KEEP,NTU.
C Common block for Ntuple image and other analysis info.  File NTU.INC
C There may be one Truth Ntuple and/or one Hits Ntuple for each track.
C These must be correlated into events using the event number and other
C event information.
C 10-Jul-95  K. W.      Increased number of planes for spectrometer to 17
C 10-Mar-95  K. W.      Increased number of planes for spectrometer
C 19-Jul-94  R. V.      Update tagnames: Cxx -> Dxx, VOLTERM -> VOLUME
C 10-May-94  R. V.      Modify Ntuples to contain new hits information;
C                       add NTTAGS common for tags; move data to NTUD.INC
C 29-Jun-93  M. Baker   Modify Ntuples
C 10-Jan-93  R.Verdier  Separate Hits and Truth ntuples; rename to NTU.CMN
C    Mar-92  D. Mcleod  Creation as COMMON.NTU
C Number:  Name:    --------------- Description:----------------------------
C>>>>>>>GENERAL INFORMATION<<<<<<<<<<<<<<<<<<
*   1      RUN      run number
*   2      EVENT    event number
*   3      TRACK    track number
*   4      GEN      event generator type; 1 = fixed, 2 = random, etc.
*   5      ANAL     event analysis type; 1 = Ntuple only, 2 = resolution, etc.
*   6      SCAT     ISCAT = IMULS + 3*ILOSS + 9*IPIXL; 1st 2 are in /GCPHYS/
*   7      BIMP     Impact Parameter   ** NOT YET IMPLEMENTED **
C>>>>>>>THE TRUTH<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
*   8      XPROD    track origin
*   9      YPROD       "    "
*   10     ZPROD       "    "
*   11     IDPART   particle type (GEANT code)
*   12     CHRG     particle charge
*   13     TP       particle momentum (at production)
*   14     TK          "     kinetic energy    "
*   15     TPHI        "     azimuth angle
*   16     TTHETA      "     polar angle
C>>>>>>>ANALYSIS RESULTS..FOR THE WHOLE TRACK<<<<<<<<<<<<<<<<<<
*   17     ACCEPT   Acceptance Flag (+/-1=acc. in +/- wing, 2=both, 0=neither)
*   18     NHITSL   number of hits on track, set to a maximum of NHTM
*   19     USR1     spare, for later use
*   20     USR2              "
C>>>>>>>TRACK HITS<<<<<<<<
* The Truth ntuple stops here; the Hits ntuple continues with
* A series of particle history words
* NHITSL groups of 12 words each, at index = 32 + 12*(Ihit-1):
*
*   21     KCASE    creation mechanism
*   22     PARENT   track number
*   23     VOLUME   creation volume name
*   24     XTERM
*   25     YTERM
*   26     ZTERM
*   27     PXTERM
*   28     PYTERM
*   29     PZTERM
*   30     ETERM
*   31     STOP     stop mechanism

*   32+    PL       plane number hit
*   33+    X        Track center in plane in master coords (maybe pixeled)
*   34+    Y          "     "     "   "    "  "      "       "       "
*   35+    Z          "     "     "   "    "  "      "       "       "
*   36+    DE       Energy deposited in the plane
*   37+    DX       Exit minus entry position in master coord system
*   38+    DY        "    "    "    "      "        "    "
*   39+    DZ        "    "    "    "      "        "    "
*   40+    EFIN     track energy at exit from plane
*   41+    TOF      time registered by a TOF detector, in seconds
*   42+    ITRK     track number for this hit
*   43+    IS       Combined information:  IS = IVOL + 1000 * ISEQ
*                   ISEQ = sequential (first = 1) hit number on this track
*                   IVOL = system volume number of the plane
C The maximal number of hits is now NHTM = 17
C There are thus 31 + 12*NHTM or 235 entries in a Hits Ntuple, with zeroes
C after the last hit

      INTEGER NVMCTUP,NVTUP,NTOFF,NHTM
      PARAMETER (NTOFF=31, NVMCTUP=20, NHTM=17)
      PARAMETER (NVTUP=31+12*NHTM)

      REAL    RUN, EVENT, TRACK, GEN, ANAL, SCAT, BIMP
      REAL    XPROD, YPROD, ZPROD, IDPART, CHRG, TP, TK, TPHI, TTHETA
      REAL    ACCEPT, NHITSL, USR1, USR2
      REAL    STOP, PARENT, VOLTERM, XTERM, YTERM, ZTERM, PXTERM, PYTERM
      REAL    PZTERM, ETERM, UNUSED
      REAL    TRK(12,NHTM), TUP(NVTUP)

      COMMON/RHTUP/ RUN, EVENT, TRACK, GEN, ANAL, SCAT, BIMP,
     1  XPROD, YPROD, ZPROD, IDPART, CHRG, TP, TK, TPHI, TTHETA,
     2  ACCEPT, NHITSL, USR1, USR2, STOP, PARENT, VOLTERM, XTERM,
     3  YTERM, ZTERM, PXTERM, PYTERM, PZTERM, ETERM, UNUSED,
     4  TRK

      EQUIVALENCE (TUP(1),RUN)

      COMMON /NTTAGS/ TAGS(NVTUP)
      CHARACTER*8 TAGS
*KEEP,NTUEX.
c NTUEX.INC: PHOBOS MC Ntuple option extensions
c 10-May-94 R. V.     Correct filenames in documentation
c 6-Jun-93 R. V.     Creation
c This include file defines options for the size and format of the
c Phobos ntuples.  Data values are assigned in NTUEXD.INC.
c Define sizes for allocating ntuples, in 4-byte words.  The NHtrks
c and NMCtrks values limit the total size of the created disk file.
c The default values assigned to NWHits and NWMC here are actually
c overridden by Nvtup and Nvmctup, respectively, in NTUD.INC.
      common /NTUEX/ LreclW, NWHits, NWMC, NWover, NWPRIM,
     1               NHtrks, NMCtrks
      integer
     1  LreclW,  ! Disk file logical record length, words
     2  NWHits,  ! # Hits ntuple variables [columns]
     3  NWMC,    ! # MC Truth ntuple variables [columns]
     4  NWover,  ! Approx. Zebra system overhead, words
     5  NWprim,  ! Primary storage buffer, words
     6  NHtrks,  ! Maximum # of tracks in Hits ntuple
     7  NMCtrks  ! Maximum # of tracks in MC ntuple
*KEEP,RHSIL.
C RHSIL.CMN: PHOBOS detector plane dimensions, positions, and
c granularity; UGC parameters
c The actual and maximum numbers of detector planes and detector hit
c parameters.  Their default values are defined in RHSILD.INC.
c 12-Dec-97 K. W.       PADSOLD and PADSNEW from UGCSET moved to a common
c  4-Nov-97 K. W.       Increased number of PADS parameters to 10
c                       Increased NSLPM to 800
c  1-Jul-97 K. W.       Added data for GEANT pixelization (GSDVN)
c  1-May-97 R. V.       Increased NSLPM to 500
c  3-MAR-97 K. W.       Removed SPL, added PADS data
C  3-Jan-97 K. W.       Increased NSLPM to 180
c  3-Jul-96 K. W.       Geometry version number added
c  5-Apr-96 D. McL.     Moved data value of UgcpMax to RHSILD
c  ...
c    Mar-92 D.McLeod    Creation
c Storage limit for number of silicon planes:
      INTEGER NSLPM
      PARAMETER (NSLPM=800)
c Storage limit for number of parameters for each hit:
      integer NSipMaxp
      parameter (NSipMaxp = 12)
c Safety limit for number of parameter values that may be entered in
c a UGC text input record:
      integer UgcpMaxp
      parameter (UgcpMaxp = 10)
c Number of silicon planes in use:
      integer NSLP
c Additional information on detectors - pad sizes:
C  DETNAME  - name of the detector volume
C  DIVXNAME - name for division in X direction
C  DIVYNAME - name for division in Y direction
C  DIVZNAME - name for division in Z direction
C  NDIVX    - number of divisions in X direction
C  NDIVY    - number of divisions in Y direction
C  NDIVZ    - number of divisions in Z direction
C  IPADFL   - pad type flag
C  PADn   - four pad parameters
C             (pad size in X, Y, Z direction for IPADFL=1 for example)
      CHARACTER*4 DETNAME(NSLPM), DIVXNAME(NSLPM),
     +             DIVYNAME(NSLPM), DIVZNAME(NSLPM)
      INTEGER NDIVX(NSLPM), NDIVY(NSLPM), NDIVZ(NSLPM)
      REAL PAD1(NSLPM),PAD2(NSLPM),PAD3(NSLPM),PAD4(NSLPM),PAD5(NSLPM)
      REAL PAD6(NSLPM),PAD7(NSLPM),PAD8(NSLPM),PAD9(NSLPM),PAD10(NSLPM)
      INTEGER NDETS, IPADFL(NSLPM)
      LOGICAL PADSOLD, PADSNEW
      COMMON /RHPADS/ NDETS, PADSOLD, PADSNEW,
     +       DETNAME, DIVXNAME, DIVYNAME, DIVZNAME,
     +       NDIVX, NDIVY, NDIVZ,
     +       IPADFL, PAD1, PAD2, PAD3, PAD4, PAD5,
     +               PAD6, PAD7, PAD8, PAD9, PAD10
c Number of silicon planes - compared with final value of NDETS
      COMMON /RHSIL/ NSLP
c UGC parameter information:
      integer UgcpMax
      real UGCpars(UgcpMaxp)
c Declare silicon plane parameters; their default values are specified
c in RHSILD.INC
      integer NSIP
c Names of parameters:
      CHARACTER*4 SINAMH(NSipMaxp)
c Mumber of bits stored for each parameter:
      INTEGER SIBITH(NSipMaxp)
c Multiplicative scale factors and origins, defined so that
c   Stored value = (Parameter value + SIORIG) * SIFACT:
      REAL SIFACT(NSipMaxp), SIORIG(NSipMaxp)
c Common for access to UGC parameters and hit parameter definitions:
      common /UGCPAR/ UgcpMax, UGCpars,
     1                NSIP, SINAMH, SIBITH, SIORIG, SIFACT
      real GEOV  !  geometry version number
      common /ugGEOV/ GEOV
*KEEP,RHSYS.
C  Common blocks for GPH PHOBOS simulation:  FILE RHSYS.CMN
C  General geometric layout of PHOBOS detector exclusive of detectors
C  These values are on input cards or data statements, except *MAX are
C  calculated to encompass all elements of the system.
C  20-Sep-96  K.W       Removed obsolete BMTBRAD, BERTHIK
C                       Removed FANGL - now local variable in BANG
C  12-Jul-96  K.W       Position of the magnet pole center MAGX, MAGZ
C                       (previously XOFF,ZOFF) separated from the magnet
C                       offset (now MOFF - 3-D vector)
C  14 Dec-92  R.V.      TMAG redefined and changed to integer
C  22-Nov-92 D. McLeod  Renamed from COMMON.SYS; TMAG, ZOFF, XOFF
C                       are added from the original GPH version
      REAL   BFLD         ! Magnetic field strength
     &      ,MAGX,MAGZ    ! Coordinates of field center
     &      ,MOFF(3)      ! offset of the field
     &      ,RFLD         ! Radius of field (perfect dipole only)
     1   ,XMAX,YMAX,ZMAX,ZMIN,XMMAX,ZMMAX
      INTEGER MAGT, TMAG  ! Field type: 0 Zebra bank, 1 = dipole,
                          ! 2 = ASCII map
      COMMON/RHSYS/ BFLD,MAGT,MAGX,MAGZ,MOFF
     1  ,RFLD,XMAX,YMAX,ZMAX,ZMIN,XMMAX,ZMMAX
      EQUIVALENCE (TMAG, MAGT)
*KEEP,UHITS.
c UHITS.CMN: control variables for UHITSOUT.FOR
c  5-Feb-93  R. Verdier     Creation
      integer draw_mode, HitsOut
      logical draw_hits
      common /Hitsocon/ HitsOut, draw_hits, draw_mode
*KEEP,TLIMIT.
C   CPU time limit flag
C   NTLIM = 1  simulation stops about 1 second before the CPU
C              time assigned to batch job is exhausted
C   NTLIM = 0  ignores estimated CPU time limit, program continues
C              with a warning message and could eventually crash.
C              It is the default as for long queues CERN library
C              assumes only 9999 seconds (about 2 hours 47 minutes)
C
      INTEGER NTLIM
      COMMON /TLIMIT/ NTLIM
*KEND.

C  Local:
      CHARACTER*8 FIELD(11)
      character*63 FileName
      character*1 apostr
      INTEGER jdebug/1/, I, NFLDS, LENFLD(11), STTOIN, ITAG, IER
      INTEGER HDATE(2), HTIME(2)
      CHARACTER RECORD*80
c Declaration to force loading
      external PMCBDAT
C Added for SERVER long form calls:
      REAL XDUM(3), BDUM(3)
      REAL STTORL
      EXTERNAL STTOIN,STTORL   !SHELL routines for decoding strings
      CHARACTER*120  LINE
c Declarations to use UGC geometry and control input:

      integer lline, lstor, iugcres
      parameter (lline = 132, lstor = 200)
      character*(lline + lstor) storage
c Fortran logical units for UGC input (file GCDEFS) and report output:
c IUSRGCIN and IUSROUT are set by calls to subroutine SERVER; but the
c latter is not used by UGC.
      integer lurep /6/

C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C Key to data cards below (these go into GPH.DAT or similar .DAT files):
C  Keyword    type            what it sets    (Units cm., Kgauss, Gev/c)
C
C  Magnetic field parameters:
C  BFLD       REAL       magnetic field, central value
C  MAGT       INTEGER    if = 0, use Zebra field map; if = 1, use
c                        dipole field; if = 2, use ASCII field map
C                        if = 3, use ASCII cylindrical field map
C  TMAG       INTEGER    alias to MAGT
C  MAGX       REAL       dipole field center X
C  MAGZ       REAL       dipole field center Z
C  MOFF       3*REAL     offset of the magnetic field
C  RFLD       REAL       dipole field radius
C
C  Standard GEANT cards we will likely use:
C  TRIG       INTEGER    number of events simulated (upper limit if reading)
C  RUNG       INTEGER    user run number
C  MULS       INTEGER    multiple scattering type: 0 = none,
C                        1 = Gaussian (default), 2 = Moliere
C  DCAY       INTEGER    decay in flight: 0 =none, 1 = make secondaries,
C                        2 = ignore secondaries.  (1 = default)
C  LOSS       INTEGER    1 = no fluctuations; average energy loss
C                        2 = Landau fluctuations (default), 0 = no loss
C  CUTS       REALS      up to 16 entries, see GEANT. We're interested in
C                        the second entry, CUTELE; set large e.g. 0.1 to
C                        suppress delta rays when LOSS = 1 is set.
C  RNDM       INTEGERS   2 integers to establish random # seed
C  DEBUG      INTEGERS   2 integers to set range of event numbers for debug
C  SWIT       INTEGERS   up to 10 user flags to control printout etc.
C                        via ISWIT(10) in /GCFLAG/

C  Flags:  0 or card omitted = option is suppressed
C                        (DATA refers to the data input file name)
C  GEN        INTEGER    Event generator type.  1 = fixed, starts at beam
C                        2 = BANG aimed toward the detectors,
C                        3 = random BANG, 4 = momentum, position or angle
C                        incremented by DMOM, DXX or DANG from one track
C                        to the next (one track per event), 5 = Monte
C                        Carlo Zebra input
C  PIXL       INTEGER    pixilated flag.. 0 = no pixel numbers are calculated
C                        1 = pixel numbers are calculated and stored
C                            in AOU file (information from geometry and
C                            SET PADS command are used)
C  ANAL       INTEGER    Event analysis type. 1 = generate Ntuple only
C                        2 = calculate errors from fit ??
C  SAOU       INTEGER    0 - no AOU output
C                        1 - only particles with hits written to AOU file
C                        2 - primariy particles and particles with hits written
C                            to AOU file
C                        3 - primary particles, particles with hits and all
C                            their parents written to AOU file (default)
C                        4 - all particles written to AOU file
C  SMEM       INTEGER    interface to the PHAT memory (classes)
C                        0 - no information transfered to PHAT
C                        1 - 4  the same selection of particles transfered to
C                               PHAT memory as for AOU file
C  SHBO       INTEGER    1 = DATA.HST binary plot output (for PAW)
C  SHDO       INTEGER    1 = HISTO is called to output plots in DATA.OUT
C  SNTU       INTEGER    1 = Ntuples are output to file DATA.NTU
C  SZEB       INTEGER    1 = Output SETS, KINE & HITS banks in DATA.FZO
C  DRAW       INTEGER    1 = draw apparatus, 2 = draw hits also
C  WPIX       REAL       1.0 = include tracking errors in chi squared sum
C  WLOS       REAL       1.0 = include energy loss in chi squared sum
C
C  Event generator parameters, depending on GEN above:
C  NAME    TYPE   GEN=     MEANING

C  RMUL   INTEG   1-4,11-14   Multiplicity in BANG random generator
C  RARM   INTEG   all         Selected one or two arms
C  RANG   3 REAL   4          Polar angle range and step
C  RPHI   2 REAL  all         Azimuth range
C  RETA   2 REAL  0,1,3       Eta max and min
C  RROT   2 REAL  0           Phi rotation angle: initial and after rewinding
C                             of the input file
C  RBXX   REAL    all         Position of particle production, vertex of
C  RBYY   REAL    all         the interaction
C  RBZZ   REAL    all
C  RBXY   REAL    0,1-3       Standard dev. of projected intersection width in x-y
C  RBDX   REAL    0,1-4       interval arround the vertex
C  RBDY   REAL    0,1-4
C  RBDZ   REAL    0,1-4
C  RMOM   3 REAL  all         Particle momentum range and step
C  REXP   REAL    3           Mean transverse momentum - falloff parameter
C  RPID   2 INTEG all         Particle type range; GEANT convention

C  RSPR   REAL    2           Radius of the circle describing the spectgrometer
C  RSPX   REAL    2           Position of the spectrometer (RSPX,RSPZ)
C  RSPZ   REAL    2           used to find cos theta range (2)

c  pARAMEters for multi-pass event processing
C
C  NTCO   INTEG          Maximum number of tracks processed concurrently
C  NTPE   INTEG          Maximum number of tracks per event

C  CPU time limit test variable
C  TLIM   INTEG   0      if not zero time left is tested and program stops.
C                        Default is 0 as for several computers the time limit
C                        is not correctly recognized

C  Parameters determining ntuple size allocation, if SNTU is non-0:
C
C  NHTK   INTEG          Total of tracks expected to hit detector
C  NTTK   INTEG          Total of tracks expected including secondaries

C  In-line histogram and scatterplot control cards:
C  HIST   ID    TAGNAME  (short form.. default bins and interval)
C               TAGNAME can be Ntuple tag name or e.g. #35 for 35th entry
C  HIST   ID    TAGNAME   NCHANNELS   XMIN   XMAX   is the long form.
C  HIST   ID  TAGX  NX  XMI  XMA TAGY NY YMI YMA  scatterplot, args as HBOOK2
C               (The number of arguments distinguishes these three formats.)

C++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

******************************************************************************
*   PMC version number should be changed with every modification of the code
      SIMVER = 5.0
******************************************************************************

c Open the standard text output file

      call UFILE (0, 'RHASC', iusrout, FileName)
      IF(IUSROUT .lt. 1)  THEN
         CALL STOPER('UGINIT', 'RHASC FILE CAN NOT BE WRITTEN', 1)
      ENDIF


C Initialise GEANT

      CALL GINIT

      CALL PH_RESTART

**************************************
*   define magnetic field parameters *
**************************************

      CALL FFKEY('BFLD',BFLD ,1,'REAL')
      CALL FFKEY('MAGT',MAGT ,1,'INTEGER')
      CALL FFKEY('TMAG',MAGT ,1,'INTEGER')  !   alias to MAGT
      CALL FFKEY('MAGX',MAGX ,1,'REAL')
      CALL FFKEY('MAGZ',MAGZ ,1,'REAL')
      CALL FFKEY('MOFF',MOFF ,3,'REAL')
      CALL FFKEY('RFLD',RFLD ,1,'REAL')

************************************************
*   set defaults for magnetic field parameters *
************************************************

      MAGT = 2           ! default - read from magnetic field map file

      RFLD = 0.0         ! uninitialized (zero) values
      MAGX = 0.0
      MAGZ = 0.0
      MOFF(1) = 0.0      ! default offset is zero
      MOFF(2) = 0.0
      MOFF(3) = 0.0

***************************************
*  Set defaults for event generators  *
***************************************

      IRMUL = 1          ! multiplicity per event
      IRARM = 2          ! number of spectrometer arms (1 or 2)
      IRPID(1) =  0      ! particle type range:  min (also used for GEN=4)
      IRPID(2) = 99      !                       max

      RMOM(1) = 0.001    ! Production momentum:  min
      RMOM(2) = 900.0    !                       max
      RMOM(3) = 0.100    !                       step (GEN=4)
      REXP    = 0.350    ! Mean transverse momentum - falloff parameter
      RETA(1) = -20.0    ! eta range:  min
      RETA(2) =  20.0    !             max
      RANG(1) = 20.0     ! polar angle range:  min (GEN=4 only)
      RANG(2) = 60.0     !                     max
      RANG(3) =  5.0     !                     step
      RPHI(1) = -360.0   ! azimuth in mother system:  min
      RPHI(2) =  360.0   !                            max
      RROT(1) = 0.0      ! phi rotation angle (GEN=0)  initial value
      RROT(2) = 15.0     ! additional phi rotation angle after file is rewound

      RBXX = 0.0         ! Production position
      RBYY = 0.0         !    "          "
      RBZZ = 0.0         !    "          "
      RBXY = 0.0         ! Production  intersection sigma
      RBDX = 0.0         ! Production position range
      RBDY = 0.0         !    "          "      "
      RBDZ = 0.0         !    "          "      "
      RSPX = 43.0        ! position of the spectrometer center  (X)
      RSPZ = 43.0        !      "               "         "     (Z)
      RSPR = 40.0        ! radius of the circle covering the spectrometer
C                        ! used to set angle range

**************************************
* Parameters for event generators    *
**************************************

      CALL FFKEY('RMUL',IRMUL,1,'INTEGER')
      CALL FFKEY('RARM',IRARM,1,'INTEGER')
      CALL FFKEY('RPID',IRPID,2,'INTEGER')

      CALL FFKEY('RMOM',RMOM,3,'REAL')
      CALL FFKEY('REXP',REXP,1,'REAL')
      CALL FFKEY('RETA',RETA,2,'REAL')
      CALL FFKEY('RANG',RANG,3,'REAL')
      CALL FFKEY('RPHI',RPHI,3,'REAL')

      CALL FFKEY('RBXX',RBXX,1,'REAL')
      CALL FFKEY('RBYY',RBYY,1,'REAL')
      CALL FFKEY('RBZZ',RBZZ,1,'REAL')
      CALL FFKEY('RBXY',RBXY,1,'REAL')
      CALL FFKEY('RBDX',RBDX,1,'REAL')
      CALL FFKEY('RBDY',RBDY,1,'REAL')
      CALL FFKEY('RBDZ',RBDZ,1,'REAL')

      CALL FFKEY('RSPX',RSPX,1,'REAL')
      CALL FFKEY('RSPZ',RSPZ,1,'REAL')
      CALL FFKEY('RSPR',RSPR,1,'REAL')

      CALL FFKEY('RROT',RROT,2,'REAL')

************************************
*   Set defaults for user flags    *
************************************

      HITSOUT = 0  !ascii hit output flag
      ISMEM = 0    ! transfer of event to PHAT memory
      ISAOU = 0    ! ASCII output file with tracks and hits
      IAOUF = 0    ! format for ASCII output file
c                  !  (0=compact - single spaces, 1=fixed format)
      ISASC = 0    !ascii event output flag
      ISHBO = 0    !.HST plot binary output
      ISHDO = 0    !HISTDO print flag
      ISNTU = 0    !Ntuple file output flag
      IZEB = 0     !Zebra bank output flag
      ISECT = 0    !secondaries tracking
      IDRAW = 0    !control graphics output
      IGEN = 0     !event generator type
      IANAL = 1    !analysis type
      IPIXL = 1    !pixel numbers calculated
      WPIXL = 1.0  !chisq includes tracking
      WLOSS = 0.0  !chisq includes de/dx

*************************
*   Set user flags      *
*************************

      CALL FFKEY('SAHO',HITSOUT,1,'INTEGER')
      CALL FFKEY('SMEM',ISMEM,1,'INTEGER')
      CALL FFKEY('SAOU',ISAOU,1,'INTEGER')
      CALL FFKEY('AOUF',IAOUF,1,'INTEGER')
      CALL FFKEY('SASC',ISASC,1,'INTEGER')
      CALL FFKEY('SHBO',ISHBO,1,'INTEGER')
      CALL FFKEY('SHDO',ISHDO,1,'INTEGER')
      CALL FFKEY('SNTU',ISNTU,1,'INTEGER')
      CALL FFKEY('SZEB',IZEB ,1,'INTEGER')
      CALL FFKEY('SECT',ISECT,1,'INTEGER')
      CALL FFKEY('DRAW',IDRAW,1,'INTEGER')
      CALL FFKEY('GEN ',IGEN ,1,'INTEGER')
      CALL FFKEY('ANAL',IANAL,1,'INTEGER')
      CALL FFKEY('PIXL',IPIXL,1,'INTEGER')
      CALL FFKEY('WPIX',WPIXL,1,'REAL')
      CALL FFKEY('WLOS',WLOSS,1,'REAL')

****************************************************************
*   CPU time limit test:                                       *
*   TLIM 1  -  program stops when estimated time is small      *
*   TLIM 0  -  program ignores time limit, for long queues     *
*              always extremely underestimated (set to 2 hours)*
****************************************************************

      NTLIM = 0
      CALL FFKEY('TLIM', NTLIM, 1, 'INTEGER')

****************************************************************
*   Set multi-pass event processing parameters in common MUPEP *
*   Defaults set in MUPEP.CDA: NTCO = 1000, NTPE = 20000       *
****************************************************************

      CALL FFKEY('NTCO', NtConc, 1, 'INTEGER')
      CALL FFKEY('NTPE', Ntpe, 1, 'INTEGER')

      NESKIP = 0
      NTSKIP = 0
      CALL FFKEY('NESK', NESKIP, 1, 'INTEGER')
      CALL FFKEY('NTSK', NTSKIP, 1, 'INTEGER')

***********************************************************
*   Set ntuple size options in common NTUEX               *
*   Defaults set by NTUEX.CDA: NHTK = 60000, NTTK = 60000 *
***********************************************************

      CALL FFKEY('NHTK', NHtrks, 1, 'INTEGER')
      CALL FFKEY('NTTK', NMCtrks, 1, 'INTEGER')

***********************************************************
*   Call USCOM to enable the user to define extra commands
***********************************************************

      CALL USCOM

**********************************************************************
* Open the standard control input file; iusrin < 0 means open failed
**********************************************************************

      call UFILE (0, 'RHDATA', iusrin, FileName)
      IF(IUSRIN .lt. 1)  THEN
         WRITE(*,*) ' ***** ERROR IN UGINIT: ',
     +              'RHDATA (CONTROL) FILE CAN NOT OPENED'
         WRITE(*,*) '       DEFAULT SETINGS WILL BE USED'
         GO TO 100
      ENDIF


****************************************************************************
* Read in the silicon positions and in-line histogram and scatterplot      *
* specifications.  This is not done via FFKEY because of                   *
* the limitations of FFREAD; we want multiple cards with the same keyword  *
* and we have to avoid overflowing the smallish FFREAD array sizes!        *
****************************************************************************

      CALL VZERO(HCONTROL,9*NHMX)
      NHSTS = 0

   20 READ(IUSRIN,'(A)',END=25) RECORD
      IF(RECORD(1:3) .EQ. 'END') GOTO 25     !exit on this or EOF
      IF(RECORD(1:1) .EQ. '*') GOTO 20       !skip comments
      CALL STPARS(RECORD,11,NFLDS,FIELD,LENFLD)   !Melanson's parsing

C     Internal histogram keyword:
      IF(FIELD(1)(1:4).EQ.'HIST')THEN
          NHSTS = NHSTS+1
          IF(NHSTS.GT.NHMX)NHSTS = NHMX
          HCONTROL(1,NHSTS) = STTORL(FIELD(2)(1:LENFLD(2)))

          IF(FIELD(3)(1:1).EQ.'#') THEN
             HCONTROL(2,NHSTS) = STTORL(FIELD(3)(2:LENFLD(3)))
          ELSE
             ITAG = 0

C            Search for match to Ntuple tag:
             DO I = 1,NVTUP
                IF(FIELD(3).EQ.TAGS(I)) ITAG = I
             ENDDO

             HCONTROL(2,NHSTS) = FLOAT(ITAG)
          ENDIF

C         Non-default histogram parameters:
          IF(NFLDS.GT.3) THEN
             HCONTROL(3,NHSTS) = STTORL(FIELD(4)(1:LENFLD(4)))
             HCONTROL(4,NHSTS) = STTORL(FIELD(5)(1:LENFLD(5)))
             HCONTROL(5,NHSTS) = STTORL(FIELD(6)(1:LENFLD(6)))
          ELSE         !default parameters
             HCONTROL(3,NHSTS) = 50.0  !Default number of bins
             HCONTROL(4,NHSTS) = 1.0   !Do automatic range selection
             HCONTROL(5,NHSTS) = 0.0
          ENDIF

          HCONTROL(6,NHSTS) = 0.0      !Default if not scatterplot

C         Full scatterplot parameter set:
          IF(NFLDS.EQ.10)THEN
             IF(FIELD(7)(1:1).EQ.'#') THEN
                HCONTROL(6,NHSTS) = STTORL(FIELD(7)(2:LENFLD(7)))
             ELSE
                ITAG = 0

                DO I = 1,NVTUP
                   IF(FIELD(7).EQ.TAGS(I)) ITAG = I
                ENDDO

                HCONTROL(6,NHSTS) = FLOAT(ITAG)
             ENDIF

             HCONTROL(7,NHSTS) = STTORL(FIELD(8)(1:LENFLD(8)))
             HCONTROL(8,NHSTS) = STTORL(FIELD(9)(1:LENFLD(9)))
             HCONTROL(9,NHSTS) = STTORL(FIELD(10)(1:LENFLD(10)))
          ELSE
             HCONTROL(6,NHSTS) = 0.0
          ENDIF
      ENDIF
      GOTO 20

   25 REWIND IUSRIN    !ready for GFFGO

c Call GFFGO to read in control data

      CALL FFSET ('LINP', iusrin)
      CALL GFFGO

*******************************************
*  REMAINDER OF GEANT INITIALIZATION      *
*******************************************
  100 CONTINUE       ! enter here if  RHDATA  does not exist
      CALL GZINIT
      CALL GPART     ! use standard list of particles

******************************************
*  Possible modification of options      *
******************************************

      CALL PH_OPT

********************************************
*  Define detector geometry and materials  *
********************************************

      call UFILE (0, 'GCDEFS', iusrgcin, FileName)
      IF(IUSRGCIN .lt. 1)  THEN
         CALL STOPER('UGINIT', 'GCDEFS FILE CAN NOT OPENED', 1)
      ENDIF

      call UGCIN (iusrgcin, lurep, lstor, lline, storage, iugcres)
      if (jdebug .gt. 1)
     *  write (lurep,'(a,i3)') ' UGINIT: UGCIN returned iugcres =',
     *  iugcres

c If the geometry file ends with UGC END i, UGCIN will return iugcres=i;
c this was formerly used to call UGEOM with default geometry.

      if (iugcres .ne. 0) write(IUSROUT,*)'UGEOM removed 9/16/95'

c Make diagnostic printout of all components of the detector

      if (jdebug .gt. 1) call GPRINT ('VOLU', 0)

**********************
* Miscellaneous      *
**********************

      IF(NDETS .NE. NSLP)  THEN
         WRITE(IUSROUT,*) ' *** WARNING IN UGINIT:',
     +    '  NUMBER OF DETECTORS FOR WHICH PADS DATA ARE',
     +    ' PROVIDED', NDETS
         WRITE(IUSROUT,*) '     DIFFERENT THAN THE DECLARED',
     +    ' NUMBER OF DETECTORS (NPLANES =', NSLP, ' )'
         NSLP = NDETS
      ENDIF

      ISCAT = IMULS + 3*ILOSS + 9*IPIXL  !track scattering type
      ISEQ = 0    !Used for IGEN = 4
      IF(IRMUL.GT.MXMUL) THEN
         IRMUL = MXMUL
         WRITE(IUSROUT,*)' MULTIPLICITY CANNOT BE > ',IRMUL
      ENDIF

      ITRIG = NEVENT   !  store the number of events declared by TRIG

C             Define user histograms
      CALL USINIT

C             Open ZEBRA output file
      IF(IZEB.NE.0) THEN
         CALL UFILE (0, 'RHZEB', IUSRZEB, FileName)
         IF(IUSRZEB .lt. 1)  THEN
            CALL STOPER('UGINIT', 'ZEBRA FILE CAN NOT BE WRITTEN', 1)
         ENDIF

         CALL GOPEN(IUSRZEB,'O',2440,IER)
         IF(IER.NE.0) WRITE(IUSROUT,*)' IN UGINIT, GOPEN IER= ',IER
      ENDIF

C              Energy loss and cross-sections initialisations
      CALL GPHYSI

C             output header lines in ASCII output
      IF(ISASC.NE.0) THEN
         CALL DATIMH(HDATE, HTIME)
         WRITE(IUSROUT,1000) HDATE, HTIME
 1000    FORMAT(' GPH DETECTOR VERSION 2 ',2A4,', ',2A4)
      ENDIF

      IF(ISAOU .NE. 0)  THEN   ! open AOU file and write the header
         call UFILE (0, 'RHAOU', iusraou, FileName)
         IF(IUSRAOU .lt. 1)  THEN
            CALL STOPER('UGINIT', 'AOU FILE CAN NOT BE WRITTEN', 1)
         ENDIF

         WRITE(IUSRAOU, '('' * PMC simulations ASCII OUTPUT file'')' )
         CALL DATIMH(HDATE, HTIME)
         WRITE(IUSRAOU, '('' S DATE '', 2A4)' ) HDATE
         WRITE(IUSRAOU, '('' S TIME '', 2A4)' ) HTIME
         WRITE(IUSRAOU, '('' S PMCV '', F15.3)' ) SIMVER
         WRITE(IUSRAOU, '('' S GEAV '', F15.3)' ) GVERSN
         WRITE(IUSRAOU, '('' S ZEBV '', F15.3)' ) ZVERSN

         WRITE(IUSRAOU, '('' * '')' )
         WRITE(IUSRAOU, '('' * Run number, random numbers '')' )
         WRITE(IUSRAOU, '('' S RUNG '', I10)' ) IDRUN
         WRITE(IUSRAOU, '('' S RNDM '', 2I10)' ) NRNDM(1), NRNDM(2)
         APOSTR = ''''
         IF(NSAVE .GT. 0)   WRITE(IUSRAOU, 7700) 'SAVE',
     +                       (APOSTR,LSAVE(I),APOSTR, I=1,NSAVE)
         IF(NSETS .GT. 0)   WRITE(IUSRAOU, 7700) 'SETS',
     +                       (APOSTR,LSETS(I),APOSTR, I=1,NSETS)
 7700    FORMAT(' S ', A4, 16(1X, A1, A4, A1) )
         WRITE(IUSRAOU, '('' * '')' )
         WRITE(IUSRAOU, '('' * Simulation parameters'')' )
         WRITE(IUSRAOU, '('' S SECT '', I10)' ) ISECT
         WRITE(IUSRAOU, '('' S RPID '', 2I10)' ) IRPID
         WRITE(IUSRAOU, '('' S RMOM '', 3F10.4)' ) RMOM
         WRITE(IUSRAOU, '('' S RETA '', 2F10.4)' ) RETA
         WRITE(IUSRAOU, '('' S RPHI '', 3F10.4)' ) RPHI
         WRITE(IUSRAOU, '('' S RANG '', 3F10.4)' ) RANG
         WRITE(IUSRAOU, '('' S RARM '', I10)' ) IRARM
         WRITE(IUSRAOU, '('' S RMUL '', I10)' ) IRMUL
         WRITE(IUSRAOU, '('' S REXP '', F10.4)' ) REXP
         WRITE(IUSRAOU, '('' S RSPX '', F10.4)' ) RSPX
         WRITE(IUSRAOU, '('' S RSPZ '', F10.4)' ) RSPZ
         WRITE(IUSRAOU, '('' S RSPR '', F10.4)' ) RSPR
         WRITE(IUSRAOU, '('' S RROT '', 2F10.4)' ) RROT
         WRITE(IUSRAOU, '('' * Vertex'')' )
         WRITE(IUSRAOU, '('' S RBXX '', F10.4)' ) RBXX
         WRITE(IUSRAOU, '('' S RBYY '', F10.4)' ) RBYY
         WRITE(IUSRAOU, '('' S RBZZ '', F10.4)' ) RBZZ
         WRITE(IUSRAOU, '('' S RBDX '', F10.4)' ) RBDX
         WRITE(IUSRAOU, '('' S RBDY '', F10.4)' ) RBDY
         WRITE(IUSRAOU, '('' S RBDZ '', F10.4)' ) RBDZ
         WRITE(IUSRAOU, '('' S RBXY '', F10.4)' ) RBXY
         WRITE(IUSRAOU, '('' * Geant parameters'')' )
         WRITE(IUSRAOU, '('' S ANNI '', I10)' ) IANNI
         WRITE(IUSRAOU, '('' S BREM '', I10)' ) IBREM
         WRITE(IUSRAOU, '('' S COMP '', I10)' ) ICOMP
         WRITE(IUSRAOU, '('' S DCAY '', I10)' ) IDCAY
         WRITE(IUSRAOU, '('' S DRAY '', I10)' ) IDRAY
         WRITE(IUSRAOU, '('' S HADR '', I10)' ) IHADR
         WRITE(IUSRAOU, '('' S MULS '', I10)' ) IMULS
         WRITE(IUSRAOU, '('' S LOSS '', I10)' ) ILOSS
         WRITE(IUSRAOU, '('' S PAIR '', I10)' ) IPAIR
         WRITE(IUSRAOU, '('' S PFIS '', I10)' ) IPFIS
         WRITE(IUSRAOU, '('' S PHOT '', I10)' ) IPHOT
         WRITE(IUSRAOU, '('' S RAYL '', I10)' ) IRAYL
         WRITE(IUSRAOU, '('' S CUTS '', 5F10.6)' )
     +      CUTGAM, CUTELE, CUTNEU, CUTHAD, CUTMUO
         WRITE(IUSRAOU, '('' * Run controls '')' )
         WRITE(IUSRAOU, '('' S ANAL '', I10)' ) IANAL
         WRITE(IUSRAOU, '('' S PIXL '', I10)' ) IPIXL
         WRITE(IUSRAOU, '('' S WPIX '', F10.4)' ) WPIXL
         WRITE(IUSRAOU, '('' S WLOS '', F10.4)' ) WLOSS
         WRITE(IUSRAOU, '('' S SAHO '', I10)' ) HITSOUT
         WRITE(IUSRAOU, '('' S SAOU '', I10)' ) ISAOU
         WRITE(IUSRAOU, '('' S AOUF '', I10)' ) IAOUF
         WRITE(IUSRAOU, '('' S SASC '', I10)' ) ISASC
         WRITE(IUSRAOU, '('' S SHBO '', I10)' ) ISHBO
         WRITE(IUSRAOU, '('' S SHDO '', I10)' ) ISHDO
         WRITE(IUSRAOU, '('' S SNTU '', I10)' ) ISNTU
         WRITE(IUSRAOU, '('' S SZEB '', I10)' ) IZEB
         WRITE(IUSRAOU, '('' S NHTK '', I10)' ) NHTRKS
         WRITE(IUSRAOU, '('' S NTTK '', I10)' ) NMCTRKS
         WRITE(IUSRAOU, '('' * Generation mode'')' )
         WRITE(IUSRAOU, '('' S GEN  '', I10)' ) IGEN
         WRITE(IUSRAOU, '('' S NTCO '', I10)' ) NTCONC
         WRITE(IUSRAOU, '('' S NTPE '', I10)' ) NTPE
         WRITE(IUSRAOU, '('' S NESK '', I10)' ) NESKIP
         WRITE(IUSRAOU, '('' S NTSK '', I10)' ) NTSKIP

      ENDIF

      IF(ISMEM .NE. 0)  THEN   ! transfer information to Phat

         CALL DATIMH(HDATE, HTIME)
         WRITE(LINE, '('' S DATE '', 2A4)' ) HDATE
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S TIME '', 2A4)' ) HTIME
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S PMCV '', F15.3)' ) SIMVER
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S GEAV '', F15.3)' ) GVERSN
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S ZEBV '', F15.3)' ) ZVERSN
          CALL ADDPMCRINFO(LINE)

         WRITE(LINE, '('' S RUNG '', I10)' ) IDRUN
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S RNDM '', 2I10)' ) NRNDM(1), NRNDM(2)
          CALL ADDPMCRINFO(LINE)

         APOSTR = ''''
         IF(NSAVE .GT. 0  .AND.  NSAVE .LT. 17)  THEN
            WRITE(LINE, 7700) 'SAVE',
     +                       (APOSTR,LSAVE(I),APOSTR, I=1,NSAVE)
            CALL ADDPMCRINFO(LINE)
         ELSE
            WRITE(LINE, 7700) 'SAVE',
     +                       (APOSTR,LSAVE(I),APOSTR, I=1,16)
            CALL ADDPMCRINFO(LINE)
            WRITE(LINE, 7700) 'SAVE',
     +                       (APOSTR,LSAVE(I),APOSTR, I=17, NSAVE)
            CALL ADDPMCRINFO(LINE)
         ENDIF

         IF(NSETS .GT. 0  .AND.  NSETS .LT. 17)  THEN
            WRITE(LINE, 7700) 'SETS',
     +                       (APOSTR,LSETS(I),APOSTR, I=1,NSETS)
            CALL ADDPMCRINFO(LINE)
         ELSE
            WRITE(LINE, 7700) 'SETS',
     +                       (APOSTR,LSETS(I),APOSTR, I=1,16)
            CALL ADDPMCRINFO(LINE)
            WRITE(LINE, 7700) 'SAVE',
     +                       (APOSTR,LSETS(I),APOSTR, I=17,NSETS)
            CALL ADDPMCRINFO(LINE)
         ENDIF

         WRITE(LINE, '('' S SECT '', I10)' ) ISECT
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S RPID '', 2I10)' ) IRPID
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S RMOM '', 3F10.4)' ) RMOM
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S RETA '', 2F10.4)' ) RETA
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S RPHI '', 3F10.4)' ) RPHI
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S RANG '', 3F10.4)' ) RANG
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S RARM '', I10)' ) IRARM
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S RMUL '', I10)' ) IRMUL
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S REXP '', F10.4)' ) REXP
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S RSPX '', F10.4)' ) RSPX
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S RSPZ '', F10.4)' ) RSPZ
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S RSPR '', F10.4)' ) RSPR
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S RROT '', 2F10.4)' ) RROT
          CALL ADDPMCRINFO(LINE)

         WRITE(LINE, '('' S RBXX '', F10.4)' ) RBXX
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S RBYY '', F10.4)' ) RBYY
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S RBZZ '', F10.4)' ) RBZZ
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S RBDX '', F10.4)' ) RBDX
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S RBDY '', F10.4)' ) RBDY
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S RBDZ '', F10.4)' ) RBDZ
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S RBXY '', F10.4)' ) RBXY
          CALL ADDPMCRINFO(LINE)

         WRITE(LINE, '('' S ANNI '', I10)' ) IANNI
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S BREM '', I10)' ) IBREM
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S COMP '', I10)' ) ICOMP
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S DCAY '', I10)' ) IDCAY
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S DRAY '', I10)' ) IDRAY
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S HADR '', I10)' ) IHADR
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S MULS '', I10)' ) IMULS
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S LOSS '', I10)' ) ILOSS
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S PAIR '', I10)' ) IPAIR
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S PFIS '', I10)' ) IPFIS
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S PHOT '', I10)' ) IPHOT
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S RAYL '', I10)' ) IRAYL
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S CUTS '', 5F10.6)' )
     +      CUTGAM, CUTELE, CUTNEU, CUTHAD, CUTMUO
          CALL ADDPMCRINFO(LINE)

         WRITE(LINE, '('' S ANAL '', I10)' ) IANAL
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S PIXL '', I10)' ) IPIXL
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S WPIX '', F10.4)' ) WPIXL
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S WLOS '', F10.4)' ) WLOSS
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S SAHO '', I10)' ) HITSOUT
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S SAOU '', I10)' ) ISAOU
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S AOUF '', I10)' ) IAOUF
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S SASC '', I10)' ) ISASC
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S SHBO '', I10)' ) ISHBO
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S SHDO '', I10)' ) ISHDO
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S SNTU '', I10)' ) ISNTU
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S SZEB '', I10)' ) IZEB
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S NHTK '', I10)' ) NHTRKS
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S NTTK '', I10)' ) NMCTRKS
          CALL ADDPMCRINFO(LINE)

         WRITE(LINE, '('' S GEN  '', I10)' ) IGEN
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S NTCO '', I10)' ) NTCONC
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S NTPE '', I10)' ) NTPE
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S NESK '', I10)' ) NESKIP
          CALL ADDPMCRINFO(LINE)
         WRITE(LINE, '('' S NTSK '', I10)' ) NTSKIP
          CALL ADDPMCRINFO(LINE)

      ENDIF  !  IF(ISMEM .NE. 0)

      CALL PH_GEOM

      IF(ISMEM .GT. 0  .OR.  ISAOU .GT. 0)  THEN
         XDUM(1) = 0.0
         XDUM(2) = 0.0
         XDUM(3) = 0.0
         CALL GUFLD(XDUM, BDUM)  !  initialize GUFLD before EOH is written
         IF(ISAOU .GT. 0)  WRITE(IUSRAOU, '('' EOH '')' )
         IF(ISMEM .GT. 0)  CALL PMCEOH
      ENDIF

      RETURN
      END
