#ifndef _TPhHough_
#define _TPhHough_
//
// TPhHough.h
//
// 19990118.0135 Inkyu.Park@bnl.gov
//
//
#include "TNamed.h"
#include "TObjArray.h"
#include "TH2.h"

#define kNumLayers 16
#define MAXPLANES 16
#define A_BINS  200
#define C_BINS  400

class TPhTrack;
class TPhMCTrack;
class TPhRecTrack;
class TPhHit;

const int kNPair = 4;
const int k0Pair[kNPair] = { 8, 9,10,12};
const int k1Pair[kNPair] = { 9,10,12,14};

class TPhHough : public TNamed {

private:

  unsigned int fDebug;					//! Control debugging output
  double fHoughQP[kNPair][A_BINS][C_BINS];		// fHoughQP
  double fHoughTH[kNPair][A_BINS][C_BINS];		// fHoughTH
  unsigned int fHoughNB[kNPair][A_BINS][C_BINS];	// fHoughNB
  TH2F *fHistQP[kNPair];				//! histogram
  TH2F *fHistTH[kNPair];				//! hist
  
public:
  
                  TPhHough();
  virtual        ~TPhHough();
  virtual void    FillTable(TPhMCTrack *t);
  virtual void    Normalize();
  virtual void    SmoothTable();
  virtual void    FillHistograms();
  virtual void    WriteTable();
  virtual void    ReadTable(int, int);

  virtual void    SetDebug(int i){fDebug=i;}
  virtual void    SetTH(int i,int ia,int ic,double th){fHoughTH[i][ia][ic]=th;}
  virtual void    SetQP(int i,int ia,int ic,double qp){fHoughQP[i][ia][ic]=qp;}
  virtual void    SetNB(int i,int ia,int ic,unsigned int nb){fHoughNB[i][ia][ic]=nb;}
  
  virtual Bool_t  GetAngleCurve(double,double,double,double,double*,double*);
  virtual Bool_t  AngleCurveToBin(int,double,double,int*,int*);
  virtual Bool_t  BinToAngleCurve(int,int,int,double*,double*);

//virtual void    Transform(int,double,double,double,double,double*,double*);
  virtual void    Transform(int,int,double,double,double,double,double*,double*);

  virtual double       GetQP(int i,int ia,int ic){return fHoughQP[i][ia][ic];}
  virtual double       GetTH(int i,int ia,int ic){return fHoughTH[i][ia][ic];}
  virtual unsigned int GetNB(int i,int ia,int ic){return fHoughNB[i][ia][ic];}
//virtual double *GetQP(){return &fHoughQP[0][0][0];}
//virtual double *GetTH(){return &fHoughTH[0][0][0];}
  virtual TH2F   *GetHistQP(int i){return fHistQP[i];}
  virtual TH2F   *GetHistTH(int i){return fHistTH[i];}
  
  ClassDef(TPhHough,1) // Finds PHOBOS track seeds            
};
#endif
