#ifndef _PhTemplateManager_INCLUDED_
#define _PhTemplateManager_INCLUDED_

///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhTemplateManager                                             //
//                                                                   //
//    PHOBOS detector simulation class                               //
//                                                                   //
//    TPhTemplateManager provides access to pre-generated track      //
//    templates                                                      //
//                                                                   //
//    Author  : Gunther Roland (rolandg@vxcern.cern.ch)              //
//    Created : 15/09/96                                             //
//    Version : 1.0                                                  //
//    Changed :                                                      //
//                                                                   //
///////////////////////////////////////////////////////////////////////

// ROOT classes

#include "TObject.h"
#include "TObjArray.h"

// Phobos classes


class TPhHit;
class TPhEvent;
class TPhTrack;
class TPhRecTrack;
class TPhMagnet;
class TPhSensor;
class TPhStuff;
class TPhStraightHough;
class TPhDetector;

class TPhTrackTemplate;

const Int_t kMaxLayers = 16;
const Int_t kMaxHits   = 10;

class TPhTemplateManager : public TObject {

public:
    TPhTemplateManager();

    virtual ~TPhTemplateManager();

    virtual TPhDetector      *GetDetector()     { return fDetector; }
    virtual TPhMagnet        *GetMagnet()       { return fMagnet;   }
    virtual TObjArray        *GetTemplateList() { return fTemplateList; }
    virtual TObjArray        *GetCovarList()    { return fCovarList; }
    
    virtual TPhTrackTemplate *GetTemplate(Float_t p, Float_t theta, Float_t dip);

    virtual Bool_t            ReadTemplateList(Char_t *TemplName);
    virtual Bool_t            ReadTemplateList(Char_t *TemplName, Char_t *CovarName);

    virtual void              SetMaxCompDist(Float_t x)         { fMaxCompDist   = x; }
    virtual void              SetMinCompEnergy(Float_t x)       { fMinCompEnergy = x; }

    virtual void              SetVerbose(Int_t i)               { fVerbose  = i;      }
    virtual void              SetMagnet(TPhMagnet *mag)         { fMagnet   = mag;    }
    virtual void              SetHough(TPhStraightHough *hough) { fHough    = hough;  }
    virtual Int_t             GetCompatibleHits(TPhTrackTemplate *t);
    virtual Int_t             GetCompatibleHits(TPhTrackTemplate *t, Double_t MaxDist);
    virtual void              CreateTrackCandidates();
    virtual Float_t           CreateTrackCandidates(Float_t CPUlimit);
    virtual Float_t           CreateTrackCandidates(Float_t CPUlimit, Double_t MaxDist);

    virtual void              ResetCandidateLoop();
    virtual TPhRecTrack      *GetNextCandidate();
    virtual TObjArray        *GetCandidateList()            { return fCandidateList; }

    virtual void              Reset();

private:

    Int_t             fMinHits;            //  Minimum number of hits for track candidates
    Float_t           fMinProb;            //  Minimum chi^2 probability for track candidates
    Float_t           fMaxCompDist;        //  For selection of hits compatible with template
    Float_t           fMinCompEnergy;      //  For selection of hits compatible with template
    TPhStuff         *fStuff;              //! Routines for chi^2 probabilities etc
    TPhMagnet        *fMagnet;             //! PHOBOS Magnet Object
    TPhDetector      *fDetector;           //! PHOBOS Detector Object
    TPhTrackTemplate *fCurrentTemplate;    //! Current track template
    TPhStraightHough *fHough;              //! Hough transform object for template pre-selection
    TObjArray        *fTemplateList;       //! List of track templates
    TObjArray        *fCovarList;          //! List of covariance matrixes (stored in templates)
    TObjArray        *fCandidateList;      //! List of track candidates
    TPhHit           *fHitArray[kMaxLayers][kMaxHits]; //! Array of hits compatible with template
    Int_t             fHitCnt[kMaxLayers]; //! Number of hits compatible with template per layer
    Int_t             fCurrentLayer;       //! For loop over hit combinations
    Int_t             fCurrentIndex[kMaxLayers]; //! For loop over hit combinations
    Bool_t            fVerbose;            //! Control debugging output

public:

    ClassDef(TPhTemplateManager,1) // Manages the pre-generated templates.

};

#endif



