#ifndef _TPhTrackSeedFinder_
#define _TPhTrackSeedFinder_
//
// ====================
// TPhTrackSeedFinder.h
// ====================
//
// 19990118.0135 Inkyu.Park@bnl.gov
//
// This routine finds seeds of PHOBOS spectrometer tracks.
//
//
//
//
#include "TObject.h"
#include "TObjArray.h"

class TPhEvent;
class TPhTrack;
class TPhHit;
class TPhMagnet;
class TPhSensor;
class TPhDetector;
class TPhRecHit;
class TPhMerging;
class TPhMulguisin;

const int kTSFNLayers=16;

class TPhTrackSeedFinder : public TObject {

private:

  Bool_t       fVerbose;            	//! Control debugging output

  int          fMergeHit;		//! Merge 
  int          fMulguisinMode;	    	//! Mulguisin mode
  float        fMulguisinResol;	    	//! Mulguisin resolution
  int          fMulguisinLayer;     	//! last layer for Mulguisin run

  TPhDetector *fDetector;		//! PHOBOS detector
  TPhMerging  *fMerge;			//! 
  TObjArray   *fHitA[kTSFNLayers];  	//! hit array
  
  TObjArray   *fTrackSeedList;	    	//! track seed list
  TObjArray   *fCompatibleSeedList; 	//! compatible track seed
  
public:
  
                     TPhTrackSeedFinder();
  virtual           ~TPhTrackSeedFinder();
  
  virtual void       SetVerbose(int i){fVerbose=i;}
 
  virtual void       SetMergeHit(int i){fMergeHit=i;} 
  virtual void       SetMulguisinMode(int i){fMulguisinMode=i;}
  virtual void       SetMulguisinResol(float x){fMulguisinResol=x;}
  virtual void       SetMulguisinLayer(int i){fMulguisinLayer=i;}

  virtual void       FillHit();
  
  virtual void       RunMulguisin(){RunMulguisin(fMulguisinMode,fMulguisinResol);}
  virtual void       RunMulguisin(int mode, float resol);

  virtual TObjArray *GetTrackSeedList(){return fTrackSeedList;}
  virtual TObjArray *GetCompatibleSeedList(TPhMomentumVector *v);
 
  virtual void       Reset(){fTrackSeedList->Delete();
                               fCompatibleSeedList->Delete();} 

  float              GetAngularDist(TPhTrack *t1,TPhTrack *t2);
  			       
  ClassDef(TPhTrackSeedFinder,0) // Finds PHOBOS track seeds            

};

#endif
