#ifndef _PhTrackTemplFinder_INCLUDED_
#define _PhTrackTemplFinder_INCLUDED_

///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhTrackTemplFinder                                            //
//                                                                   //
//    PHOBOS detector simulation class                               //
//                                                                   //
//    TPhTrackTemplFinder provides access to pre-generated track     //
//    templates - TPhMCTrackTemplate class type                      //
//                                                                   //
//    Author  : as TPhTemplateManager class: Gunther Roland          //
//              (revised by K.Wozniak)                               //
//    Created : 15/09/96                                             //
//    Version : 1.0                                                  //
//    Changed :                                                      //
//                                                                   //
///////////////////////////////////////////////////////////////////////

// ROOT classes

#include "TObject.h"
#include "TObjArray.h"

// Phobos classes


class TPhHit;
class TPhEvent;
class TPhTrack;
class TPhRecTrack;
class TPhMerging;
class TPhSensor;
class TPhStraightHough;
class TPhDetector;
class TPhTrackSeedFinder;

class TPhMCTrackTemplate;

const Int_t kMaxLayerNo = 16;
const Int_t kMaxCompatHits   = 10;

class TPhTrackTemplFinder : public TObject {

public:
    TPhTrackTemplFinder();

    virtual ~TPhTrackTemplFinder();

    virtual TPhDetector      *GetDetector()     { return fDetector; }
    virtual TObjArray        *GetTemplateList() { return fTemplateList; }
    virtual TObjArray        *GetCovarList()    { return fCovarList; }

    virtual TPhMCTrackTemplate *GetTemplate(Float_t ptot, Float_t theta, Float_t dip);
    virtual TPhMCTrackTemplate *GetTemplatePxPz(Float_t px, Float_t pz);

    virtual Bool_t            ReadTemplateList(Char_t *TemplName);
    virtual Bool_t            ReadTemplateList(Char_t *TemplName, Char_t *CovarName);
    virtual void              AddTemplate(TPhMCTrackTemplate *t);

    virtual void              SetMaxCompDist(Float_t x)         { fMaxCompDist   = x; }
    virtual void              SetMinCompEnergy(Float_t x)       { fMinCompEnergy = x; }

    virtual void              SetVerbose(Int_t i)               { fVerbose  = i;      }
    virtual void              SetMerging(TPhMerging *merg)      { fMerging = merg;    }
    virtual void              SetHough(TPhStraightHough *hough) { fHough    = hough;  }
    virtual void              SetTrackSeed(TPhTrackSeedFinder *tsf) { fSeed = tsf;    }
    virtual Int_t             GetCompatibleHits(TPhMCTrackTemplate *t);
    virtual Int_t             GetCompatibleHits(TPhMCTrackTemplate *t, Double_t MaxDist);
    virtual Int_t             GetCompatibleHits(TPhMCTrackTemplate *t, Double_t MaxDist, Double_t SlopeY, Int_t SlopeYLayer);

    virtual void              CreateTrackCandidates();
    virtual Float_t           CreateTrackCandidates(Float_t CPUlimit);
    virtual Float_t           CreateTrackCandidates(Float_t CPUlimit, Double_t MaxDist);

    virtual void              ResetCandidateLoop();
    virtual TPhRecTrack      *GetNextCandidate();
    virtual TObjArray        *GetCandidateList()            { return fCandidateList; }

    virtual void              Reset();

private:

    Int_t             fMinHits;            //  Minimum number of hits for track candidates
    Float_t           fMinProb;            //  Minimum chi^2 probability for track candidates
    Float_t           fMaxCompDist;        //  For selection of hits compatible with template
    Float_t           fMinCompEnergy;      //  For selection of hits compatible with template
    TPhMerging       *fMerging;            //! Management for merged hits
    TPhDetector      *fDetector;           //! PHOBOS Detector Object
    TPhMCTrackTemplate *fCurrentTemplate;  //! Current track template
    TPhStraightHough *fHough;              //! Hough transform object for template pre-selection
    TPhTrackSeedFinder  *fSeed;            //! Track seed objest for template preselection
    TObjArray        *fTemplateList;       //! List of track templates
    TObjArray        *fCovarList;          //! List of covariance matrixes (stored in templates)
    TObjArray        *fCandidateList;      //! List of track candidates
    TPhHit           *fHitArray[kMaxLayerNo][kMaxCompatHits]; //! Array of hits compatible with template
    Int_t             fHitCnt[kMaxLayerNo]; //! Number of hits compatible with template per layer
    Int_t             fCurrentLayer;       //! For loop over hit combinations
    Int_t             fCurrentIndex[kMaxLayerNo]; //! For loop over hit combinations
    Bool_t            fVerbose;            //! Control debugging output

public:

    ClassDef(TPhTrackTemplFinder,1) // Manages the pre-generated templates.

};

#endif



