#include <math.h>
#include "TPhQPair.h"
#include "TPhTrack.h"

#include "PhysConst.h"

#define pairDEBUG

ClassImp(TPhQPair)
//////////////////////////////////////////////////////////////////////
//
// $Id: TPhQPair.cxx,v 1.2 1999/03/12 07:21:04 decowski Exp $
//
//////////////////////////////////////////////////////////////////////

const Double_t QGamov = 0.025;

TPhQPair::TPhQPair(TPhTrack *t1, TPhTrack *t2,Bool_t WeightFlag)
{
    Calc(t1,t2,WeightFlag);
}

void TPhQPair::Calc(TPhTrack *t1, TPhTrack *t2,Double_t max,Bool_t WeightFlag)
{
    Double_t eta;

    Double_t dpx, dpz, dpy, dE;
    Double_t px1, py1, pz1, E1;
    Double_t px2, py2, pz2, E2;
    Double_t c, k1, k2, k3, k4, qlimit;
    Double_t beta;
    Double_t gamma1;

    px1 = (*t1).GetPz();
    py1 = (*t1).GetPy();
    pz1 = (*t1).GetPx();
    px2 = (*t2).GetPz();
    py2 = (*t2).GetPy();
    pz2 = (*t2).GetPx();

    dpx = px1 - px2;
    dpy = py1 - py2;
    dpz = pz1 - pz2;

    gamma1 = cosh(0.5 * (t1->GetRap(kPIMASS) + t2->GetRap(kPIMASS)));
    beta = sqrt(1-1/(gamma1*gamma1));

    E1 = sqrt(px1*px1 + py1*py1 + pz1*pz1 + kPIMASS2);
    px1 = gamma1 * (px1 - beta * E1);
    E1 = sqrt(px1*px1 + py1*py1 + pz1*pz1 + kPIMASS2);

    E2 = sqrt(px2*px2 + py2*py2 + pz2*pz2 + kPIMASS2);
    px2 = gamma1 * (px2 - beta * E2);
    E2 = sqrt(px2*px2 + py2*py2 + pz2*pz2 + kPIMASS2);

    dE  = E1 - E2;
    fQ0 = fabs(dE);
    dpx = px1-px2;

    qlimit = dpx*dpx - dE*dE;
    fQ     = sqrt(dpx*dpx+dpy*dpy+dpz*dpz);
    fQinv  = sqrt(fabs(dpy*dpy + dpz*dpz + qlimit));
    fQlong = fabs(px1-px2);
    fQt    = sqrt(dpy*dpy + dpz*dpz);

    k1 = (pz1+pz2)*(pz1+pz2) + (py1+py2)*(py1+py2);
    k2 = dpz*(pz1+pz2) + dpy*(py1+py2);
    c  = k2/k1;
    fQout = fabs(c)*sqrt(k1);
    k3 = dpz - c*(pz1+pz2);
    k4 = c*(py1+py2) - dpy;
    fQside = sqrt(k3*k3 + k4*k4);

    fDpx = dpx;
    fDpy = dpy;
    fDpz = dpz;

    fQlcmp = fQlong;

    eta = 0.00640243/fQinv;
    fGamov = eta/(exp(eta)-1.);
    fModGamov = (fGamov - 1) * exp(-fQinv/QGamov) + 1;

}


Double_t TPhQPair::CalcQinvFast(TPhTrack *t1, TPhTrack *t2,Double_t MaxDpz)
{
    Double_t dpx, dpz, dpy, dE, E1, E2;

    if((dpz = t1->GetPz() - t2->GetPz()) < MaxDpz)
    {
        dpx = t1->GetPx() - t2->GetPx();
        dpy = t1->GetPy() - t2->GetPy();
    
        E1 = sqrt(t1->GetPx()*t1->GetPx() + t1->GetPy()*t1->GetPy() 
		  + t1->GetPz()*t1->GetPz() + kPIMASS2);
        E2 = sqrt(t2->GetPx()*t2->GetPx() + t2->GetPy()*t2->GetPy() 
		  + t2->GetPz()*t2->GetPz() + kPIMASS2);
    
        dE  = E1 - E2;
        fQinv  = sqrt(fabs(dpx*dpx + dpy*dpy + dpz*dpz - dE*dE));
     }
     else
     {
        fQinv = MaxDpz;
     }
     return fQinv;
}

