#include "math.h"
#include "TPhTrackSelector.h"
#include "TPhHit.h"
#include "TPhTrack.h"
#include "TPhRecTrack.h"

ClassImp(TPhTrackSelector)

TPhTrackSelector::TPhTrackSelector()
{

    fCandidateList  = NULL; 
    fTrackList      = new TObjArray();

    fMinHits       = 11;
    fMinProb       = 0.01;
    fMaxSharedHits = 2;  
}

void TPhTrackSelector::SelectTracks(TObjArray *list)
{

    Int_t max;
    Int_t i;
    Int_t j;

    TPhRecTrack *track1;
    TPhRecTrack *track2;


    fCandidateList = list;
    max = fCandidateList->GetEntries();
    if(fVerbose)
        printf("Found %d track candidates\n",max);
    for(i=0;i<max-1;i++)
    {
        track1 = (TPhRecTrack *) fCandidateList->At(i);
        for(j=i+1;j<max;j++)
        {
            track2 = (TPhRecTrack *) fCandidateList->At(j);
            if(track1->GetStatus() && track2->GetStatus())
            {
                if(DoCompete(track1,track2))
                {
                    ChooseTrack(track1,track2);
                }
            }
        }
    }
    if(fVerbose)
        printf("Create selected track list\n");
    if(!fTrackList)
        fTrackList = new TObjArray;
    else
        fTrackList->Clear();
    for(i=0;i<max;i++)
    {
        track1 = (TPhRecTrack *) fCandidateList->At(i);
        if(track1->GetStatus())
             fTrackList->Add(track1);
    }
}

Bool_t TPhTrackSelector::DoCompete(TPhRecTrack *t1, TPhRecTrack *t2)
{

     Int_t i;
     Int_t cnt = 0;
     void *hits1[20];
     void *hits2[20];

     TPhHit *hit;

     TIter NextHit1(t1->GetHitList());
     while(hit = (TPhHit *) NextHit1())
     {
         hits1[hit->GetLayer()] = hit;
     }

     TIter NextHit2(t2->GetHitList());
     while(hit = (TPhHit *) NextHit2())
     {
         hits2[hit->GetLayer()] = hit;
     }
     for(i=0;i<20;i++)
     {
         if(hits1[i]==hits2[i])
         {
             if(hits1[i] != NULL)
             {
                 cnt++;
             }
         }
     }
     if(fVerbose)
         printf("track %g %g %g and %g %g %g share %d hits\n",
                    t1->GetPx(),
                    t1->GetPy(),
                    t1->GetPz(),
                    t2->GetPx(),
                    t2->GetPy(),
                    t2->GetPz(),
                    cnt);
     if(cnt > fMaxSharedHits)
        return kTRUE;
     else
        return kFALSE;
}

void TPhTrackSelector::ChooseTrack(TPhRecTrack *t1, TPhRecTrack *t2)
{
    if(t1->GetProb() > t2->GetProb())
    {
        t1->SetStatus(kTRUE);
        t2->SetStatus(kFALSE);
    }
    else
    {
        t1->SetStatus(kFALSE);
        t2->SetStatus(kTRUE);
    }
}
