#ifndef _PhSwim_INCLUDED_
#define _PhSwim_INCLUDED_

///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhSwim                                                        //
//                                                                   //
//    PHOBOS detector simulation class                               //
//                                                                   //
//    TPhSwim accepts a PHOBOS event with track-momentum information //
//    as input and propagates the tracks through the detector,       //
//    creating the hit information and storing it in the event       //
//                                                                   //
//    Author  : Gunther Roland (rolandg@vxcern.cern.ch)              //
//    Created : 15/09/96                                             //
//    Version : 1.0                                                  //
//    Changed :                                                      //
//                                                                   //
///////////////////////////////////////////////////////////////////////

// ROOT classes

#include "TH1.h"
#include "TObject.h"
#include "TObjArray.h"

// Phobos classes

class TPhHit;
class TPhMCHit;
class TPhEvent;
class TPhTrack;
class TPhMCTrack;
class TPhMagnet;
class TPhSensor;
class TPhDetector;


#define PIXNUM    2000
#define CPIXNUM   200
#define COARSEFAK 10
#define MAXX      75
#define MINX      0

const Bool_t kCoarse = kFALSE;
const Bool_t kFine   = kTRUE;  

class TPhSwim : public TObject {

public:
    TPhSwim();

    virtual ~TPhSwim() {};


    virtual TPhDetector  *GetDetector() { return fDetector; }
    virtual TPhMagnet    *GetMagnet()   { return fMagnet;   }
    virtual Double_t      GetLandau();               
    virtual Double_t      GetRelRise(Double_t p, Double_t mass);               
    virtual Bool_t        GetMS()       { return fDoMS;     }
    virtual Bool_t        GetPixelize() { return fPixelize; }
    virtual Int_t         GetVerbose()  { return fVerbose;  }
    
    virtual void          SetMS(Bool_t b)           { fDoMS = b;      }
    virtual void          SetPixelize(Bool_t b)     { fPixelize = b;  }
    virtual void          SetVerbose(Int_t i)       { fVerbose = i;  }
    virtual void          SetMagnet(TPhMagnet *mag) { fMagnet = mag;  }
    virtual void          SetDetector(TPhDetector *det) { fDetector = det;  }

    virtual void          InitLandau();               
    virtual Bool_t        SwimEvent(TPhEvent *event);
    virtual Bool_t        SwimTrack(TPhMCTrack *track);

    virtual void          Draw();
 
private:

    Bool_t       fDoMS;               //  Switch for controlling mult. scattering
    Bool_t       fPixelize;           //  Switch for controlling pixelization of hits
    Int_t        fVerbose;            //  Control debugging output
    TH1F        *fLandau;             //  Histogram for generating landau distribution
    TPhEvent    *fEvent;              //! PHOBOS event object
    TPhMagnet   *fMagnet;             //! PHOBOS Magnet Object
    TPhDetector *fDetector;           //! PHOBOS Magnet Object
    TObjArray   *fSensorList;         //! List of Si sensors relevant for spectrometer tracking 

public:

    ClassDef(TPhSwim,1)  // A PHOBOS Mickey-Mouse GEANT 

};

#endif



